/***************************************************************************
 *            contactlist.h
 *
 *  Thu Jan 27 20:26:16 2005
 *  Copyright  2005  User
 *  Email
 ****************************************************************************/

#ifndef _CONTACTLIST_H
#define _CONTACTLIST_H

#include <gtk/gtk.h>
#include "gtkcellrenderertextpixbuf.h"
#include <iostream>

#include "utilities.h"
#include "settings.h"
#include "gettext.h"


// identifiers for the contact list columns
enum
{
	CL_SELECTED,
	CL_HIGHLIGHTED,
	CL_ORDER,
	CL_ICON,
	CL_ICONVISIBLE,
	CL_ALIAS,
	CL_ALIASCOLOR,
	CL_ALIASSTYLE,
	CL_ENTRYPOINTER,
	CL_ID,
	CL_STATEMASK,
	CL_NUMCOLS
};	




// identifers for the contact list view modes
enum
{
	CVIEW_GROUPS = 1,
	CVIEW_ONEGROUP = 2,
	CVIEW_NOEMPTYGROUPS = 4,
	CVIEW_NOGROUPS = 8,
	CVIEW_HIDEOFFLINE = 16,
	CVIEW_SHOWCHECKBOX = 32
};


enum
{
	CE_UNDEFINED,
	CE_GROUP,
	CE_PROTO,
	CE_USER,
	CE_MULTUSER
};


// sorting identifiers
enum
{
	CSORT_STATUS
};
	



class contactListEntry
{
	public:
		gchar	*id,
				*name;
		gint	sortNum,
				status;
	
		guchar	type;
	
		GtkTreeStore	*store;
		GtkTreeIter		*iter;
		GtkWidget		*tv;
		gint			lastViewMode;
		gboolean		selected,
						highlighted;
	
		gpointer data;
		GList	*children;
	
		contactListEntry	*parent;
	
		contactListEntry(gchar *_id, gchar *_name, gpointer _data);
		virtual ~contactListEntry();
	
	
		contactListEntry *findEntryByID(gchar* _id, guchar toFind);
	
		void 	 addChild(contactListEntry* child);
		gboolean hasChildren();
		GList	 *getChildrenOfType(guchar toFind, GList* startList = NULL);
	
		virtual void showEntries(gint viewMode);
	
		// misc functions
		void moveToTop();
		guint* getCount();
		void updateParents();
		virtual void fillEntry();
		virtual void deleteEntries();
		void setSelected(gboolean select);
		void setHighlighted(gboolean highlight);
};




class contactListUser: public contactListEntry
{
	public:
		contactListUser(gchar *_id, gchar *_name, gchar *_icon, gint _status, gpointer _data, GList *_similies);
		~contactListUser();
	
		// virtual void showEntries(gint viewMode, GtkTreeIter *parent = NULL);
	
		void setFlashing(gchar	*_flashIcon);
		gboolean isFlashing();
		void setStatus(guint _status, gchar *_icon);
		void setStateMask(gulong state);
		gulong getStateMask();
		void setName(gchar* _name);
	
		void blinkEntry();
	
		virtual void fillEntry();
		virtual void showEntries(gint viewMode);	
	
	
	protected:
		GdkPixbuf	*flashIcon,
					*icon;
	
		gchar		*flashIconName,
					*iconName;
	
		gint 		blinkCounter,
					flashCounter;
	
		gulong		stateIconMask;

		GList		*similies;
	
		guint		flashTimeout;
	
		// callbacks
		static gboolean cb_blinkEntry(contactListUser* self);
		static gboolean cb_flashIcon(contactListUser* self);
		
};




class contactListGroup: public contactListEntry
{
	public:
		guint numOnline,
			  numChildren;	
		gboolean lastOpened,
				 isDefault;
	
		contactListGroup(gchar* _id, gchar* _name, gpointer _data);
		~contactListGroup();
	
		virtual void fillEntry();
		virtual void showEntries(gint viewMode);
		virtual void deleteEntries();
		void setDefault(gboolean setting);
	
		void 	 setGroupOpened(gboolean open);
		gboolean getGroupOpened();
	
};




class contactList: public contactListEntry
{
	public:
		GtkWidget	*listWidget;	
	
		contactList(GList *stateIconList);
		~contactList();
	
		contactListEntry* getFocusedEntry();
		contactListEntry* getEntryAtCursor(gint x, gint y);
		void			  clearList();
		void			  setCheckSelectionMode(gboolean select);
	
		// callbacks
		static gint cb_sortingFunction(GtkTreeModel *model,
                    			   	   GtkTreeIter  *a,
                       	   			   GtkTreeIter  *b,
                       	   			   gpointer      userData);
	
	private:
		GtkCellRenderer		*checkboxRenderer;
	
		static void cb_checkboxToggled(GtkCellRenderer* t, gchar* pathString, contactList* self);
};





#endif /* _CONTACTLIST_H */
