/***************************************************************************
 *            imownerdaemon.h
 *
 *  Mon Jan 31 02:45:40 2005
 *  Copyright  2005  User
 *  Email
 ****************************************************************************/

#ifndef _IMOWNERDAEMON_H
#define _IMOWNERDAEMON_H

#include <gtk/gtk.h>
#include <iostream>
#include "imuserdaemon.h"
#include "licq_log.h"


enum contactListType
{
	CLT_NORMAL = 1,
	CLT_IGNORE = 2,
	CLT_VISIBLE = 4
};



typedef struct redirectedEvent
{
	gchar	*newLicqID;
	gint	eventID;
};




typedef struct searchEventInfo: generalEventInfo
{
	gchar		*id,
				*alias,
				*firstName,
				*lastName,
				*email;
	
	gint		more;
	gboolean 	lastEntry;
	gchar		status,
				gender,
				age;
	gboolean 	requireAuthorization;
};


typedef struct logInfo
{
	gchar	*message;
	gushort type;
};

//! the logging daemon
class IMLogDaemon: public CPluginLog
{
	public:
		GList	*loggedInfo;		
	
		IMLogDaemon();
		~IMLogDaemon();
	
		void	setCallback(IMEventCallback _callBack, gpointer _callBackData);
		void	unsetCallback();
		void	clearLog();
	
		IMEventCallback callBack;
		gpointer		callBackData;
	
	private:
		guint 		watchSource;
		GIOChannel	*channel;	
	
		static gboolean cb_pipeCallback(GIOChannel* source, GIOCondition condition, IMLogDaemon* self);	
};



//! the user search manager 
class IMUserSearchManager: public IMEventManager
{
	public:
		IMUserSearchManager(IMUserInfo* _info);
		~IMUserSearchManager();
	
		void searchWhitePages(gchar* firstName, gchar* lastName,
							  gchar* alias, gchar* email,
							  gushort minAge, gushort maxAge,
							  gchar gender, gchar language,
							  gchar* city, gchar* state, gushort country,
							  gchar* companyName, gchar *companyDep,
							  gchar* companyPos,
							  gchar* keyWord,
							  gboolean onlineOnly);
	
		void searchByID(gchar* id);
	
		void setRandomChatGroup(gulong group);
		gulong getRandomChatGroup();
		void searchRandomChatGroup(gulong group);
	
		gboolean finishEvent(ICQEvent* event);
	
	
	private:
		gulong randomChatGroup;
};



// the protocol daemon - manages protocol functions such as loading/unloading
class IMProtoDaemon
{
	public:
		gboolean isProtoPlugin,
				 isEnabled,
				 isLoaded;
		gchar	 *name,
				 *version,
			     *libraryName,
				 *licqName;
	
		gushort	 id;
		gulong	 pluginID,
				 functions;
	
		GList	 *owners;
	
	
	
		IMProtoDaemon(CProtoPlugin* licqProto);
		IMProtoDaemon(CPlugin* licqProto);
		IMProtoDaemon(gchar* _libraryName);
		~IMProtoDaemon();
	
		gboolean loadPlugin();
		gboolean unloadPlugin();
		gboolean registerUser(gchar* userName, gchar* password);
		gboolean verifyRegistration(gchar* verifyStr);
		gboolean addExistingUser(gchar* userName, gchar* password);
		
		void removeOwner(IMOwnerDaemon* toRemove);
		
		void evaluatePluginSignal(CICQSignal* signal);
		void evaluatePluginEvent(ICQEvent* event);
		gboolean startCallback(gint command, gint lastResult, gpointer data);
		
		void setCallback(IMEventCallback _callBack, gpointer _callBackData);
		void unsetCallback();
		
	private:
		IMEventCallback	callBack;
		gpointer		callBackData;
};




// the owner daemon - contains every function that might be neccessary for
//					  protocol events
class IMOwnerDaemon: public IMUserDaemon
{
	public:	
		GList			*users;	
		IMProtoDaemon	*protocol;
		IMGroupManager	*groupMan;
		guchar			lastMask;
	
		IMOwnerDaemon(gchar* _ownerID, guint _pluginID);
		virtual ~IMOwnerDaemon();

		// dispatches owner events (and directs them further if neccessary)
		virtual void dispatchPendingEvents();	
	
		virtual void dispatchInformation(CICQSignal* signal);

		// load all contacts for this protocol
		void loadContacts(guchar mask);
	
		// set the status
		void setStatus(guint status);
	
		// sets the user invisible
		void setInvisible(gboolean setting);
				
		void evaluateLicqSignal(CICQSignal* signal);
		void evaluateLicqEvent(ICQEvent* event);
		IMUserDaemon *getLastEvent(basicEventInfo** toFill, gboolean favourSystemEvents);
	
		IMUserDaemon* findUserByID(gchar* id);	
		IMUserDaemon* addUserTemporally(gchar* idString);
		void		  addUserToList(gchar* idString, gboolean alertUser);
		void		  removeUserFromList(IMUserDaemon* toRemove);
	
	private:
		guchar userMask;
		GList	*redirectedQueue;
	
		IMUserDaemon* addUser(ICQUser* pUser);
		gchar*	findEventReferrer(CUserEvent *event);
};



// returns the already created owners list
GList *IO_getOwnerList();


// returns a list of all plugins
GList *IO_getPluginsList();


// installs the licq pipe and registers the plugin
void IO_installLicqPipe();

// the pipe callback
gboolean IO_pipeCallback(GIOChannel* source, GIOCondition condition);

#endif /* _IMOWNERDAEMON_H */
