/***************************************************************************
 *            contactlist.cpp
 *
 *  Thu Jan 27 20:34:53 2005
 *  Copyright  2005  User
 *  Email
 ****************************************************************************/

#include "contactlist.h"


guint sortNums = 0;


contactListEntry::contactListEntry(gchar *_id, gchar* _name, gpointer _data)
{
	if (_id) id = g_strdup(_id);
		else id = NULL;
	if (_name) name = g_strdup(_name);
		else   name = NULL;
	
	data = _data;
	
	children = NULL;
	parent = NULL;
	store = NULL;
	tv = NULL;
	iter = NULL;
	
	status = 0;
		
	sortNum = 0;
	
	type = CE_UNDEFINED;
	selected = FALSE;	
	highlighted = FALSE;
		
	lastViewMode = 0;
}



contactListEntry::~contactListEntry()
{
	GList	*search,
			*searchs;
	
	searchs = g_list_copy(children);
	
	search = searchs;
	
	// free all children (recursively)
	while(search)
	{
		delete (contactListEntry*)search->data;
		search = search->next;
	}
	
	g_list_free(searchs);
	
	if (id)	g_free(id);
	if (name) g_free(name);	
	
	if (store && iter)
		gtk_tree_store_remove(store, iter);		
	
	if (parent)
	{
		parent->children = g_list_remove(parent->children, this);
		updateParents();
	}
}




contactListEntry *contactListEntry::findEntryByID(gchar* _id, guchar toFind)
{
	GList			 *search;
	contactListEntry *entry;
	
	if (type == toFind && (_id && id) && !strcmp(_id, id))
		return this;
	
	search = children;
	
	while(search)
	{
		if (entry = ((contactListEntry*)search->data)->findEntryByID(_id, toFind))
			return entry;
		
		search = search->next;
	}
	
	return NULL;	
}




void contactListEntry::addChild(contactListEntry *child)
{
	child->parent = this;
	child->store  = store;
	child->tv	  = tv;
	
	children = g_list_append(children, child);

//	if ((lastViewMode & (CVIEW_GROUPS | CVIEW_NOEMPTYGROUPS)) && g_list_length(children)==1)
//		showEntries(lastViewMode);
//	else
		child->showEntries(lastViewMode);
	
	child->updateParents();	
}




gboolean contactListEntry::hasChildren()
{
	return (g_list_length(children)>0);
}


GList* contactListEntry::getChildrenOfType(guchar toFind, GList* startList)
{
	GList	*search;
	
	if (type == toFind)
		startList = g_list_append(startList, this);
	
	search = children;
	
	while(search)
	{
		startList = ((contactListEntry*)search->data)->getChildrenOfType(toFind, startList);
		search = search->next;
	}
	
	return startList;
}



void contactListEntry::showEntries(gint viewMode)
{
	GList		*search;
	GtkTreeIter *parentIter;
	
	if (!store)
		return;
	
	lastViewMode = viewMode;
	
	if (iter)
	{
		g_free(iter);
		iter = NULL;
	}
		
	if (parent)
		parentIter = parent->iter;
	else
		parentIter = NULL;
	
	// there is no parent iter yet? try to create one now
	if ( (viewMode & CVIEW_GROUPS) && parent && !parentIter && parent->type != CE_UNDEFINED)
		parent->showEntries(viewMode);
	else
	{
		if (type != CE_UNDEFINED)
		{
			if ((type == CE_GROUP && 
				(viewMode & CVIEW_GROUPS) && 
				!((viewMode & CVIEW_NOEMPTYGROUPS) && !g_list_length(children))) ||
				(type != CE_GROUP && !((viewMode & CVIEW_HIDEOFFLINE) && status == 0xFFFF))
			   )
			{
				iter = (GtkTreeIter*)g_malloc(sizeof(GtkTreeIter));
				gtk_tree_store_append(store, iter, parentIter);
		
				fillEntry();		
			} 
		}
	
		search = children;
	
		// show children
		while(search)
		{
			((contactListEntry*)search->data)->showEntries(viewMode);
			search = search->next;
		}	
	}
}



void contactListEntry::deleteEntries()
{
	GList	*search;

	// look whether also the parent has to be deleted
	if (parent && (lastViewMode & (CVIEW_GROUPS | CVIEW_NOEMPTYGROUPS)) && !parent->children)
		parent->deleteEntries();	
	else
	{
		search = children;

		while(search)
		{
			((contactListEntry*)search->data)->deleteEntries();
			search = search->next;
		}	

		if (iter && store)
		{
			gtk_tree_store_remove(store, iter);
			g_free(iter);
			iter = NULL;
		}
	}		
}




void contactListEntry::fillEntry()
{
	// nothing to fill for proto type
}




void contactListEntry::moveToTop()
{
	sortNums++;
	sortNum = sortNums;
}




guint* contactListEntry::getCount()
{
	GList	*search;

	guint nums1[2] = {0,0};
	guint* nums2;
	
	if (type!=CE_GROUP)
	{
		nums1[0]++;
		
		if (status!=0xFFFF)
			nums1[1]++;
	}
	
	
	search = children;
	while(search)
	{		
		nums2 = ((contactListEntry*)search->data)->getCount();
		nums1[0]+=nums2[0];
		nums1[1]+=nums2[1];
		
		search = search->next;
	}
	
	
	return nums1;
	
}



void contactListEntry::updateParents()
{
	if (!parent)
		return;
	
	if (parent->type == CE_GROUP)
		parent->fillEntry();
	
	parent->updateParents();	
}



void contactListEntry::setSelected(gboolean select)
{
	GList* search;
	
	selected = select;
	
	fillEntry();
	
	// if a group is selected also select its children
	search = children;
	while(search)
	{
		((contactListEntry*)search->data)->setSelected(select);
		search = search->next;
	}
}




void contactListEntry::setHighlighted(gboolean highlight)
{
	highlighted = highlight;
	
	fillEntry();
}



// ----------------------- contactListUser -------------------------------------

contactListUser::contactListUser(gchar* _id, gchar *_name, gchar *_icon, gint _status, gpointer _data, GList* _similies):
contactListEntry::contactListEntry(_id, _name, _data)
{
	if (_icon)
	{
		icon = u_getPixbufFromStock(_icon, GTK_ICON_SIZE_SMALL_TOOLBAR);
		iconName = _icon;
	}
	else
		icon = NULL;
	
	status = _status;
	flashIcon = NULL;
	flashIconName = NULL;
	flashTimeout = NULL;
	similies = _similies;
	stateIconMask = 0;
	
	blinkCounter = -1;
	
	type = CE_USER;
}



contactListUser::~contactListUser()
{	
	if (flashTimeout)
		g_source_remove(flashTimeout);
	
	if (icon)
		g_object_unref(icon);
	
	if (flashIcon)
		g_object_unref(flashIcon);
}


void contactListUser::fillEntry()
{	
	if (!iter)
		return;
	
	gtk_tree_store_set(store, iter,
					   CL_ORDER, sortNum,
					   CL_ALIAS, name,
					   CL_ICON, icon,
					   CL_ICONVISIBLE, TRUE,	
					   CL_ENTRYPOINTER, this,
					   CL_ID, id,
					   CL_STATEMASK, stateIconMask,
					   CL_SELECTED, selected,
					   CL_HIGHLIGHTED, highlighted,
					   -1);			
	
}



void contactListUser::showEntries(gint viewMode)
{
	GList	*search;
	
	lastViewMode = viewMode;
	
	if (!((viewMode & CVIEW_HIDEOFFLINE) && status == 0xFFFF))
	{
		if (!(viewMode & CVIEW_GROUPS))
		{
			search = similies;
			while(search)
			{
				if (((contactListUser*)search->data)->iter)
					return;
			}
		}
	}
	
	contactListEntry::showEntries(viewMode);
}



void contactListUser::setFlashing(gchar *_flashIcon)
{
	// if we are already flashing this icon, return
	if (flashIconName == _flashIcon)
		return;
	
	if (flashIcon)
		g_object_unref(flashIcon);	
		
	if (_flashIcon)
		flashIcon = u_getPixbufFromStock(_flashIcon, GTK_ICON_SIZE_SMALL_TOOLBAR);
	else
		flashIcon = NULL;
	
	
	if (!flashIconName)
	// nothing was flashed before -> install timeout
		flashTimeout = g_timeout_add(300, (GSourceFunc)this->cb_flashIcon, this);
	
	flashIconName = _flashIcon;
	
	if (!flashIconName && flashTimeout)
	{
		g_source_remove(flashTimeout);
		flashTimeout = 0;
	}
		
	// ...
}


gboolean contactListUser::isFlashing()
{
	return (flashTimeout!=0);
}


void contactListUser::setName(gchar* _name)
{
	g_free(name);
	name = g_strdup(_name);
	
	fillEntry();
}



void contactListUser::setStatus(guint _status, gchar *_icon)
{
	gboolean openGroup;
	
	if (icon)
		g_object_unref(icon);
	
	if (_icon)
		icon = u_getPixbufFromStock(_icon, GTK_ICON_SIZE_SMALL_TOOLBAR);
	else
		icon = NULL;

	iconName = _icon;
	
	openGroup = FALSE;
	
	if ( (lastViewMode & CVIEW_HIDEOFFLINE) && (status != _status) && (status == 0xFFFF || _status == 0xFFFF))
	{		
		status = _status;
		
		if (status == 0xFFFF)
			deleteEntries();
		else
		{
			if (parent->type == CE_GROUP && ((contactListGroup*)parent)->numOnline == 0)
				openGroup = ((contactListGroup*)parent)->getGroupOpened();
			showEntries(lastViewMode);
		}
	} else
	{
		status = _status;		
		fillEntry();
	}
	
	updateParents();
	
	if (openGroup)
		((contactListGroup*)parent)->setGroupOpened(TRUE);
}



void contactListUser::setStateMask(gulong state)
{
	stateIconMask = state;
	
	fillEntry();
}


gulong contactListUser::getStateMask()
{
	return stateIconMask;
}


void contactListUser::blinkEntry()
{
	if (blinkCounter != -1)
		blinkCounter = 0;
	else
		g_timeout_add(300, (GSourceFunc)this->cb_blinkEntry, this);
}




gboolean contactListUser::cb_blinkEntry(contactListUser* self)
{
	gchar	*res;
	
	self->blinkCounter++;

	if ((self->blinkCounter % 2) && self->blinkCounter<15)
		res = g_strdup_printf("<b>%s</b>", self->name);
	else
	{
		if (self->blinkCounter>=15) 
			self->blinkCounter = -1;
		res = g_strdup(self->name);
	}
		
	if (self->iter)
		gtk_tree_store_set(self->store, self->iter,
							   CL_ALIAS, res,
						 	  -1);		
		
	g_free(res);	
	
	return (self->blinkCounter!=-1);
}


gboolean contactListUser::cb_flashIcon(contactListUser* self)
{
	self->flashCounter ++;
	self->flashCounter = self->flashCounter % 2;
	
	if (self->iter)
	{
		if (!self->flashCounter || !self->flashIconName)
		{
			gtk_tree_store_set(self->store, self->iter,
							   CL_ICON, self->icon,
							   -1);
		} else
			gtk_tree_store_set(self->store, self->iter,
							   CL_ICON, self->flashIcon,
							   -1);
	}
	
	
	return TRUE;
}




// ----------------------- contactListGroup ------------------------------------

contactListGroup::contactListGroup(gchar* _id, gchar* _name, gpointer _data):
contactListEntry::contactListEntry(_id, _name, _data)
{
	type = CE_GROUP;
	numOnline = 0;
	numChildren = 0;
	lastOpened = FALSE;
	isDefault = FALSE;
}


contactListGroup::~contactListGroup()
{
	
}



void contactListGroup::fillEntry()
{
	guint *count;
	gchar *toinsert;
	
	if (!iter)
		return;

	count = getCount();
	
	numOnline = count[1];
	numChildren = count[0];	
	
	toinsert = g_strdup_printf("<b>%s</b> <small>(%d/%d)</small>", name, count[1], 
							   count[0]);
	
	gtk_tree_store_set(store, iter, 
					   CL_ALIAS, toinsert,
					   CL_ICONVISIBLE, FALSE,
					   CL_ENTRYPOINTER, this,
					   CL_SELECTED, selected, 
					   CL_HIGHLIGHTED, highlighted,
					   -1);		
		
	g_free(toinsert);	
}


void contactListGroup::showEntries(gint viewMode)
{
	if (iter && tv)
		lastOpened = getGroupOpened();
	
	contactListEntry::showEntries(viewMode);
		
	setGroupOpened(lastOpened);
}



void contactListGroup::deleteEntries()
{
	if (iter && tv)
		lastOpened = getGroupOpened();
	
	contactListEntry::deleteEntries();
}



void contactListGroup::setGroupOpened(gboolean open)
{
	GtkTreePath *path;
	
	lastOpened = open;
	
	if (!iter || !tv)
		return;
	
	path = gtk_tree_model_get_path(GTK_TREE_MODEL(store), iter);
	
	if (open)
		gtk_tree_view_expand_row(GTK_TREE_VIEW(tv), path, FALSE);
	else
		gtk_tree_view_collapse_row(GTK_TREE_VIEW(tv), path);
	
	gtk_tree_path_free(path);
}


gboolean contactListGroup::getGroupOpened()
{
	GtkTreePath *path;
	
	if (!iter || !tv)
		return lastOpened;
	
	if (!(lastViewMode & CVIEW_HIDEOFFLINE && numOnline==0) && numChildren!=0)
	{
		path = gtk_tree_model_get_path(GTK_TREE_MODEL(store), iter);
		lastOpened = gtk_tree_view_row_expanded(GTK_TREE_VIEW(tv), path);
		gtk_tree_path_free(path);
	}
	
	return lastOpened;
}


void contactListGroup::setDefault(gboolean setting)
{
	isDefault = setting;
}



// ------------------------ contactList ----------------------------------------


contactList::contactList(GList *stateIconList): contactListEntry::contactListEntry(NULL, NULL, NULL)
{
	GtkCellRenderer		*renderer;
	GtkTreeViewColumn	*column;
	GdkColor			highlightCol;

	
	children = NULL;
	name = NULL;
	id = NULL;
	
	highlightCol.red = 0xF000;
	highlightCol.green = 0xF000;
	highlightCol.blue = 0xF000;
	
	
	
	// add a store
	store	= gtk_tree_store_new(	CL_NUMCOLS,
									G_TYPE_BOOLEAN,
									G_TYPE_BOOLEAN,
									G_TYPE_INT, 
									GDK_TYPE_PIXBUF, 
									G_TYPE_BOOLEAN,
									G_TYPE_STRING, 
									GDK_TYPE_COLOR, 
									PANGO_TYPE_STYLE,
									G_TYPE_INT,
									G_TYPE_POINTER,
									G_TYPE_ULONG);
	
	
	tv 		= gtk_tree_view_new_with_model(GTK_TREE_MODEL(store));

	column = gtk_tree_view_column_new();
	
	
	checkboxRenderer = gtk_cell_renderer_toggle_new();
	g_object_set(G_OBJECT(checkboxRenderer), "cell-background-gdk", &highlightCol, NULL);
	g_object_set(G_OBJECT(checkboxRenderer), "visible", selected, NULL);
	g_signal_connect(checkboxRenderer, "toggled", G_CALLBACK(this->cb_checkboxToggled), this);	
	
	gtk_tree_view_column_pack_start(column, checkboxRenderer, FALSE);
	
	gtk_tree_view_column_set_attributes(column, checkboxRenderer,
												"active", CL_SELECTED,
												"cell-background-set", CL_HIGHLIGHTED,
												NULL);	
	
	
	// make the icon field
	renderer = gtk_cell_renderer_pixbuf_new();
	g_object_set(G_OBJECT(renderer), "cell-background-gdk", &highlightCol, NULL);
	g_object_set(renderer, "stock-size", GTK_ICON_SIZE_DIALOG, NULL);
	gtk_tree_view_column_pack_start(column, renderer, FALSE);
	
	gtk_tree_view_column_set_attributes(column, renderer,
										"pixbuf", CL_ICON,
										"visible", CL_ICONVISIBLE,
										"cell-background-set", CL_HIGHLIGHTED,
										NULL);
										
	
	// make the alias field
	renderer = gtk_cell_renderer_text_pixbuf_new();
	g_object_set(G_OBJECT(renderer), "cell-background-gdk", &highlightCol, NULL);
	gtk_cell_renderer_text_pixbuf_set_list(GTK_CELL_RENDERER_TEXT_PIXBUF(renderer), stateIconList);
	gtk_cell_renderer_text_pixbuf_set_distance(GTK_CELL_RENDERER_TEXT_PIXBUF(renderer), 10);
	gtk_cell_renderer_text_pixbuf_set_padding(GTK_CELL_RENDERER_TEXT_PIXBUF(renderer), 2);
	gtk_cell_renderer_text_pixbuf_set_use_mask(GTK_CELL_RENDERER_TEXT_PIXBUF(renderer), TRUE);
	
	gtk_tree_view_column_pack_start(column, renderer, TRUE);
	
	gtk_tree_view_column_set_attributes(column, renderer,
										"markup", CL_ALIAS,
										"foreground-gdk", CL_ALIASCOLOR,
										"style", CL_ALIASSTYLE,
										"mask", CL_STATEMASK,
										"cell-background-set", CL_HIGHLIGHTED,
										NULL); 


	// append the column
	gtk_tree_view_append_column(GTK_TREE_VIEW(tv), column);	 	

	gtk_tree_view_set_headers_visible(GTK_TREE_VIEW(tv), FALSE);



	// set sorting function
	gtk_tree_sortable_set_sort_func(GTK_TREE_SORTABLE(store), CSORT_STATUS, 
									cb_sortingFunction,
                                    NULL, NULL);
									
	gtk_tree_sortable_set_sort_column_id(GTK_TREE_SORTABLE(store), CSORT_STATUS, GTK_SORT_ASCENDING);

	
	
	// make a scrolling-window around the tree view
	
	listWidget = gtk_scrolled_window_new(NULL, NULL);
	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(listWidget),
								   GTK_POLICY_AUTOMATIC,
								   GTK_POLICY_AUTOMATIC );
								   
	gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(listWidget), GTK_SHADOW_IN);
	gtk_container_add(GTK_CONTAINER(listWidget), tv);
}




contactList::~contactList()
{
	// FIXME: free store ... 
	
}



void contactList::setCheckSelectionMode(gboolean select)
{
	g_object_set(G_OBJECT(checkboxRenderer), "visible", select, NULL);
	selected = select;
}



gint contactList::cb_sortingFunction(GtkTreeModel *model,
                     			   	 GtkTreeIter  *a,
                       	   			 GtkTreeIter  *b,
                       	   			 gpointer      userData)
{
	contactListEntry	*entry1,
						*entry2;
	contactListGroup	*group1,
						*group2;
	
	
	entry1=NULL;
	entry2=NULL;
	
	if (!a || !b)
		return 0;
	
	gtk_tree_model_get(model, a, CL_ENTRYPOINTER, &entry1, -1);
	gtk_tree_model_get(model, b, CL_ENTRYPOINTER, &entry2, -1);
	
	
	if (entry1->type == CE_GROUP && entry2->type == CE_GROUP)
	{
		group1 = (contactListGroup*)entry1;
		group2 = (contactListGroup*)entry2;
		/*
		if (group2->isDefault || group1->numChildren < group2->numChildren)
			return 1;
		else
		if (group1->isDefault || group1->numChildren > group2->numChildren)
			return -1;
		else */
			return 0;
	}
	
	
	// sort entries by their status
	if (entry1->status < entry2->status)
		return -1;
	else
	if (entry1->status > entry2->status)
		return 1;
	
	// entries with a bigger sortnum come first
	if (entry2->sortNum > entry1->sortNum)
		return 1;
	else
	if (entry2->sortNum < entry1->sortNum)
		return -1;
	
	return 0;
	
}





contactListEntry* contactList::getFocusedEntry()
{
	contactListEntry* 	toreturn;
	GtkTreePath* 		path;
	GtkTreeIter  		iter;
	
	gtk_tree_view_get_cursor (GTK_TREE_VIEW(tv), &path, NULL);

	if (!path) 
		return NULL;
	
	toreturn = NULL;
	
	gtk_tree_model_get_iter(GTK_TREE_MODEL(store), &iter, path);
	gtk_tree_path_free(path);
	gtk_tree_model_get(GTK_TREE_MODEL(store), &iter, 
					   CL_ENTRYPOINTER, &toreturn, -1);
	
	return toreturn;
	
}


contactListEntry* contactList::getEntryAtCursor(gint x, gint y)
{
	contactListEntry* 	toreturn;
	GtkTreePath* 		path;
	GtkTreeIter  		iter;
	
	if (!gtk_tree_view_get_path_at_pos(GTK_TREE_VIEW(tv), x, y, &path, NULL, NULL, NULL))
		return NULL;

	toreturn = NULL;
	
	gtk_tree_model_get_iter(GTK_TREE_MODEL(store), &iter, path);
	gtk_tree_path_free(path);
	gtk_tree_model_get(GTK_TREE_MODEL(store), &iter, 
					   CL_ENTRYPOINTER, &toreturn, -1);
	
	return toreturn;	
}



void contactList::cb_checkboxToggled(GtkCellRenderer* t, gchar* pathString, contactList* self)
{
	GtkTreePath*	path;
	GtkTreeIter		iter;
	contactListEntry *entry;
	
	path = gtk_tree_path_new_from_string(pathString);

	gtk_tree_model_get_iter(GTK_TREE_MODEL(self->store), &iter, path);
	gtk_tree_model_get(GTK_TREE_MODEL(self->store), &iter, CL_ENTRYPOINTER, &entry, -1);

	entry->setSelected(!entry->selected);
	
	gtk_tree_path_free(path);
}
