/***************************************************************************
 *            convwindow.cpp
 *
 *  Fri Jan 21 23:21:09 2005
 *  Copyright  2005  User
 *  Email
 ****************************************************************************/

#include "convwindow.h"

GdkCursor	*cw_cursorHover = NULL;
GdkCursor	*cw_cursorNormal = NULL;

conversationWindow::conversationWindow()
{
	
	if (!cw_cursorHover)
		cw_cursorHover = gdk_cursor_new(GDK_HAND2);

	if (!cw_cursorNormal)
		cw_cursorNormal = gdk_cursor_new(GDK_XTERM);
	
	wType = UE_MESSAGE;
	tabLabelImage = NULL;
	smileysMenu = NULL;
	keyPressTime = 0;
	isURLMode = FALSE;
	hoveredLink = NULL;
	multipleSelection = NULL;
}



conversationWindow::~conversationWindow()
{

}



void conversationWindow::windowActivated()
{	
	if (!hasFocus)
	{		
		basicWindow::windowActivated();
		// window is visible now, so all sunken messages should be seen and can
		// be deleted
		manager->deleteEvent(EO_ALL | EO_NORMAL);
	
		// also set the user name color to black again
		if (tabLabelText)
			gtk_label_set_text(GTK_LABEL(tabLabelText), manager->info->alias);
	}
	// focus the text entry	
	gtk_widget_grab_focus(localView);
}



void conversationWindow::windowDeactivated()
{
	basicWindow::windowDeactivated();
}



GtkWidget* conversationWindow::createWindowContent()
{
	GtkWidget	*rtoolbar,
				*ltoolbar, 
				*buttonbar,
				*localbox,
				*remotebox,
				*entrybuttonbox,
				*vbox;

	
	// create all boxes
	if (!(rtoolbar  = createRemoteToolbar()) ) return NULL;
	if (!(ltoolbar	= createLocalToolbar())	 ) return NULL;
	if (!(remotebox = createRemoteEntryBox())) return NULL;
	if (!(localbox  = createLocalEntryBox()) ) return NULL;	
	if (!(buttonbar	= createButtonbar())	 ) return NULL;	

	
	// create a bar to hold the URL field
	URLbox = gtk_hbox_new(FALSE, 2);	
	
	
	// create the box for the menu + entry
	entrybuttonbox = gtk_vbox_new(FALSE, 1);
	gtk_box_pack_start(GTK_BOX(entrybuttonbox), ltoolbar, FALSE, TRUE, 0);
	gtk_box_pack_start(GTK_BOX(entrybuttonbox), localbox, TRUE, TRUE, 0);
	gtk_box_pack_start(GTK_BOX(entrybuttonbox), URLbox, FALSE, TRUE, 0);
	
		
	
	// create divider
	localRemoteDivider = gtk_vpaned_new();
	
	
	//if (lastPanedPosition)
	//	gtk_paned_set_position(GTK_PANED(textEntryDivider), lastPanedPosition - 1);
	
	gtk_paned_pack1(GTK_PANED(localRemoteDivider),remotebox	,TRUE,TRUE);
	gtk_paned_pack2(GTK_PANED(localRemoteDivider),entrybuttonbox,FALSE,TRUE);
		
	
	
	
	// create the main vertical box
	vbox = gtk_vbox_new(FALSE, 0);	

	gtk_box_pack_start(GTK_BOX(vbox), rtoolbar		    ,FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(vbox), localRemoteDivider,TRUE,TRUE,0);
	gtk_box_pack_start(GTK_BOX(vbox), buttonbar			,FALSE, FALSE, 0);


	// create a hbox around the vbox (so the multiple send window can be added)
	mainHBox = gtk_hbox_new(FALSE, 4);
	gtk_box_pack_start(GTK_BOX(mainHBox), vbox, TRUE, TRUE, 0);
	gtk_container_set_border_width(GTK_CONTAINER(mainHBox), 5);
	
	
	updateLocalColors();

	setWindowSize(600, 500);
	
	addSettingsListener("conversations");
	
	
	return mainHBox;	
}



void conversationWindow::enableURLMode()
{
	GtkWidget	*goToButton;
	GtkTooltips	*tips;
	
	
	if (!isURLMode)
	{
		goToButton = u_createStockImageButton(GTK_STOCK_JUMP_TO);
		URLentry = gtk_entry_new();
	
		gtk_box_pack_start(GTK_BOX(URLbox), goToButton, FALSE, FALSE, 0);
		gtk_box_pack_start(GTK_BOX(URLbox), URLentry, TRUE, TRUE, 0);	
	
		tips = gtk_tooltips_new();
		gtk_tooltips_set_tip(tips, goToButton, 
							 tr("Open the entered URL"), NULL);
	
		gtk_tooltips_enable(tips);	
		
		gtk_widget_show_all(URLbox);
		
		gtk_widget_grab_focus(URLentry);
		
		isURLMode = TRUE;
	}

}


void conversationWindow::disableURLMode()
{
	GList	*children,
			*search;
	
	if (isURLMode)
	{
		children = gtk_container_get_children(GTK_CONTAINER(URLbox));
		
		search = children;
		while(search)
		{
			gtk_container_remove(GTK_CONTAINER(URLbox), GTK_WIDGET(search->data));
			search = search->next;
		}
		
		g_list_free(children);

		isURLMode = FALSE;		
	}
}



GtkWidget* conversationWindow::createContainerTabWidget()
{
	GtkWidget* hbox;
	
	tabLabelImage = gtk_image_new_from_stock(uu_getStatusSymbol(manager->info->status, manager->info->isInvisible), GTK_ICON_SIZE_SMALL_TOOLBAR);
	tabLabelText = gtk_label_new(manager->info->alias);
	
	hbox = gtk_hbox_new(FALSE, 2);
	gtk_box_pack_start(GTK_BOX(hbox), tabLabelImage, FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(hbox), tabLabelText, TRUE, TRUE, 0);
	
	return hbox;
}



void conversationWindow::destroyWindowContent()
{
	sendAni->stop();
	g_source_remove(timeTimeoutID);
	((IMMessageManager*)manager)->sendTypingNotification(0);
	delete manager;
}





void conversationWindow::sendMessage()
{	
	// user clicked when not sending	
	if (GTK_WIDGET_SENSITIVE(localView))
	{
		// get all neccessary information
		toSendText = getTextWithoutSmileys();
		if (isURLMode)
			toSendLink = g_strdup((gchar*)gtk_entry_get_text(GTK_ENTRY(URLentry)));
		else
			toSendLink = NULL;
		
		// don't send the message if it's empty
		if ((!strlen(toSendText) && !isURLMode) || (isURLMode && !strlen(toSendLink)))
		{
			g_free(toSendText);
			if (toSendLink)
				g_free(toSendLink);
			
			return;
		}
		
		sendAni->play();

		gtk_button_set_label(GTK_BUTTON(sendButton), tr("_Cancel"));
		
		gtk_widget_set_sensitive(localView,FALSE);
		gtk_widget_set_sensitive(serverCheckbox, FALSE);
		gtk_widget_set_sensitive(multipleButton, FALSE);

		if (isURLMode)
				gtk_widget_set_sensitive(URLentry, FALSE);		
		
		
		
		// is there a multiple selection? -> start multiple sending of 5 entries
		if (multipleSelection)
		{
			multipleSelection->getSelectedEntries();
			printf("send message multiple1\n");
			sendMessageMultiple(5);
		} else
		// no multiple selection -> send the appropriate event
		{
			if (toSendLink)
			{
				((IMMessageManager*)manager)->sendURL(toSendLink, toSendText);
				g_free(toSendLink);
			} else
				((IMMessageManager*)manager)->sendMessage(toSendText);
			
			g_free(toSendText);
		}
		
		// FIXME: auto send messages urgent ?
		// FIXME: check serverCheckbox state		
	} else
	{
		stopSendMessage();
		// user clicked while sending -> cancel sending	
		manager->cancelInitatedEvent();	
	}
}



void conversationWindow::sendMessageMultiple(gint count)
{	
	GList	*search;
	contactListEntry* entry;
	IMMessageManager* man;
	IMUserDaemon* user;
	
	if (!multipleSelection)
		return;
	
	search = multipleSelection->selectedEntries;
	while(search && count)
	{
		entry = (contactListEntry*)search->data;
		if (entry->selected && !entry->highlighted)
		{
			entry->setHighlighted(TRUE);
			
			user = (IMUserDaemon*)entry->data;
		
			if (user != manager->parentUser)
			{
				man = new IMMessageManager(user->info);
				man->sendViaServer = TRUE;
				man->setCallback((IMEventCallback)this->cb_eventCallback, this);
				user->addManager(man);
			} else
				man = (IMMessageManager*)manager;
		
			if (isURLMode)
				man->sendURL(toSendLink, toSendText);
			else
				man->sendMessage(toSendText);
			
			count--;
		}
		
		search = search->next;
	}
}




void conversationWindow::stopSendMessage()
{
	GList* search;
	contactListEntry *entry;
	
	if (multipleSelection)
	{
		search = multipleSelection->selectedEntries;
		while(search)
		{
			entry = (contactListEntry*)search->data;
			if ((IMUserDaemon*)entry->data != manager->parentUser)
				((IMUserDaemon*)entry->data)->removeManagerByCallback((gpointer)this->cb_eventCallback);
			search = search->next;
		}
		
		if (toSendLink)
			g_free(toSendLink);
		g_free(toSendText);
	}
	
	sendAni->stop();
	gtk_button_set_label(GTK_BUTTON(sendButton), tr("_Send"));
	// gtk_label_set_markup_with_mnemonic(GTK_LABEL(sendLabel), "_Senden");
	gtk_widget_set_sensitive(localView,TRUE);
	gtk_widget_set_sensitive(serverCheckbox, TRUE);
	gtk_widget_set_sensitive(multipleButton, TRUE);
	
	gtk_widget_grab_focus(localView);
	
	if (isURLMode)
		gtk_widget_set_sensitive(URLentry, TRUE);
}




void conversationWindow::finishSendMessage(messageEventInfo* messageEvent, gint result)
{
	GtkTextIter start, end;
	GtkTextBuffer* tb;
	GList* search;
	contactListEntry* entry;
	
	if (result != EVENT_RESULT_SUCCESS && result != EVENT_RESULT_ACKED)
	{
		uu_showSendErrorMessage(tr("Sending of message failed"), tr("The message could not be sent"), result, manager->parentUser->info->status);
		stopSendMessage();
		
		// if there is a multisending running stop it and destroy all managers
		if (multipleSelection)
		{
			search = multipleSelection->selectedEntries;
			while(search)
			{
				entry = (contactListEntry*)search->data;
				((IMUserDaemon*)entry->data)->removeManagerByCallback((gpointer)this->cb_eventCallback);
				search = search->next;
			}
		}
		
		return;
	}
	
	// is there a multi send running? -> send the next 5 events
	if (multipleSelection && !messageEvent->isSplit)
	{
		// delete the entry from the list of selected entries		
		search = multipleSelection->selectedEntries;
		while(search)
		{
			entry = (contactListEntry*)search->data;
			if (!strcmp(entry->id, messageEvent->id))
			{
				entry->setSelected(FALSE);
				entry->setHighlighted(FALSE);
				if ((IMUserDaemon*)entry->data != manager->parentUser)
					((IMUserDaemon*)entry->data)->removeManagerByCallback((gpointer)this->cb_eventCallback);
				multipleSelection->selectedEntries = g_list_remove(multipleSelection->selectedEntries, entry);
				break;
			}
			
			search = search->next;
		}
		
		sendMessageMultiple(1);
	}
	
	// there is no multiple selection or there are no more entries in it
	if (!multipleSelection || (multipleSelection && !multipleSelection->selectedEntries))
	{
		if (result == EVENT_RESULT_SUCCESS || result == EVENT_RESULT_ACKED)
			insertMessage(manager->info->ownerAlias, time(NULL), messageEvent->text, messageEvent->url, MT_OUTGOING);
	
		// clear text buffer
		tb = gtk_text_view_get_buffer(GTK_TEXT_VIEW(localView));	
		gtk_text_buffer_get_bounds(tb, &start, &end);
		gtk_text_buffer_delete(tb, &start, &end);
		gtk_widget_set_sensitive(sendButton, FALSE);
		
		if (hasFocus)
			gtk_widget_grab_focus(localView);	

		keyPressTime = 0;
		((IMMessageManager*)manager)->sendTypingNotification(0);
		
		if (!messageEvent->isSplit)
			stopSendMessage();
	}
}




void conversationWindow::insertMessage(gchar* alias, time_t time, gchar* msg, gchar* url, messageType msgType)
{
	GtkTextIter 	iter;
	GtkTextBuffer* 	tb;
	gchar			ftime[26];
	gchar*			toInsert;
	gchar*			typeStr;
	GtkTextMark*	mark;

	
	switch(msgType)
	{
		case MT_INCOMING: typeStr = "incoming"; break;
		case MT_OUTGOING: typeStr = "outgoing"; break;
		case MT_HINCOMING: typeStr = "history_incoming"; break;
		case MT_HOUTGOING: typeStr = "history_outgoing"; break;
		default: typeStr = "notification"; break;
	}

	
	tb = gtk_text_view_get_buffer(GTK_TEXT_VIEW(remoteView));

	gtk_text_buffer_get_end_iter(tb, &iter);
	
	// create a gap
	gtk_text_buffer_insert_with_tags_by_name(tb, &iter, "\n", -1, "newline", NULL);		

	// create the time stamp
	strftime(ftime, 26, "%H:%M:%S", localtime(&time));
	ftime[25]=0;
	toInsert = g_strdup_printf("(%s)", ftime);
	gtk_text_buffer_insert_with_tags_by_name(tb, &iter, toInsert, -1, "timestamp", typeStr, NULL);
	g_free(toInsert);
		
	// show alias
	if (msgType != MT_NOTIFICATION)
		toInsert = g_strdup_printf(" %s: ", alias);
	else
		toInsert = g_strdup_printf(" %s ", alias);
	
	gtk_text_buffer_insert_with_tags_by_name(tb, &iter, toInsert, -1, "HL", typeStr, NULL);
	g_free(toInsert);
	
	
	// eventually show an url
	if (url)
	{
		gtk_text_buffer_insert_with_tags_by_name(tb, &iter, url, -1, "url", typeStr, "margin", NULL);
		gtk_text_buffer_insert_with_tags_by_name(tb, &iter, "\n", -1, typeStr, NULL);
	}
	
	
	// show text
	toInsert = g_strdup_printf("%s\n", msg);
	if (msgType != MT_NOTIFICATION)
		insertTextInterpreted(toInsert, typeStr, url != NULL, &iter);
	else
		gtk_text_buffer_insert_with_tags_by_name(tb, &iter, toInsert, -1, typeStr, NULL);	
	g_free(toInsert);
		
	// scroll down
	gtk_text_buffer_get_end_iter(tb, &iter);
	mark = gtk_text_buffer_get_mark(tb, "last_pos");
	gtk_text_buffer_move_mark(tb, mark, &iter);
	gtk_text_view_scroll_to_mark(GTK_TEXT_VIEW(remoteView), mark, 0, FALSE, 0, 0);
	
}




// ----------------------- helper functions ------------------------------------


GtkWidget* conversationWindow::createLocalToolbar()
{
	// FIXME: load this from file (make it customizable)
	toolbarItem items[]=
	{		
		{tr_NOOP("Insert Smiley / Emoticon"), ICON_SMILEYS, TOOL_SMILEYS, TRUE},
		{tr_NOOP("Send files"), ICON_FILE, TOOL_SENDFILE, TRUE},
		{tr_NOOP("Send an URL"), ICON_URL, TOOL_SENDLINK, TRUE},
		{tr_NOOP("Start a chat"), ICON_CHAT, TOOL_STARTCHAT, TRUE},
		{NULL, NULL, 0, TRUE},
		{tr_NOOP("font color"), GTK_STOCK_SELECT_FONT, TOOL_FCOLOR, TRUE},
		{tr_NOOP("background color"), GTK_STOCK_SELECT_COLOR, TOOL_BCOLOR, TRUE},
/*		
		{"Benutzer-Informationen", ICON_INFO, TOOL_INFO, FALSE},		
		{"Verlauf", ICON_HISTORY, TOOL_HISTORY, FALSE},			
		{NULL, NULL, 0, FALSE}	*/
	};
	
	
	GtkWidget	*infobox,		
				*toolbar,
				*atool;
	
	gint		i, spacing;
	GtkTooltips *tooltips;
	
	
	
	tooltips = gtk_tooltips_new();
	
	infobox=gtk_vbox_new(FALSE,3);
	
	// make the toolbar	
	toolbar = gtk_hbox_new(FALSE, 2);
//	g_signal_connect(G_OBJECT(toolbar), "button_press_event", G_CALLBACK(cWtoolbarRightClick),this);

	// create items
	
	for (i=0;i<7;i++)
	{
		if (items[i].tip)
		{
			atool = u_createStockImageButton(items[i].stockIcon);
			g_object_set_data(G_OBJECT(atool), "icqnd-menuid", GUINT_TO_POINTER(items[i].id));
			g_signal_connect(G_OBJECT(atool), "clicked", G_CALLBACK(this->cb_toolbarClicked), this);

			gtk_tooltips_set_tip(tooltips, atool, items[i].tip, items[i].tip);
			spacing = 2;
			
		} else
		{
			atool = gtk_vseparator_new();
			spacing = 4;
		}
		
		if (items[i].alignLeft)
			gtk_box_pack_start(GTK_BOX(toolbar), atool, FALSE, FALSE, spacing);
		else
			gtk_box_pack_end(GTK_BOX(toolbar), atool, FALSE, FALSE, spacing);			

	}
	
//	gtk_box_pack_start(GTK_BOX(infobox), gtk_hseparator_new(),FALSE, TRUE, 0);
	gtk_box_pack_start(GTK_BOX(infobox),toolbar, TRUE, TRUE, 0);
//	gtk_box_pack_start(GTK_BOX(infobox), gtk_hseparator_new(),FALSE, TRUE, 0);
	
	gtk_container_set_border_width(GTK_CONTAINER(infobox), 2);
	
	gtk_tooltips_enable(tooltips);

	
	
	
	return infobox;
	
}



GtkWidget* conversationWindow::createRemoteToolbar()
{
	GtkWidget	*historyButton,
				*infoButton,
				*charsetButton,
				*hbox,
				*label1,
				*label2,
				*alignment,
				*toReturn,
				*charsetMenuItem;
	charsetType *charsets;
	GSList		*charsetGroup;
	guint		i;
	gchar		*tmp;

	
	historyButton = u_createStockImageButton(ICON_HISTORY);
	g_object_set_data(G_OBJECT(historyButton), "icqnd-convev", GUINT_TO_POINTER(UE_HISTORY));
	g_signal_connect(historyButton, "clicked", G_CALLBACK(this->cb_userEventButtonClicked), this);

	infoButton = u_createStockImageButton(ICON_INFO);
	g_object_set_data(G_OBJECT(infoButton), "icqnd-convev", GUINT_TO_POINTER(UE_USERINFO));
	g_signal_connect(infoButton, "clicked", G_CALLBACK(this->cb_userEventButtonClicked), this);
	
	encryptButton = u_createStockImageButton(GTK_STOCK_DIALOG_AUTHENTICATION);
	g_object_set_data(G_OBJECT(encryptButton), "icqnd-convev", GUINT_TO_POINTER(UE_ENCRYPTION));
	g_signal_connect(encryptButton, "clicked", G_CALLBACK(this->cb_userEventButtonClicked), this);
	
	charsetButton = u_createStockImageButton(GTK_STOCK_UNINDENT);
	g_signal_connect_swapped(charsetButton, "clicked", G_CALLBACK(this->cb_charsetButtonClicked), this);
	
	//	typingImage = gtk_image_new_from_stock(ICON_TYPING3, (GtkIconSize)getDefaultSettings()->iconSizes.normal);
	typingImage = gtk_image_new_from_stock(GTK_STOCK_BOLD, GTK_ICON_SIZE_SMALL_TOOLBAR);
	
	label1 = gtk_label_new(tr("Status:"));
	label2 = gtk_label_new(tr("Time:"));
	
	statusEntry = gtk_entry_new();
	gtk_widget_set_size_request(statusEntry, 100, -1);
	gtk_editable_set_editable(GTK_EDITABLE(statusEntry), FALSE);
	gtk_entry_set_text(GTK_ENTRY(statusEntry), uu_getStatusDescription(manager->info->status));
	alignment = gtk_alignment_new(0.0f, 0.0f, 1.0f, 1.0f);
	gtk_alignment_set_padding(GTK_ALIGNMENT(alignment), 0, 0, 0, 10);
	gtk_container_add(GTK_CONTAINER(alignment), statusEntry);
	
	timeEntry = gtk_entry_new();
	gtk_widget_set_size_request(timeEntry, 100, -1);
	gtk_editable_set_editable(GTK_EDITABLE(timeEntry), FALSE);
	cb_setUserTime(this);
	timeTimeoutID = g_timeout_add(1000, (GSourceFunc)this->cb_setUserTime, this);
	
	hbox = gtk_hbox_new(FALSE, 2);
	gtk_box_pack_start(GTK_BOX(hbox), label1, FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(hbox), alignment, TRUE, TRUE, 0);
	gtk_box_pack_start(GTK_BOX(hbox), label2, FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(hbox), timeEntry, TRUE, TRUE, 0);
	// gtk_box_pack_start(GTK_BOX(hbox), frame, FALSE, FALSE, 0);	
	
	gtk_box_pack_end(GTK_BOX(hbox), infoButton, FALSE, FALSE, 0);
	gtk_box_pack_end(GTK_BOX(hbox), historyButton, FALSE, FALSE, 0);
	gtk_box_pack_end(GTK_BOX(hbox), encryptButton, FALSE, FALSE, 0);
	gtk_box_pack_end(GTK_BOX(hbox), gtk_vseparator_new(), FALSE, FALSE, 0);
	gtk_box_pack_end(GTK_BOX(hbox), charsetButton, FALSE, FALSE, 0);
	gtk_box_pack_end(GTK_BOX(hbox), gtk_vseparator_new(), FALSE, FALSE, 0);
	
	
	toReturn = gtk_alignment_new(0.0f, 0.0f, 1.0f, 1.0f);
	gtk_alignment_set_padding(GTK_ALIGNMENT(toReturn), 0, 5, 0, 0);
	gtk_container_add(GTK_CONTAINER(toReturn), hbox);
	
	
	// create the charset list
	charsetMenu = gtk_menu_new();
	
	charsets = charset_getCharsetList();
	charsetGroup = NULL;
	i=0;
	
	while(charsets[i].name)
	{
		tmp = g_strdup_printf("%s (%s)", charsets[i].name, charsets[i].id);
		
		charsetMenuItem = gtk_radio_menu_item_new_with_label(charsetGroup, tmp);

		if (!strcmp(charsets[i].id, manager->info->charset))
			gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(charsetMenuItem), TRUE);
		
		g_signal_connect(charsetMenuItem, "activate", G_CALLBACK(this->cb_charsetListItemClicked), this);
		g_object_set_data(G_OBJECT(charsetMenuItem), "icqnd-charset-set", charsets[i].id);
		charsetGroup = gtk_radio_menu_item_get_group(GTK_RADIO_MENU_ITEM(charsetMenuItem));

		gtk_menu_shell_append(GTK_MENU_SHELL(charsetMenu), charsetMenuItem);
		
		g_free(tmp);
		i++;
	}
	
	
	
	return toReturn;
}



GtkWidget* conversationWindow::createRemoteEntryBox()
{
	GtkWidget	*scroll;		
	
	GtkTextBuffer* tb;		
	GtkTextIter    iter;	
	
	// set up cText	
	// (cText from global definition)
	remoteView = gtk_text_view_new();
	gtk_text_view_set_left_margin(GTK_TEXT_VIEW(remoteView), 5);
	gtk_text_view_set_right_margin(GTK_TEXT_VIEW(remoteView), 5);
	
	
	gtk_text_view_set_editable(GTK_TEXT_VIEW(remoteView), FALSE);
	gtk_text_view_set_wrap_mode(GTK_TEXT_VIEW(remoteView), GTK_WRAP_WORD);
	gtk_text_view_set_cursor_visible(GTK_TEXT_VIEW(remoteView), FALSE);

	tb = gtk_text_buffer_new(settings_getSettings()->textTags);
	
	gtk_text_view_set_buffer(GTK_TEXT_VIEW(remoteView), tb);
	
	gtk_text_buffer_get_end_iter(tb, &iter);
	gtk_text_buffer_create_mark(tb, "last_pos", &iter, TRUE);


	g_signal_connect(remoteView, "motion-notify-event", G_CALLBACK(this->cb_remoteViewMouseMoved), this);
	g_signal_connect(remoteView, "visibility-notify-event", G_CALLBACK(this->cb_remoteViewVisibilityChanged), this);

	
	// create the scrolling window 
	scroll = gtk_scrolled_window_new(0, 0);
	gtk_scrolled_window_set_policy(	GTK_SCROLLED_WINDOW(scroll),
				       				GTK_POLICY_AUTOMATIC,
				       				GTK_POLICY_ALWAYS);
	gtk_container_add(GTK_CONTAINER(scroll), remoteView);
	gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(scroll), GTK_SHADOW_IN);

	return scroll;	
}



GtkWidget* conversationWindow::createLocalEntryBox()
{
	GtkWidget	*scroll;
	GtkTextBuffer* buf;			
	
	// set up cEntry
	// (cEntry from global definition)
	localView = gtk_text_view_new();

	gtk_text_view_set_editable(GTK_TEXT_VIEW(localView), TRUE);
  	gtk_text_view_set_wrap_mode(GTK_TEXT_VIEW(localView), GTK_WRAP_WORD);
	g_signal_connect_swapped(G_OBJECT(localView), "key-press-event", G_CALLBACK(this->cb_keyPressed), this);
	g_signal_connect_swapped(G_OBJECT(gtk_text_view_get_buffer(GTK_TEXT_VIEW(localView))), "changed", G_CALLBACK(this->cb_localEntryChanged), this);
	
	buf = gtk_text_view_get_buffer(GTK_TEXT_VIEW(localView));

	gtk_widget_set_size_request(localView, -1, 70);

		
	scroll = gtk_scrolled_window_new(0, 0);
	gtk_scrolled_window_set_policy(	GTK_SCROLLED_WINDOW(scroll),
				       				GTK_POLICY_AUTOMATIC,
				       				GTK_POLICY_AUTOMATIC);
	gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(scroll), GTK_SHADOW_IN);
	gtk_container_add(GTK_CONTAINER(scroll), localView);

	
	
	return scroll;
	
}



GtkWidget* conversationWindow::createButtonbar()
{
	GtkWidget		*buttonbox;		
	GtkWidget		*alignment;

	GtkTooltips		*toolTips;

	
	
	sendAni = new processingAni();
	sendAni->setupAnimation(settings_getSettings()->images.searchingAni, 22, 22);	
	
	// create the send button
	sendButton = gtk_button_new_with_mnemonic(tr("_Send"));
	gtk_widget_set_size_request(sendButton, 80, -1);
	gtk_widget_set_sensitive(sendButton, FALSE);
	g_signal_connect_swapped(sendButton, "clicked",
			   		 		 G_CALLBACK(this->cb_sendButtonClicked), this);
	
	
	// create the multiple-send button
	multipleButton = gtk_button_new();
	gtk_container_add(GTK_CONTAINER(multipleButton), gtk_image_new_from_stock(GTK_STOCK_GO_FORWARD, GTK_ICON_SIZE_BUTTON));
	gtk_container_set_border_width(GTK_CONTAINER(multipleButton), 0);
	g_signal_connect_swapped(multipleButton, "clicked", G_CALLBACK(this->cb_multipleSendButtonClicked), this);


	// create the send-via-server-button
	serverCheckbox = gtk_check_button_new_with_label(tr("Send via server"));	
	g_signal_connect_swapped(serverCheckbox, "clicked", G_CALLBACK(this->cb_sendServerButtonClicked), this);


	// create the send-urgent-button
	urgentCheckbox = gtk_check_button_new_with_label(tr("Send urgent"));
	gtk_widget_set_sensitive(urgentCheckbox, FALSE);


	// add everything into the button box
	buttonbox = gtk_hbox_new(FALSE, 0);
	
	gtk_box_pack_start(GTK_BOX(buttonbox), sendAni->widget, FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(buttonbox), serverCheckbox, FALSE, FALSE, 2);
	gtk_box_pack_start(GTK_BOX(buttonbox), urgentCheckbox, FALSE, FALSE, 4);
	
	gtk_box_pack_end(GTK_BOX(buttonbox), multipleButton, FALSE, FALSE, 0);
	gtk_box_pack_end(GTK_BOX(buttonbox), sendButton, FALSE, FALSE, 0);
	
	
	// create a padding
	alignment = gtk_alignment_new(0.0f, 0.0f, 1.0f, 1.0f);
	gtk_alignment_set_padding(GTK_ALIGNMENT(alignment), 3,0,0,0);
	
	gtk_container_add(GTK_CONTAINER(alignment), buttonbox);




	// create all tooltips
	toolTips = gtk_tooltips_new();
	gtk_tooltips_set_tip(toolTips, sendButton, 
						 tr("Send this message"),
						 NULL);
	gtk_tooltips_set_tip(toolTips, multipleButton,
						 tr("Send to multiple users"),
						 NULL);
	gtk_tooltips_set_tip(toolTips, serverCheckbox, 
						 tr("Sends this message via the server"), 
						 NULL);
	gtk_tooltips_set_tip(toolTips, urgentCheckbox,
						 tr("Marks this message as urgent"),
						 NULL);
	
	gtk_tooltips_enable(toolTips);	




	return alignment;	
}



void conversationWindow::selectColor(gboolean foreground)
{
	GtkWidget	*dialog,
				*useCheckbox;
	GtkColorSelection *colorSel;
	gint		res;
	GdkColor	col;
	gchar		*txt,
				*colToGet;
	bool		isSet;
	settings 	*settings;
	

	settings = settings_getSettings();

	if (foreground)
	{
		txt = tr_NOOP("Select a font color");
		colToGet = "foregroundColor";
	} else
	{
		txt = tr_NOOP("Select a background color");
		colToGet = "backgroundColor";
	}
	
	settings->getProperties("conversations",
							colToGet, &col,
							"colorsSet", &isSet,
							NULL);
		
	dialog = gtk_color_selection_dialog_new(tr(txt));
	gtk_widget_set_no_show_all(GTK_COLOR_SELECTION_DIALOG(dialog)->help_button, TRUE);		

	useCheckbox = gtk_check_button_new_with_label(tr("Use own colors"));
	g_signal_connect(useCheckbox, "toggled", G_CALLBACK(this->cb_colorSelectionCheckboxClicked), dialog);
	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), useCheckbox, FALSE, TRUE, 0);
	gtk_widget_show(useCheckbox);	
	
	colorSel = GTK_COLOR_SELECTION (GTK_COLOR_SELECTION_DIALOG (dialog)->colorsel);
	
	gtk_color_selection_set_current_color(colorSel, &col);
	if (isSet)
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(useCheckbox), TRUE);
	else
	{
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(useCheckbox), FALSE);
		gtk_widget_set_sensitive(GTK_COLOR_SELECTION_DIALOG(dialog)->colorsel, FALSE);
	}
		
	res = gtk_dialog_run(GTK_DIALOG(dialog));

	if (res == GTK_RESPONSE_OK)
	{
		isSet = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(useCheckbox));
		if (isSet)
			gtk_color_selection_get_current_color(colorSel, &col);
		settings->setProperties(TRUE, "conversations",
								colToGet, &col,
								"colorsSet", isSet,
								NULL); 
	}
	
	gtk_widget_destroy(dialog);
}



void conversationWindow::updateLocalColors()
{
	GdkColor	bgColor,
				fgColor;
	gboolean	colorsSet;
	
	settings_getSettings()->getProperties("conversations",
										  "colorsSet", &colorsSet,	
										  "foregroundColor", &fgColor,
										  "backgroundColor", &bgColor,
										  NULL);
	
		
	if (colorsSet)
	{
		gtk_widget_modify_text(localView, GTK_STATE_NORMAL, &fgColor);
		gtk_widget_modify_base(localView, GTK_STATE_NORMAL, &bgColor);
		((IMMessageManager*)manager)->setColorsEnabled(TRUE);
		((IMMessageManager*)manager)->setColors(&fgColor, &bgColor);
	}
	else
	{
		gtk_widget_modify_text(localView, GTK_STATE_NORMAL, NULL); 
		gtk_widget_modify_base(localView, GTK_STATE_NORMAL, NULL);
		((IMMessageManager*)manager)->setColorsEnabled(FALSE);		
	}
}






void conversationWindow::createSmileysMenu()
{
	GtkWidget	*smileyBackground,
				*smileybox,
				*hrow,
				*asmiley,
				*smileyimg,
				*frame;
	guint i;
	settings* config;
	gint 		width, 
				height, 
				posX, 
				posY, 
				maxX, 
				maxY;
	GdkScreen* scr;
	
	
	config = settings_getSettings();
	
	
	// *** create the window ***
	smileysMenu = gtk_window_new(GTK_WINDOW_POPUP);
	g_signal_connect(smileysMenu, "key-release-event", 
					 G_CALLBACK(this->cb_smileysMenuKeyReleased), &smileysMenu);
	g_signal_connect(smileysMenu, "button-release-event", 
					 G_CALLBACK(gtk_widget_destroy), this);
	// don't show a title
	gtk_window_set_decorated(GTK_WINDOW(smileysMenu), FALSE);
	// don't show window in taskbar
	gtk_window_set_skip_taskbar_hint(GTK_WINDOW(smileysMenu), TRUE);
	
	// don't show window in pager
	gtk_window_set_skip_pager_hint(GTK_WINDOW(smileysMenu), TRUE); 

	
	// create a frame around the window
	frame = gtk_frame_new(0);
	gtk_frame_set_shadow_type(GTK_FRAME(frame), GTK_SHADOW_OUT);
	gtk_container_add (GTK_CONTAINER (smileysMenu),
                      frame);

	


	// create the smiley buttons
	smileyBackground = gtk_event_box_new();
	u_setWidgetBackground(smileyBackground,"#ffffff");


	smileybox = gtk_vbox_new(FALSE,0);
	
		
	for (i=0;i<28;i++)
	{
		if (!(i % 4))
		{
			hrow=gtk_hbox_new(FALSE,0);

		}
		
		asmiley=gtk_button_new();
		// u_setWidgetBackground(asmiley,"#ffffff");
		gtk_container_set_border_width(GTK_CONTAINER(asmiley), 0);

		smileyimg=gtk_image_new_from_pixbuf(config->images.smileys[i].icon);
		gtk_container_add(GTK_CONTAINER(asmiley),smileyimg);
		gtk_box_pack_start(GTK_BOX(hrow),asmiley,FALSE,TRUE,0);
		gtk_button_set_relief(GTK_BUTTON(asmiley),GTK_RELIEF_NONE);
		
		g_object_set_data(G_OBJECT(asmiley), "ICQmenuID", GUINT_TO_POINTER(i));
		g_signal_connect(asmiley,"clicked", G_CALLBACK(this->cb_smileyClicked), this);

		if (i%4==3)
			gtk_box_pack_start(GTK_BOX(smileybox),hrow,FALSE,TRUE,0);
	}
			
	
	gtk_container_add(GTK_CONTAINER(smileyBackground),smileybox);	
	gtk_container_add(GTK_CONTAINER(frame),smileyBackground);
	 
	 
	// position the window
	// FIXME: we don't need the mouse position but the position of the pressed button
	scr = gdk_screen_get_default();
	gdk_display_get_pointer(gdk_display_get_default(), &scr, &posX, &posY, NULL);

	// FIXME: returns screen size, not the actual max-window-size
	//		  e.g. gnome-panel shrinkes it
	maxX = gdk_screen_get_width(scr);
	maxY = gdk_screen_get_height(scr);	

	
	gtk_window_get_size(GTK_WINDOW(smileysMenu), &width, &height);
	
	// set position so window doesn't get out of screen
	if (posX + width > maxX) 
		posX-= width;
	
	if (posY + height > maxY) 
		posY-= height;
		
	gtk_window_move(GTK_WINDOW(smileysMenu),posX,posY);

	
	

	// show the window
	gtk_widget_show_all(smileysMenu);	
	
	
	
	// grab mouse and keyboard events
	if (gdk_pointer_grab(smileysMenu->window, TRUE,
						 (GdkEventMask)(GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE_MASK |
						 GDK_ENTER_NOTIFY_MASK | GDK_LEAVE_NOTIFY_MASK |
						 GDK_POINTER_MOTION_MASK),
						 NULL, NULL,
						 gtk_get_current_event_time())==0)
	{
		gdk_keyboard_grab(smileysMenu->window, TRUE,
						  gtk_get_current_event_time());
	}
	gtk_grab_add(smileysMenu);
}




void conversationWindow::showOldEntries(gint toSkip)
{
	IMHistoryManager *hman;
	GList			 *entries,
					 *search;
	IMHistoryEntry	 *hentry;
	GtkTextIter		 iter;
	GtkTextMark		 *mark;


	hman = new IMHistoryManager(manager->info);
	
	manager->parentUser->addManager(hman);
	
	// only show these events
	hman->readMask = HRT_MESSAGE | HRT_URL | HRT_INCOMING | HRT_OUTGOING;
	
	
	if (hman->loadHistory())
	{
		hman->setReadReverse(TRUE);
		hman->resetHistoryReading();
		hman->moveHistoryPosition(-toSkip);
		
		
		if (hman->historyLength() - toSkip >= 5)		
			entries = hman->getHistoryEntries(5, TRUE);
		else
		{
			if (hman->historyLength()-toSkip>0)
				entries = hman->getHistoryEntries(hman->historyLength()-toSkip, TRUE);
			else
				entries = NULL;
		}

		
		search = g_list_last(entries);

		while(search)
		{		
			hentry = (IMHistoryEntry*)search->data;

			if (hentry->incoming)
				insertMessage(manager->info->alias, hentry->time, hentry->text, NULL, MT_HINCOMING);
			else
				insertMessage(manager->info->ownerAlias, hentry->time, hentry->text, NULL, MT_HOUTGOING);

			search = search->prev;
		}
	}
	
	delete hman;	

	// scroll down
	gtk_text_buffer_get_end_iter(gtk_text_view_get_buffer(GTK_TEXT_VIEW(remoteView)), &iter);
	mark = gtk_text_buffer_get_mark(gtk_text_view_get_buffer(GTK_TEXT_VIEW(remoteView)), "last_pos");
	gtk_text_buffer_move_mark(gtk_text_view_get_buffer(GTK_TEXT_VIEW(remoteView)), mark, &iter);
	gtk_text_view_scroll_to_mark(GTK_TEXT_VIEW(remoteView), mark, 0, FALSE, 0, 0);	
}




gboolean conversationWindow::eventCallback(gint command, gint lastResult, gpointer info)
{
	messageEventInfo *msg;
	gchar			 *tmp;

	
	switch(command)
	{
		// a new message event was received
		case UC_EVENT_RECEIVED:
			msg = (messageEventInfo*)info;
		
			insertMessage(manager->info->alias, msg->basic->time, msg->text, msg->url, MT_INCOMING);	
		
			// clear event if current window is active. if this is not the case
			// just drown them (maybe the user doesn't realize that there is an
			// event on another window and closes the program without having read
			// the message. this way the message won't be deleted from the main
			// event buffer but won't be printed again
				
			if (hasFocus)
				manager->deleteEvent(EO_FIRST | EO_NORMAL); 
			else 
			{
				// msg->basic->sunken = TRUE;
				
				if (tabLabelText)
				{
					// set the text color of the alias tab to blue (to inform the user that a message was received)
					tmp = g_strdup_printf("<span foreground=\"blue\">%s</span>", manager->info->alias);
					gtk_label_set_markup(GTK_LABEL(tabLabelText), tmp);
					g_free(tmp);
				}
			}
			break;
		
		case UC_EVENT_FINISHED:
			finishSendMessage((messageEventInfo*)info, lastResult);
			break;
		
		
			
		// the manager was installed -> add the history entries (that won't be shown later)
		case EC_INSTALLED:
			showOldEntries(lastResult);	
			oldStatus = manager->info->status;
		// the connection state has changed - update the label
		case UC_SENDSERVER:
			gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(serverCheckbox), manager->sendViaServer);
			break;	
		
		
		case UC_UPDATEUSER:
			switch(lastResult)
			{
				/*
				case USER_UPDATE_TYPING:
					switch(manager->info->typing)
					{
						case 0:
							gtk_image_set_from_stock(GTK_IMAGE(typingImage), ICON_TYPING3, (GtkIconSize)getDefaultSettings()->iconSizes.normal);
							break;
						
						case 1:
							gtk_image_set_from_stock(GTK_IMAGE(typingImage), ICON_TYPING2, (GtkIconSize)getDefaultSettings()->iconSizes.normal);
							break;
						
						case 2:
							gtk_image_set_from_stock(GTK_IMAGE(typingImage), ICON_TYPING1, (GtkIconSize)getDefaultSettings()->iconSizes.normal);
							break;
					}
					break; */
				
				default:				
					if (oldStatus != manager->info->status)
					{
						if (manager->info->status == BUDDY_STATUS_OFFLINE)
							insertMessage(manager->info->alias, time(NULL), "has logged off!", NULL, MT_NOTIFICATION);
						else
						if (manager->info->status != BUDDY_STATUS_OFFLINE && oldStatus == BUDDY_STATUS_OFFLINE)
							insertMessage(manager->info->alias, time(NULL), "is online now!", NULL, MT_NOTIFICATION);

						gtk_entry_set_text(GTK_ENTRY(statusEntry), uu_getStatusDescription(manager->info->status));

						if (tabLabelImage)
							gtk_image_set_from_stock(GTK_IMAGE(tabLabelImage), uu_getStatusSymbol(manager->info->status, manager->info->isInvisible), GTK_ICON_SIZE_SMALL_TOOLBAR);						
				
						oldStatus = manager->info->status;
					}
					break;
			}
			break;
		
		
		default:
			printf("conversationWindow::eventCallback: Unknown event (%d)!\n", command);
	}
	
	return TRUE;
}



void conversationWindow::settingsChangedCallback(gchar* sname, GList *values)
{
	GList 			 *search;
	settingsProperty *prop;
	gboolean		 updateColors;
	
	if (strcmp(sname, "conversations")!=0)
		return;
	
	updateColors = FALSE;
	
	search = values;
	
	while(search)
	{	
		prop = (settingsProperty*)search->data;
		
		if (!updateColors &&
			(!strcmp(prop->name, "colorsSet") ||
			 !strcmp(prop->name, "foregroundColor") ||
			 !strcmp(prop->name, "backgroundColor")))
			updateColors = TRUE;
		
		search = search->next;		
	}
	
	
	if (updateColors)
		updateLocalColors();
}




gchar* conversationWindow::getTextWithoutSmileys()
{
	GtkTextIter 		startIter, 
						endIter;
	gchar				*raw;
	GString				*toReturn;
	gunichar			c;
	GtkTextChildAnchor	*anchor;
	GtkTextBuffer		*buf;
	

	buf = gtk_text_view_get_buffer(GTK_TEXT_VIEW(localView));
	
	gtk_text_buffer_get_bounds(buf, &startIter, &endIter);
	
	toReturn = g_string_new("");
	
	while(!gtk_text_iter_equal(&startIter, &endIter))
	{
		c = gtk_text_iter_get_char(&startIter);
		
		// this could be an image
		if (c == 0xFFFC && (anchor = gtk_text_iter_get_child_anchor(&startIter)))
		{
			raw = (gchar*)g_object_get_data(G_OBJECT(anchor), "ICQsmiley");
			g_string_append(toReturn, raw);
		} else
			g_string_append_unichar(toReturn, c);
		
		gtk_text_iter_forward_char(&startIter);
	}
	
	
	return g_string_free(toReturn, FALSE);
}



void conversationWindow::insertTextInterpreted(gchar* toAppend, gchar* tagName, gboolean createMargin, GtkTextIter* pos)
{	
	smileysNode *child;
	GList	*search;
	guint offset, i, l;
	gint	linkEnd, linkStart, i1;
	settings	*config;
	GString		*insertSlice;
	GdkColor	*col;
	GtkTextChildAnchor *anchor;
	GtkTextBuffer	*buf;
	GtkWidget		*img;
	GtkTextTag		*tag,
					*tagMargin,
					*tagURL,
					*tagURLNew;
	gchar			*link;
	
	
	
	insertSlice = g_string_new("");
	config = settings_getSettings();
	buf = gtk_text_view_get_buffer(GTK_TEXT_VIEW(remoteView));
	
	tag = gtk_text_tag_table_lookup(gtk_text_buffer_get_tag_table(buf), tagName);
	tagURL = gtk_text_tag_table_lookup(gtk_text_buffer_get_tag_table(buf), "url");
	g_object_get(G_OBJECT(tagURL), "foreground-gdk", &col, NULL);

	
	if (createMargin)
		tagMargin = gtk_text_tag_table_lookup(gtk_text_buffer_get_tag_table(buf), "margin");
	else
		tagMargin = NULL;
	
	l=0;
	
	linkEnd = -1;
	
	// go through the string
	for (i=0; i<strlen(toAppend); i++)
	{	
		
		// look for URLs
		// --------------
		
		// www/ftp-URLS
		if (((toAppend[i] == 'w' || toAppend[i] == 'W') && 
			toAppend[i+1] == toAppend[i] && 
			toAppend[i+2] == toAppend[i] &&
			toAppend[i+3] == '.') ||
		
		   ((toAppend[i] == 'f' || toAppend[i] == 'F') &&
			(toAppend[i+1] == 't' || toAppend[i+1] == 'T') &&
			(toAppend[i+2] == 'p' || toAppend[i+2] == 'P'))		
		)
		{			
			for (i1=i+3; i1<strlen(toAppend); i1++)
				if (toAppend[i1] == ' ' || toAppend[i1] == '\n')
				{
					linkEnd = i1;
					break;
				}
				
			if (linkEnd == -1)
				linkEnd = strlen(toAppend)-1;
			
			linkStart = i;
		} else
		
		// xxxx://-URLS
		if (toAppend[i] == ':' && toAppend[i+1] == '/' && toAppend[i+2] == '/')
		{
			// go back to the beginning of the protocol identifier
			// use insertSlice and not toAppend here. Text which is in
			// insertslice wasn't shown yet. This is not perfect and will
			// for example misunderstand something like htt:)p://...
			
			linkStart = i;
			
			
			for(i1=strlen(insertSlice->str)-1; i1>=-1; i1--)
			{
				if (i1 == -1 ||
					insertSlice->str[i1] == ' ' || 
					insertSlice->str[i1] == '\n' ||
					insertSlice->str[i1] == '.' ||
					insertSlice->str[i1] == '!' ||
					insertSlice->str[i1] == '?' ||
					insertSlice->str[i1] == '-' ||
					insertSlice->str[i1] == ',')
				{
					linkStart = i - (strlen(insertSlice->str)-i1)+1;
					// cut the insert slice so things won't be shown again
					g_string_set_size(insertSlice, i1+1);
					break;
				}
			} 
				
			// find the end
			for (i1=i; i1<strlen(toAppend); i1++)
				if (toAppend[i1] == ' ' || toAppend[i1] == '\n')
				{
					linkEnd = i1;
					break;
				}
				
			if (linkEnd == -1)
				linkEnd = strlen(toAppend)-1;
		}
		
		
		if (linkEnd != -1)
		{
			
			// insert the old slice content
			// gtk_text_buffer_get_end_iter(buf, &iter);
			gtk_text_buffer_insert_with_tags(buf, pos, insertSlice->str, strlen(insertSlice->str), tag, tagMargin, NULL);
			g_string_set_size(insertSlice, 0);

			// create a new tag and add the appropriate link
			tagURLNew = gtk_text_tag_new(NULL);
			g_object_set(G_OBJECT(tagURLNew), "foreground-gdk", col,
											  "underline", PANGO_UNDERLINE_SINGLE,
											  NULL);
			gtk_text_tag_table_add(gtk_text_buffer_get_tag_table(buf), tagURLNew);
			
			link = (gchar*)g_malloc0((linkEnd - linkStart)*sizeof(gchar)+1);
			memcpy(link, toAppend + linkStart, (linkEnd - linkStart)*sizeof(gchar));
			g_object_set_data_full(G_OBJECT(tagURLNew), "ICQLink", link, g_free);
			
			// insert the link
			// gtk_text_buffer_get_end_iter(buf, &iter);
			gtk_text_buffer_insert_with_tags(buf, pos, toAppend + linkStart, linkEnd - linkStart, tag, tagURLNew, tagMargin, NULL);

			i+=(linkEnd - linkStart) - (i - linkStart);			
			linkEnd = -1;
		}
		
		
		
		// look for smileys
		// ----------------
		offset = 0;
		// start at the first level of the smiley tree
		search = config->smileyTree->children;
		while(search)
		{
			l++;
			child = (smileysNode*)search->data;
			
			// does the character match?
			if (toAppend[offset+i] == child->character)
			{
				// if the child itself has children the smiley must be longer
				if (child->children && strlen(toAppend)>offset+i)
				{
					// go to the next level
					search = child->children;
					offset++;
					
					// restart the search
					continue;
				} else
				{
					// no children -> add the smiley
					if (strlen(insertSlice->str)>0)
					{
						//gtk_text_buffer_get_end_iter(buf, &iter);
						gtk_text_buffer_insert_with_tags(buf, pos, insertSlice->str, strlen(insertSlice->str), tag, tagMargin, NULL);
						
						g_string_set_size(insertSlice, 0);
					}
					
					img = gtk_image_new_from_pixbuf(config->images.smileys[child->smileysIndex].icon);
					gtk_widget_show(img);
			
					// gtk_text_buffer_get_end_iter(buf, &iter);
					anchor = gtk_text_buffer_create_child_anchor(buf, pos);
					gtk_text_view_add_child_at_anchor(GTK_TEXT_VIEW(remoteView), img, anchor);
					i+=offset;
					break;
				}
			}
			search = search->next;
		}
		
		if (!search)
			g_string_append_c(insertSlice, toAppend[i]);
	}


	// insert the rest
	if (strlen(insertSlice->str)>0)
	{
		// gtk_text_buffer_get_end_iter(buf, &iter);
		gtk_text_buffer_insert_with_tags(buf, pos, insertSlice->str, strlen(insertSlice->str), tag, tagMargin, NULL);
	}	
	
	g_string_free(insertSlice, TRUE);
}




void conversationWindow::checkForURLCursor(gint x, gint y)
{
	GSList			*tags,
					*search;
	GtkTextBuffer	*buf;
	GtkTextIter		iter;
	gint			bX,
					bY;
	GtkTextTag		*curTag;
	gchar			*curLink;


 	gtk_text_view_window_to_buffer_coords(GTK_TEXT_VIEW (remoteView), 
                                          GTK_TEXT_WINDOW_WIDGET,
                                          x, y, &bX, &bY);
		
	buf = gtk_text_view_get_buffer(GTK_TEXT_VIEW(remoteView));
	
	gtk_text_view_get_iter_at_location(GTK_TEXT_VIEW(remoteView), &iter, bX, bY);
	
	
	tags = gtk_text_iter_get_tags(&iter);
	search = tags;
	while(search)
	{
		curTag = (GtkTextTag*)search->data;
		
		curLink = (gchar*)g_object_get_data(G_OBJECT(curTag), "ICQLink");
		
		if (curLink)
		{
			if (!hoveredLink)
			{
				gdk_window_set_cursor(gtk_text_view_get_window(GTK_TEXT_VIEW(remoteView), GTK_TEXT_WINDOW_TEXT), cw_cursorHover);
				hoveredLink = curLink;
			}
			
			break;
		}
		
		search = search->next;
	}
	
	if (!search && hoveredLink != NULL)
	{
		gdk_window_set_cursor(gtk_text_view_get_window(GTK_TEXT_VIEW(remoteView), GTK_TEXT_WINDOW_TEXT), cw_cursorNormal);
		hoveredLink = NULL;
	}
	
	g_slist_free(tags);
}



void conversationWindow::showMultipleSelectionList()
{
	if (multipleSelection)
		return;
	
	multipleSelection = new contactsSelection(manager->parentUser);
	gtk_widget_set_size_request(multipleSelection->widget, 150, -1);
	allocateExtraSpace(150+4);
	gtk_box_pack_start(GTK_BOX(mainHBox), multipleSelection->widget, FALSE, TRUE, 0);
	
	gtk_widget_show_all(multipleSelection->widget);
	
	u_setStockButtonImage(multipleButton, GTK_STOCK_GO_BACK);
	gtk_button_set_use_stock(GTK_BUTTON(multipleButton), TRUE);
}



void conversationWindow::hideMultipleSelectionList()
{
	if (!multipleSelection)
		return;
	
	gtk_container_remove(GTK_CONTAINER(mainHBox), multipleSelection->widget);
	deallocateExtraSpace();
	delete multipleSelection;
	multipleSelection = NULL;
	
	u_setStockButtonImage(multipleButton, GTK_STOCK_GO_FORWARD);
}





// =============================================================================

void conversationWindow::cb_sendButtonClicked(conversationWindow *cw)
{
	cw->sendMessage();
}



void conversationWindow::cb_sendServerButtonClicked(conversationWindow *cw)
{
	cw->manager->setSendViaServer(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(cw->serverCheckbox)));
}



void conversationWindow::cb_multipleSendButtonClicked(conversationWindow* self)
{
	if (self->multipleSelection)
		self->hideMultipleSelectionList();
	else
		self->showMultipleSelectionList(); 
}



void conversationWindow::cb_toolbarClicked(GtkWidget* clickedWidget, conversationWindow *cw)
{
	guint toCall;
	
	toCall = GPOINTER_TO_UINT(g_object_get_data(G_OBJECT(clickedWidget), "icqnd-menuid"));

	switch(toCall)
	{
		case TOOL_SENDFILE:
			cw->manager->callUser(UE_FILETRANSFERREQUEST, NULL);
			break;
		
		case TOOL_SENDLINK:
			cw->manager->callUser(UE_URL, NULL);
			break;
		
		case TOOL_STARTCHAT:
			cw->manager->callUser(UE_CHATREQUEST, NULL);
			break;
		
		case TOOL_FCOLOR:
		case TOOL_BCOLOR:
			cw->selectColor(toCall == TOOL_FCOLOR);
			break;
		
		case TOOL_SMILEYS:
			cw->createSmileysMenu();
			break;
	}		
}


void conversationWindow::cb_userEventButtonClicked(GtkWidget* w, conversationWindow *cw)
{
	guint toCall;
	
	toCall = GPOINTER_TO_UINT(g_object_get_data(G_OBJECT(w), "icqnd-convev"));
	cw->manager->callUser(toCall, NULL);
}




gboolean conversationWindow::cb_setUserTime(conversationWindow *cw)
{
	gchar* tmp;
	time_t curTime;
	struct tm *tmTime;
	
	curTime = time(NULL) + cw->manager->info->localtime;
	tmTime = localtime(&curTime);
	
	
	tmp = g_strdup_printf("%02d:%02d:%02d",
						  tmTime->tm_hour, 
						  tmTime->tm_min, 
						  tmTime->tm_sec);
	gtk_entry_set_text(GTK_ENTRY(cw->timeEntry), tmp);
	g_free(tmp);
	
	if (cw->keyPressTime>0 && time(NULL) - cw->keyPressTime == 3)
		((IMMessageManager*)cw->manager)->sendTypingNotification(1);

	return TRUE;	
}



gboolean conversationWindow::cb_keyPressed(conversationWindow *cw, GdkEventKey *event)
{
	if (!event)
		return FALSE;
	
	if (event->keyval == GDK_Return && (event->state & GDK_CONTROL_MASK || event->state & GDK_MOD1_MASK))
		cw->sendMessage();	
	
	return FALSE;
}



void conversationWindow::cb_localEntryChanged(conversationWindow *self)
{
	GtkTextIter	start, end;
	
	gtk_text_buffer_get_bounds(gtk_text_view_get_buffer(GTK_TEXT_VIEW(self->localView)), &start, &end);
	gtk_widget_set_sensitive(self->sendButton, !gtk_text_iter_equal(&start, &end));
	
	if (!self->keyPressTime || time(NULL) - self->keyPressTime>=3)
		((IMMessageManager*)self->manager)->sendTypingNotification(2);
			
	self->keyPressTime = time(NULL);
}



void conversationWindow::cb_colorSelectionCheckboxClicked(GtkWidget* w, GtkWidget* d)
{
	if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(w)))
		gtk_widget_set_sensitive(GTK_COLOR_SELECTION_DIALOG(d)->colorsel, TRUE);
	else
		gtk_widget_set_sensitive(GTK_COLOR_SELECTION_DIALOG(d)->colorsel, FALSE);
}



gboolean conversationWindow::cb_smileysMenuKeyReleased(GtkWidget *w, GdkEventKey *event, GtkWidget **toDestroy)
{
	if (event->keyval != GDK_Left &&
		event->keyval != GDK_Right &&
		event->keyval != GDK_Up &&
		event->keyval != GDK_Down &&
		event->keyval != GDK_Tab)
	{
		gtk_widget_destroy(*toDestroy);
		*toDestroy = NULL;
	}
	
	return FALSE;
}



void conversationWindow::cb_smileyClicked(GtkWidget* w, conversationWindow *cw)
{
	GtkTextChildAnchor	*anchor;
	GtkTextBuffer		*buf;
	GtkTextIter			iter;
	GtkWidget			*img;
	guint				num;
	settings			*config;
	
	
	num = GPOINTER_TO_UINT(g_object_get_data(G_OBJECT(w), "ICQmenuID"));
	
	
	config = settings_getSettings();
	buf = gtk_text_view_get_buffer(GTK_TEXT_VIEW(cw->localView));
	
	gtk_text_buffer_get_iter_at_mark(buf, &iter,
									 gtk_text_buffer_get_mark(buf, "insert"));
	
	anchor = gtk_text_buffer_create_child_anchor(buf, &iter);
	g_object_set_data_full(G_OBJECT(anchor), "ICQsmiley", g_strdup(config->images.smileys[num].token), g_free);
	
	img = gtk_image_new_from_pixbuf(config->images.smileys[num].icon);
	gtk_widget_show(img);
	
	gtk_text_view_add_child_at_anchor(GTK_TEXT_VIEW(cw->localView), img, anchor);
	
	
	if (cw->smileysMenu)
		gtk_widget_destroy(cw->smileysMenu);
	
	cb_keyPressed(cw, NULL);
}



gboolean conversationWindow::cb_remoteViewMouseMoved(GtkWidget* v, GdkEventMotion* event, conversationWindow *self)
{
	self->checkForURLCursor((gint)event->x, (gint)event->y);
	
	gdk_window_get_pointer(self->remoteView->window, NULL, NULL, NULL);
	
	return FALSE;
}



gboolean conversationWindow::cb_remoteViewVisibilityChanged(GtkWidget* v, GdkEventVisibility* event, conversationWindow *self)
{
	gint cX, cY;
		
 	gdk_window_get_pointer (self->remoteView->window, &cX, &cY, NULL);
	self->checkForURLCursor(cX, cY);
	
	return FALSE;
}



void conversationWindow::cb_charsetButtonClicked(conversationWindow *self)
{
	gtk_widget_show_all(self->charsetMenu);
	gtk_menu_popup(GTK_MENU(self->charsetMenu), NULL, NULL, NULL, NULL, 1, gtk_get_current_event_time());
}



void conversationWindow::cb_charsetListItemClicked(GtkWidget *w, conversationWindow *self)
{
	gchar	*newCharset;
	
	if (gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(w)))
	{
		newCharset = (gchar*)g_object_get_data(G_OBJECT(w), "icqnd-charset-set");
		self->manager->parentUser->setCharset(newCharset, TRUE);
	}
}
