/***************************************************************************
 *            imownerdaemon.cpp
 *
 *  Mon Jan 31 02:47:17 2005
 *  Copyright  2005  User
 *  Email
 ****************************************************************************/

#include "imownerdaemon.h"


// omfg - a global variable!
GList 			*ownersList = NULL;
GList			*pluginsList = NULL;



IMLogDaemon::IMLogDaemon()
{
	loggedInfo = NULL;
	gLog.AddService(new CLogService_Plugin(this, L_MOST));
	
	channel = g_io_channel_unix_new(Pipe());
	watchSource = g_io_add_watch(channel, G_IO_IN, (GIOFunc)this->cb_pipeCallback, this);	

	callBack = NULL;	
}



IMLogDaemon::~IMLogDaemon()
{
	clearLog();
	
	if (watchSource != 0)
	{
		g_source_remove(watchSource);
		g_io_channel_shutdown(channel, TRUE, NULL);
	}	
}


void IMLogDaemon::setCallback(IMEventCallback _callBack, gpointer _callBackData)
{
	callBack = _callBack;
	callBackData = _callBackData;
}



void IMLogDaemon::unsetCallback()
{
	callBack = NULL;
}



gboolean IMLogDaemon::cb_pipeCallback(GIOChannel* source, GIOCondition condition, IMLogDaemon* self)
{
	gchar 	buffer[4];
	gsize	bytesRead;
	logInfo	*newInfo;
	
	if ( (g_io_channel_read_chars(source, buffer, 1, &bytesRead, NULL) == G_IO_STATUS_ERROR) ||
		 (!bytesRead) )
	{
		printf("IMLogDaemon::cb_pipeCallback(): FATAL: couldn't read from the log info pipe!\n");
		return TRUE;
	}	

	
	newInfo = (logInfo*)g_malloc0(sizeof(logInfo));
	newInfo->message = localeToSystemCharset(self->NextLogMsg());;
	newInfo->type = self->NextLogType();
	self->ClearLog();
	
	self->loggedInfo = g_list_append(self->loggedInfo, newInfo);
	
	if (self->callBack)
		self->callBack(UE_UNDEFINED, 0, 0, g_list_last(self->loggedInfo), self->callBackData);

	return TRUE;	
}


void IMLogDaemon::clearLog()
{
	GList	*search;
	logInfo	*info;
	
	search = loggedInfo;
	while(search)
	{
		info = (logInfo*)search->data;
		g_free(info->message);
		g_free(info);
		search = search->next;
	}
	
	g_list_free(loggedInfo);
	loggedInfo = NULL;
}

// ====================== IMUserSearchManager ==================================


IMUserSearchManager::IMUserSearchManager(IMUserInfo* _info):
IMEventManager::IMEventManager(_info)
{
	randomChatGroup = 0;
}



IMUserSearchManager::~IMUserSearchManager()
{
	
}



void IMUserSearchManager::searchWhitePages(gchar* firstName, gchar* lastName,
							  			   gchar* alias, gchar* email,
							  			   gushort minAge, gushort maxAge,
							  			   gchar gender, gchar language,
							  			   gchar* city, gchar* state, gushort country,
							  			   gchar* companyName, gchar *companyDep,
							  			   gchar* companyPos,
							  			   gchar* keyWord,
							  			   gboolean onlineOnly)
{
	gchar	*_firstName, *_lastName, *_alias, *_email, *_city, *_state,
			*_companyName, *_companyDep, *_companyPos, *_keyWord;
	
	_firstName = convertToSystemCharset(firstName, info->charset);
	_lastName = convertToSystemCharset(lastName, info->charset);
	_alias = convertToSystemCharset(alias, info->charset);
	_email = convertToSystemCharset(email, info->charset);
	_city = convertToSystemCharset(city, info->charset);
	_state = convertToSystemCharset(state, info->charset);
	_companyName = convertToSystemCharset(companyName, info->charset);
	_companyDep = convertToSystemCharset(companyDep, info->charset);
	_keyWord = convertToSystemCharset(keyWord, info->charset);
	_companyPos = convertToSystemCharset(companyPos, info->charset);
		
	initatedEventID = getLicqDaemon()->icqSearchWhitePages(_firstName, _lastName,
														   _alias, _email, 
														   minAge, maxAge, 
														   gender, language,
														   _city, _state, country, 
														   _companyName, _companyDep, 
														   _companyPos, 
														   _keyWord, 
														   onlineOnly);
														   
	g_free(_firstName);
	g_free(_lastName);
	g_free(_alias);
	g_free(_email);
	g_free(_city);
	g_free(_state);
	g_free(_companyName);
	g_free(_companyDep);
	g_free(_companyPos);
	g_free(_keyWord); 
}



void IMUserSearchManager::searchByID(gchar* id)
{
	initatedEventID = getLicqDaemon()->icqSearchByUin(strtoul(id, (char**)NULL, 10));
}




void IMUserSearchManager::setRandomChatGroup(gulong group)
{
	randomChatGroup = group;
	initatedEventID = getLicqDaemon()->icqSetRandomChatGroup(group);
}


gulong IMUserSearchManager::getRandomChatGroup()
{
	return randomChatGroup;
}



void IMUserSearchManager::searchRandomChatGroup(gulong group)
{
	initatedEventID = getLicqDaemon()->icqRandomChatSearch(group);
}




gboolean IMUserSearchManager::finishEvent(ICQEvent* event)
{
	searchEventInfo	*searchEvent;
		
	if (!event->SearchAck())
	{
		initatedEventID = 0;
		return startCallback(UC_EVENT_FINISHED, event->Result(), NULL);
	}
	
	if (event->SearchAck()->Uin() != 0)
	{	
		searchEvent = (searchEventInfo*)g_malloc0(sizeof(searchEventInfo));
	
		searchEvent->id = (gchar*)event->SearchAck()->Id();
		searchEvent->alias = convertToSystemCharset((gchar*)event->SearchAck()->Alias(), info->charset);
		searchEvent->firstName = convertToSystemCharset((gchar*)event->SearchAck()->FirstName(), info->charset);
		searchEvent->lastName = convertToSystemCharset((gchar*)event->SearchAck()->LastName(), info->charset);
		searchEvent->email = convertToSystemCharset((gchar*)event->SearchAck()->Email(), info->charset);
		searchEvent->status = event->SearchAck()->Status();
		searchEvent->gender = event->SearchAck()->Gender();
		searchEvent->age = event->SearchAck()->Age();
		searchEvent->requireAuthorization = (event->SearchAck()->Auth() != 0);
	
		if (event->Result() == EVENT_RESULT_SUCCESS)
		{
			searchEvent->more = event->SearchAck()->More();		
			searchEvent->lastEntry = (event->Result() == EVENT_RESULT_SUCCESS);
			initatedEventID = 0;
		}
		startCallback(UC_EVENT_FINISHED, event->Result(), searchEvent);

		g_free(searchEvent->alias);
		g_free(searchEvent->firstName);
		g_free(searchEvent->lastName);
		g_free(searchEvent->email);
	
		g_free(searchEvent);	
	} else
	{
		initatedEventID = 0;
		return startCallback(UC_EVENT_FINISHED, event->Result(), NULL);
	}
	
	return TRUE;
}



// -----------------------------------------------------------------------------

IMOwnerDaemon::IMOwnerDaemon(gchar* _ownerID, guint _pluginID) :
IMUserDaemon::IMUserDaemon(_ownerID, _pluginID, NULL)
{	
	users = NULL;
	redirectedQueue = NULL;
	groupMan = IO_getGroupManager();
	lastMask = 0;
}


IMOwnerDaemon::~IMOwnerDaemon()
{
	GList	*search;
	redirectedEvent	*ev;
	
	search = redirectedQueue;
	
	while(search)
	{
		ev = (redirectedEvent*)search->data;
		g_free(ev->newLicqID);
		g_free(ev);
		search = search->next;
	}
	
	g_list_free(redirectedQueue);
}



void IMOwnerDaemon::dispatchPendingEvents()
{
	ICQOwner		*u;
	gint			i;
	CUserEvent		*event;
	gchar			*idstr;
	IMUserDaemon 	*user;
	redirectedEvent *rEv;
	
	u = gUserManager.FetchOwner(info->pluginID, LOCK_R);
	if (!u) return;
			
	
	// check all events that we have received	
	for (i=0;i<u->NewMessages();i++)
	{	
		printf("IMOwnerDaemon::dispatchPendingEvents: new owner event\n");		
		event = u->EventPeek(i);
		
		idstr = findEventReferrer(event);
		
		
		// check whether the event has to be directed to a user
		if (idstr)
		{	

			// add the event to the list of redirected events
			rEv = (redirectedEvent*)g_malloc0(sizeof(redirectedEvent));
			rEv->newLicqID = g_strdup(idstr);
			rEv->eventID = event->Id();
			redirectedQueue = g_list_append(redirectedQueue, rEv);
			
			
			// no user was found? seems to be an event for a new user
			if (!(user = findUserByID(idstr)))
				// because there is no user in the contact list the request
				// for meta information will cause licq to create a new user
				// which can be retrieved from ICQEvent::UnknownUser(). 
				// the userDaemon will be created then (this will delay this
				// event)
				user = addUserTemporally(idstr);
			
			user->addEventToQueue(event, TRUE);
			
		} else
		// this is an event for an owner -> add it to its queue
			addEventToQueue(event, TRUE);
	}
		
		
	
	gUserManager.DropOwner(info->pluginID);		
	
}



IMUserDaemon* IMOwnerDaemon::addUserTemporally(gchar* idString)
{
	IMUserDaemon* user;
	
	user = new IMUserDaemon(new ICQUser(idString, info->pluginID), this);
	users = g_list_append(users, user);				
				
	getLicqDaemon()->icqRequestMetaInfo(strtoul(idString, (char**)NULL, 10));
	
	return user;
}



void IMOwnerDaemon::addUserToList(gchar* idString, gboolean alertUser)
{
	getLicqDaemon()->AddUserToList(idString, info->pluginID, alertUser);
}



void IMOwnerDaemon::removeUserFromList(IMUserDaemon* toRemove)
{	
	// remove the user from the contact list
	getLicqDaemon()->RemoveUserFromList(toRemove->info->licqID, toRemove->info->pluginID);
}



void IMOwnerDaemon::dispatchInformation(CICQSignal* signal)
{
	GList	*search;
	gint	id;
	redirectedEvent	*rEv;
	IMUserDaemon	*rUser;

	if (!signal)
	{
		IMUserDaemon::dispatchInformation(NULL);
		return;
	}
	
	// delete an redirected event if neccessary	
	if (signal->SubSignal() == USER_UPDATE_EVENTS && signal->Argument()<0)
	{
		id = -signal->Argument();
		
		// find out whether this was an event for us or whether it was redirected
		search = basicQueue;
		while(search)
		{
			if (((basicEventInfo*)search->data)->eventID == id)
				break;
			search = search->next;
		}
		
		if (search)
			IMUserDaemon::dispatchInformation(signal);
		else
		{
			// the event was redirected. Find the right user
			search = redirectedQueue;
			while(search)
			{
				rEv = (redirectedEvent*)search->data;
				if (rEv->eventID == id)
					break;
				search = search->next;
			}
			
			if (search)
			{
				rUser = findUserByID(rEv->newLicqID);
				if (rUser)
				{
					rUser->dispatchInformation(signal);
					
					redirectedQueue = g_list_remove(redirectedQueue, rEv);
					g_free(rEv->newLicqID);
					g_free(rEv);
				} else
				printf("IMOwnerDaemon::dispatchInformation(): FATAL: couldn't find a redirected event although there MUST be one!");
			}
		}
		
	} else
		IMUserDaemon::dispatchInformation(signal);
}



void IMOwnerDaemon::loadContacts(guchar mask)
{
	userMask = mask;
	
	// okay, now flood the connected function with groups, users ...
	if (!callBack) return;
		
	printf("IMOwnerDaemon::loadContacts(): Loading Contacts for %s\n", protocol->name);
/*	
	// groups
	groupsList = gUserManager.LockGroupList(LOCK_R);
	defaultGroup = gUserManager.DefaultGroup();
	
	for (i=0;i<groupsList->size();i++)
	{
		groupName = localeToSystemCharset((*groupsList)[i]);
		groups = g_list_append(groups, g_strdup(groupName));
		callBack(UE_UNDEFINED, OC_LOADGROUP, i == defaultGroup, groupName, callBackData);
		g_free(groupName);
	}
	
	// append pseudo group for groupless users
	groups = g_list_append(groups, (gchar*)"Not in Group");
	callBack(UE_UNDEFINED, OC_LOADGROUP, 0, (gchar*)"Not in Group", callBackData);
	*/
	
	lastMask = mask;
	
	// users (macro from licq_user.h)
	FOR_EACH_PROTO_USER_START(info->pluginID, LOCK_R)
	{	
/*		
		if ( !(pUser->IgnoreList() && (mask & CLT_IGNORE)) &&
			 !(pUser->VisibleList() && (mask & CLT_VISIBLE)) &&
			 !(!pUser->IgnoreList() && !pUser->VisibleList() && (mask & CLT_NORMAL))
	 	  )
		FOR_EACH_PROTO_USER_CONTINUE;
	*/	
		addUser(pUser);
	}
	FOR_EACH_PROTO_USER_END 	
}



IMUserDaemon* IMOwnerDaemon::addUser(ICQUser* pUser)
{	
	IMUserDaemon *user;
	gint		 i,
				 count;
	gushort		 group;
	GList		 *currentGroups;
	groupInfo	 *userGroup;	

	if ( !(pUser->IgnoreList() && (lastMask & CLT_IGNORE)) &&
		 !(pUser->VisibleList() && (lastMask & CLT_VISIBLE)) &&
		 !(!pUser->IgnoreList() && !pUser->VisibleList() && (lastMask & CLT_NORMAL))
	  )
		return NULL;
	

	// FIXME: find another place to count the number
	// set group count
	currentGroups = IO_getGroupManager()->groups;
	count = g_list_length(currentGroups);
	group = pUser->GetGroups(GROUPS_USER);
	
	if (!group)
		IO_getGroupManager()->getNoGroup()->numUsers++;
	else
	for  (i=0; i<count;i++)
		if (group & (1<<i))
		{
			userGroup = (groupInfo*)g_list_nth_data(currentGroups, i+1);
			if (userGroup)
				userGroup->numUsers++;
			// the group doesn't exist? There must be a new group ...
			else
			{
				IO_getGroupManager()->checkForNewGroups();
				userGroup = (groupInfo*)g_list_nth_data(IO_getGroupManager()->groups, i+1);
				if (userGroup)
					userGroup->numUsers++;
				else
					printf("IMOwnerDaemon::addUser(): No group found for user %s\n", pUser->GetAlias());
			}
		}

	
		
	// normally a user is created here. but there are situations when the user
	// is already in the list but the buddy doesn't know yet (eg sysmsg: "user added you")
	user = findUserByID(pUser->IdString());
	if (!user)
	{
		user = new IMUserDaemon(pUser->IdString(), pUser->PPID(), this);
		users = g_list_append(users, user);
	} else
	{
		if (user->licqUser)
		{
			// since there is now a user on the official list we don't need
			// our temporally user
			delete user->licqUser;
			user->licqUser = NULL;

			user->owner = this;
			user->editUserInformation(TRUE, USER_UPDATE_ALL); 	
		}
	}
		
	return user;
}




void IMOwnerDaemon::setStatus(guint status)
{	
	if (info->isInvisible)
		status = status | BUDDY_STATUS_INV;
	
	
	if (info->status == BUDDY_STATUS_OFFLINE && status != BUDDY_STATUS_OFFLINE)
		getLicqDaemon()->ProtoLogon(info->pluginID, status);
	else
	if (status == BUDDY_STATUS_OFFLINE)
		getLicqDaemon()->ProtoLogoff(info->pluginID);
	else
		getLicqDaemon()->ProtoSetStatus(info->pluginID, status);
}


void IMOwnerDaemon::setInvisible(gboolean setting)
{
	gulong status;
	
	info->isInvisible = setting;
	
	if (setting)
		status = info->status | BUDDY_STATUS_INV;
	else
		status = info->status;
	
	getLicqDaemon()->ProtoSetStatus(info->pluginID, status);
}
		




IMUserDaemon* IMOwnerDaemon::findUserByID(gchar* id)
{
	GList	*search;
	
	search = users;
	
	while(search)
	{
		if (!strcmp(id, ((IMUserDaemon*)search->data)->info->licqID))
			return (IMUserDaemon*)search->data;
		search = search->next;
	}
	
	return NULL;
}



IMUserDaemon* IMOwnerDaemon::getLastEvent(basicEventInfo** toFill, gboolean favourSystemEvents)
{
	GList	*searchu;
	IMUserDaemon *curUser, *retUser;
	basicEventInfo *lastEvent;
	gboolean sysFound;
	time_t t;
	
	searchu = users;
	
	t = 0;
	retUser = NULL;
	sysFound = FALSE;
	*toFill = NULL;
	
	// find the last event of a user
	while(searchu)
	{
		curUser = (IMUserDaemon*)searchu->data;
		
		if (favourSystemEvents)
			lastEvent = curUser->getEvent(EO_LAST | EO_SYSTEM);
		
		if (!lastEvent)
			lastEvent = curUser->getEvent(EO_LAST | EO_NORMAL);
		else
			sysFound = TRUE;
		
		if (lastEvent)
		{
			if (lastEvent->time > t)
			{
				*toFill = lastEvent;
				retUser = curUser;
			}
		}
		searchu = searchu->next;
	}
	
	// see whether an owner event is more recent
	lastEvent = getEvent(EO_LAST | EO_NORMAL);
	if (lastEvent)
	{
		if (lastEvent->time > t || (favourSystemEvents && !sysFound))
		{
			*toFill = lastEvent;
			retUser = this;
		}
	}
	
	return retUser;
}




gchar* IMOwnerDaemon::findEventReferrer(CUserEvent *event)
{
	switch(event->SubCommand())
	{
		case ICQ_CMDxSUB_AUTHxREQUEST:
			return ((CEventAuthRequest*)event)->IdString();
			break;
			
		case ICQ_CMDxSUB_AUTHxGRANTED:
			return ((CEventAuthGranted*)event)->IdString();
			break;
			
		case ICQ_CMDxSUB_AUTHxREFUSED:
			return ((CEventAuthRefused*)event)->IdString();
			break;
		
		case ICQ_CMDxSUB_ADDEDxTOxLIST:
			return ((CEventAdded*)event)->IdString();
	}
		
	return NULL;		
}




void IMOwnerDaemon::evaluateLicqSignal(CICQSignal* signal)
{
	IMUserDaemon *user;
	ICQUser		 *pUser;
	
	
	switch(signal->Signal())
	{
		// the list should be updated
		case SIGNAL_UPDATExLIST:	
		
			switch(signal->SubSignal())
			{
				case LIST_ALL:
					startCallback(OC_UPDATELIST, 0, this);
					break;
				
				// remove user from the list				
				case LIST_REMOVE:
					user = findUserByID(signal->Id());
					if (user)
					{
						startCallback(OC_REMOVEUSER, 0, user);
						users = g_list_remove(users, user);						
					}
					break;
				
				// add user					
				case LIST_ADD:
					pUser = gUserManager.FetchUser(	signal->Id(), signal->PPID(), LOCK_R);
					user = addUser(pUser);
					if (user)
						startCallback(OC_LOADUSER, 0, user);
					gUserManager.DropUser(pUser);
					break;
			}
			break;
		
			
			
		// a user should be updated
		case SIGNAL_UPDATExUSER:
			user = findUserByID(signal->Id());
		
			if (user)
			{
				// make user do something
				user->dispatchInformation(signal);
				// inform owner that something has to be changed
				startCallback(OC_UPDATEUSER, signal->SubSignal(), user);
			}
			// no user? maybe a signal for ME?
			else
				if (!strcmp(info->licqID, signal->Id()))
				{
					dispatchInformation(signal);
				}
			else
				printf("IMOwnerDaemon::evaluateLicqSignal: no user found for signal (%d)!\n", signal->SubSignal());
			
			
			break;
			
				
			
		// the protocol logged on
		case SIGNAL_LOGON:
			startCallback(OC_SETSTATE, BUDDY_STATUS_ONLINE, this);
			break;
		
		
		
		// the protocol logged off
		case SIGNAL_LOGOFF:
			switch(signal->SubSignal())
			{
				case LOGOFF_REQUESTED:
					startCallback(OC_SETSTATE, BUDDY_STATUS_OFFLINE, this);
					break;
				
				case LOGOFF_RATE:
					startCallback(OC_LOGON_RATE_ERROR, 0, this);
					break;
				
				case LOGOFF_PASSWORD:
					startCallback(OC_LOGON_PASSWORD_ERROR, 0, this);
					break;
			}
			break;
	}
}




void IMOwnerDaemon::evaluateLicqEvent(ICQEvent* event)
{
	IMUserDaemon *user;
	ICQUser		 *tmpUser;
	IMUserUpdateManager	*tmpUpdate;
	
	
	switch(event->SNAC())
	{
		
		// meta event received
		case MAKESNAC(ICQ_SNACxFAM_VARIOUS, ICQ_SNACxMETA):
		
		// don't look into the users list -> the event is only for the owner
		if (event->SubCommand() == ICQ_CMDxMETA_SEARCHxWPxLAST_USER ||
			event->SubCommand() == ICQ_CMDxMETA_SEARCHxWPxFOUND)
		{
			finishEvent(event);
			break;
		}
		
			
		// some event was finished
		case 0:
		case MAKESNAC(ICQ_SNACxFAM_MESSAGE, ICQ_SNACxMSG_SERVERxMESSAGE):
		case MAKESNAC(ICQ_SNACxFAM_MESSAGE, ICQ_SNACxMSG_SERVERxREPLYxMSG):
		case MAKESNAC(ICQ_SNACxFAM_MESSAGE, ICQ_SNACxMSG_SENDxSERVER):
			if ( (user = findUserByID(event->Id())) )	
			{
				// the user is in our list but not in the original licq list
				if (event->UnknownUser() && user->licqUser)
				{
					printf("IMOwnerDaemon::evaluateLicqEvent: unknown user found (%s)!\n", event->UnknownUser()->GetAlias());
					
					// get the information from the temporally user to our user
					tmpUser = user->licqUser;
					user->licqUser = event->UnknownUser();
					tmpUpdate = new IMUserUpdateManager(user->info);
					user->addManager(tmpUpdate);
					user->editUserInformation(TRUE, USER_UPDATE_ALL);
					user->licqUser = tmpUser;
					tmpUpdate->saveUserInfo();
					user->removeManager(tmpUpdate);
				}
				printf("Finish User: %s\n", user->info->alias);
				user->finishEvent(event);
			}
			else
				finishEvent(event);
			break;
	}		
}


// ==================== IMProtoDaemon ==========================================
	
	
IMProtoDaemon::IMProtoDaemon(CProtoPlugin* licqProto)
{
	IMOwnerDaemon	*newOwner;
	
	if (!strcmp(licqProto->Name(), "Licq"))
		name = g_strdup("ICQ / Aim");
	else
		name = g_strdup(licqProto->Name());
	
	version 	= g_strdup(licqProto->Version());
	if (strlen(licqProto->LibName())>0)
		libraryName = g_strdup_printf("protocol_%s.so", g_strdup(licqProto->LibName()));
	else
		libraryName = g_strdup("");
	licqName	= g_strdup(licqProto->Name());
	pluginID 	= licqProto->PPID();
	id 			= licqProto->Id();
	functions 	= licqProto->SendFunctions();
	isLoaded 	= TRUE;
	isProtoPlugin = TRUE;
	
	owners = NULL;
	FOR_EACH_OWNER_START(LOCK_R)
		if (pOwner->PPID() == pluginID)
		{
			printf("registering user\n");
			newOwner = new IMOwnerDaemon(pOwner->IdString(), pOwner->PPID());
			newOwner->protocol = this;
			owners = g_list_append(owners, newOwner);
			ownersList = g_list_append(ownersList, newOwner);			
		}
	FOR_EACH_OWNER_END
	
		printf("ready\n");
	unsetCallback();
}



IMProtoDaemon::IMProtoDaemon(CPlugin* licqProto)
{
	name 		= g_strdup(licqProto->Name());
	licqName	= g_strdup(licqProto->Name());	
	version 	= g_strdup(licqProto->Version());
	libraryName = g_strdup_printf("licq_%s.so", licqProto->LibName());
	pluginID 	= 0;
	id 			= licqProto->Id();
	functions 	= 0;
	isLoaded 	= TRUE; 
	isProtoPlugin = FALSE;
	owners		= NULL;
	unsetCallback();
	
}



IMProtoDaemon::IMProtoDaemon(gchar* _libraryName)
{
	printf("plugin: %s\n", _libraryName);
	
	if (g_str_has_prefix(_libraryName, "licq_"))
	{
		name = (gchar*)g_malloc0(strlen(_libraryName)-5-3+1);
		g_strlcpy(name, _libraryName+5, strlen(_libraryName)-5-3+1); 
		isProtoPlugin = FALSE;
	} else
	if (g_str_has_prefix(_libraryName, "protocol_"))
	{
		name = (gchar*)g_malloc0(strlen(_libraryName)-9-3+1);
		g_strlcpy(name, _libraryName+9, strlen(_libraryName)-9-3+1); 
		isProtoPlugin = TRUE;
	}
	
	licqName	= g_strdup(name);
	libraryName = g_strdup(_libraryName);
	version 	= g_strdup("");
	pluginID 	= 0;
	id 			= 0;
	functions 	= 0;
	isLoaded 	= FALSE;	
	owners		= NULL;
	unsetCallback();
}



IMProtoDaemon::~IMProtoDaemon()
{
	g_free(name);
	g_free(licqName);
	g_free(version);
	g_free(libraryName);
}
	

gboolean IMProtoDaemon::loadPlugin()
{
	// ...
	return FALSE;
}



gboolean IMProtoDaemon::unloadPlugin()
{
	// ...
	return FALSE;
}



void IMProtoDaemon::removeOwner(IMOwnerDaemon* toRemove)
{
	owners = g_list_remove(owners, toRemove);
	ownersList = g_list_remove(owners, toRemove);
	gUserManager.RemoveOwner(toRemove->info->pluginID);
	getLicqDaemon()->SaveConf();
}


gboolean IMProtoDaemon::registerUser(gchar* userName, gchar* password)
{
	if (pluginID == LICQ_PPID)
	{
		getLicqDaemon()->icqRegister(password);
		getLicqDaemon()->SaveConf();
	}
	else
		return FALSE;
	
	return TRUE;
}



gboolean IMProtoDaemon::addExistingUser(gchar* userName, gchar* password)
{
	ICQOwner	*newOwner;
	
	newOwner = gUserManager.FetchOwner(pluginID);
	
	if (!newOwner)
	{
		gUserManager.AddOwner(userName, pluginID);	
		getLicqDaemon()->SaveConf();
		newOwner = gUserManager.FetchOwner(pluginID);
	}
	
	newOwner->SetPassword(password);	
	gUserManager.DropOwner(pluginID);
	
	getLicqDaemon()->ProtoLogon(pluginID, BUDDY_STATUS_ONLINE);	
	
	
	return TRUE;
}


gboolean IMProtoDaemon::verifyRegistration(gchar* verifyStr)
{
	if (pluginID == LICQ_PPID)
		getLicqDaemon()->icqVerify(verifyStr);
	else
		return FALSE;
	
	return TRUE;
}


void IMProtoDaemon::evaluatePluginSignal(CICQSignal* signal)
{
	IMOwnerDaemon	*toSend;
	gchar			*verifyName,
					*ownerName;
	IMOwnerDaemon	*newOwner;
	ICQOwner		*licqOwner;	
	
	
	// FIXME: this must be changed if there can be more than 1 owner per protocol
	if (owners)
		toSend = (IMOwnerDaemon*)owners->data;
	else
		toSend = NULL;
	
	
	if (toSend)
		toSend->evaluateLicqSignal(signal);
			
	// signals that should be directed further
	switch(signal->Signal())
	{
		// the user shall verify his registration by retyping the letters seen on an image
		case SIGNAL_VERIFY_IMAGE:
			verifyName = g_strdup_printf("%s/%s_verify.jpg", BASE_DIR, licqName);
			startCallback(PC_VERIFY, 0, verifyName);
			g_free(verifyName); 
			break;
		
		// a new user was successfully created
		case SIGNAL_NEW_OWNER: 	
			getLicqDaemon()->SaveConf();
		
			newOwner = new IMOwnerDaemon(signal->Id(), signal->PPID());
			newOwner->protocol = this;
			owners = g_list_append(owners, newOwner);
			ownersList = g_list_append(ownersList, newOwner);
		
			startCallback(PC_NEWOWNER, 0, newOwner);
			break;
		
		case SIGNAL_LOGON:
			// the user already exists, just inform
			if (toSend)
				startCallback(OC_SETSTATE, BUDDY_STATUS_ONLINE, this);
			// the user doesn't exist -> create a new one
			else
			{				
				licqOwner = gUserManager.FetchOwner(signal->PPID());
				ownerName = g_strdup(licqOwner->IdString());
				gUserManager.DropOwner(signal->PPID());
				newOwner = new IMOwnerDaemon(ownerName, signal->PPID());
				newOwner->protocol = this;
				owners = g_list_append(owners, newOwner);
				ownersList = g_list_append(ownersList, newOwner);
				g_free(ownerName);
				
				startCallback(PC_NEWOWNER, 0, newOwner);
			}
			break;
		
		// fixme: we already wrote this in IMOwnerDaemon::evaluateLicqSignal
		case SIGNAL_LOGOFF:
			switch(signal->SubSignal())
			{
				case LOGOFF_REQUESTED:
					startCallback(OC_SETSTATE, BUDDY_STATUS_OFFLINE, this);
					break;
				
				case LOGOFF_RATE:
					startCallback(OC_LOGON_RATE_ERROR, 0, this);
					break;
				
				case LOGOFF_PASSWORD:
					startCallback(OC_LOGON_PASSWORD_ERROR, 0, this);
					break;
			}
			break;
	}
}


void IMProtoDaemon::evaluatePluginEvent(ICQEvent* event)
{
	IMOwnerDaemon	*toSend;
	
	// FIXME: this must be changed if there can be more than 1 owner per protocol
	if (owners)
		toSend = (IMOwnerDaemon*)owners->data;
	else
		toSend = NULL;
	
	if (toSend)
		toSend->evaluateLicqEvent(event);
	else
		printf("IMProtoDaemon::evaluatePluginEvent(): Fatal - couldn't find an owner for an %s event (%d)\n", name, event->SubCommand());
}


gboolean IMProtoDaemon::startCallback(gint command, gint lastResult, gpointer data)
{
	if (callBack)
		return callBack(UE_UNDEFINED, command, lastResult, data, callBackData);
	
	return FALSE;
}
	

void IMProtoDaemon::setCallback(IMEventCallback _callBack, gpointer _callBackData)
{
	callBack = _callBack;
	callBackData = _callBackData;
}


void IMProtoDaemon::unsetCallback()
{
	callBack = NULL;
	callBackData = NULL;
}


// ==================== Misc ===================================================

GList *IO_getOwnerList()
{
	return ownersList;
}



GList *IO_getPluginsList()
{
	ProtoPluginsList		licqProtoList;
	ProtoPluginsListIter	licqProtoListIter;
	
	PluginsList				licqPluginList;
	PluginsListIter			licqPluginListIter;
	
	GDir					*pluginDir;
	const gchar				*fileName;
	gchar					*longFileName;
	GList					*pluginsCopy,
							*search;
	
	if (pluginsList)
		return pluginsList;
	
	pluginsList = NULL;
	
	// load protocol plugins
	getLicqDaemon()->ProtoPluginList(licqProtoList);
	
	for (licqProtoListIter = licqProtoList.begin(); licqProtoListIter != licqProtoList.end(); licqProtoListIter++)
		pluginsList = g_list_append(pluginsList, new IMProtoDaemon(*licqProtoListIter));

	
	// load other loaded plugins
	getLicqDaemon()->PluginList(licqPluginList);
	for (licqPluginListIter = licqPluginList.begin(); licqPluginListIter != licqPluginList.end(); licqPluginListIter++)
		pluginsList = g_list_append(pluginsList, new IMProtoDaemon(*licqPluginListIter));
	
	
	// load all unloaded protocols
	pluginsCopy = g_list_copy(pluginsList);
	if ((pluginDir = g_dir_open(LIB_DIR, 0, NULL)))
	{
		while(fileName = g_dir_read_name(pluginDir))
		{
			longFileName = g_strconcat(LIB_DIR, fileName, NULL);
			if (g_file_test(longFileName, G_FILE_TEST_IS_REGULAR) && g_str_has_suffix(fileName, ".so"))
			{
				// don't add loaded plugins
				search = pluginsCopy;
				while(search)
				{
					if (!strcmp(((IMProtoDaemon*)search->data)->libraryName, fileName))
					{
						pluginsCopy = g_list_remove(pluginsCopy, search->data);
						break;
					}
					search = search->next;
				}
			
			if (!search)
				pluginsList = g_list_append(pluginsList, new IMProtoDaemon((gchar*)fileName));
			}

			g_free(longFileName);
		}
		
		g_dir_close(pluginDir);
	}
	g_list_free(pluginsCopy);
	
	
	search = pluginsList;
	while(search)
	{
		printf("IO_getPluginsList(): Found Plugin: %s (%s)\n", ((IMProtoDaemon*)search->data)->name, ((IMProtoDaemon*)search->data)->libraryName);
		search = search->next;
	}
	
	return pluginsList;
}



void IO_installLicqPipe()
{
	GIOChannel	*channel;
		
	channel = g_io_channel_unix_new(getLicqDaemon()->RegisterPlugin(SIGNAL_ALL));
	g_io_add_watch_full(channel, 0, G_IO_IN, (GIOFunc)IO_pipeCallback, NULL, NULL);

}





gboolean IO_pipeCallback(GIOChannel* source, GIOCondition condition)
{
	gchar 		buffer[16];
	gsize 		bytesRead;
	guint 		pluginID;
	
	GList		*search;
	
	CICQSignal	*signal;
	ICQEvent	*event;
	
	IMProtoDaemon *targetPlugin;
	
	if ( (g_io_channel_read_chars(source, buffer, 1, &bytesRead, NULL) == G_IO_STATUS_ERROR) ||
		 (!bytesRead) )
	{
		printf("IO_PipeCallback: FATAL: couldn't read from the licq info pipe!\n");
		return FALSE;
	}	

	
	// check the type of information
	switch(buffer[0])
	{
		// a signal is sent
		case 'S':
			signal = getLicqDaemon()->PopPluginSignal();
			pluginID = signal->PPID();
			break;
			
		// an event is sent
		case 'E':
			event = getLicqDaemon()->PopPluginEvent();
			pluginID = event->PPID();
			break;
			
		// program should be quit (FIXME)
		case 'X':
			break;
	}
	
	
	// find our protocol for this event / signal and let him evaluate the information
	targetPlugin = NULL;
	search = pluginsList;
	while(search)
	{
		if ( ((IMProtoDaemon*)search->data)->pluginID == pluginID)
		{
			targetPlugin = (IMProtoDaemon*)search->data;
			break;
		}
		search = search->next;
	}
		
	if (targetPlugin)
	{
		if (buffer[0]=='S')
		{
			targetPlugin->evaluatePluginSignal(signal);
			delete signal;
		}
		if (buffer[0]=='E')
		{
			targetPlugin->evaluatePluginEvent(event);
			delete event;
		}
	} else
	// also no plugin was found? No idea what to do with that event			
	{
		printf("IO_PipeCallback: FATAL: Licq sent a request which doesn't match any registered protocol or plugin\n");
		if (buffer[0]=='S')
			printf("		-> SignalID: %d\n", signal->Signal());
		if (buffer[0]=='E')
			printf("		-> EventID: %d\n", event->SubCommand());
			
		return FALSE;	
	}
		
	return TRUE;
		
		
	/*				
		
		target = NULL;
		search = owners;
	
		while(search)
		{
			if ( ((IMOwnerDaemon*)search->data)->info->pluginID == pluginID)
			{
				target = (IMOwnerDaemon*)search->data;
				break;
			}
			search = search->next;
		}
	
		// no user was found -> this event / signal is dedicated directly to a plugin
		if (!target)
		{
			search = IO_getPluginsList();
			targetPlugin = NULL;
			while(search)
			{
				if ( ((IMProtoDaemon*)search->data)->pluginID == pluginID)
				{
					targetPlugin = (IMProtoDaemon*)search->data;
					break;
				}
				search = search->next;
			}
			
			if (targetPlugin)
			{
				if (buffer[0]=='S')
				{
					targetPlugin->evaluatePluginSignal(signal);
					delete signal;
				}
				if (buffer[0]=='E')
				{
					targetPlugin->evaluatePluginEvent(event);
					delete event;
				}
				return TRUE;
			} else
			// also no plugin was found? No idea what to do with that event			
			{
				printf("IO_PipeCallback: FATAL: Licq sent a request which doesn't match any registered protocol or plugin\n");
				if (buffer[0]=='S')
					printf("		-> SignalID: %d\n", signal->Signal());
				if (buffer[0]=='E')
					printf("		-> EventID: %s\n", event->SubCommand());
				
				return FALSE;	
			}
		}
	
		
		if (buffer[0]=='S')
		{
			target->evaluateLicqSignal(signal);
			delete signal;
		} else
		if (buffer[0]=='E')
		{
			target->evaluateLicqEvent(event);
			delete event;
		} 
	} else
	// the logoff-signal is special: it is sent to all protocols
	{
		search = owners;
		
		while(search)
		{
			((IMOwnerDaemon*)search->data)->evaluateLicqSignal(signal);
			search = search->next;
		}
	} */
	
	return TRUE;
}
