/***************************************************************************
 *            logwindow.cpp
 *
 *  Fri Jul 22 01:13:10 2005
 *  Copyright  2005  User
 *  Email
 ****************************************************************************/

#include "logwindow.h"

logWindow::logWindow(IMLogDaemon* _logDaemon)
{
	wType = UE_LOGMESSAGES;
	
	logDaemon = _logDaemon;
	
	logDaemon->setCallback((IMEventCallback)this->cb_loggingCallback, this);
}



logWindow::~logWindow()
{

}


		
GtkWidget* logWindow::createWindowContent()
{
	GtkWidget	*frame,
				*vbox;
	GtkTextIter	iter;
	GdkColor	errorColor,
				warningColor,
				packetColor,
				defaultColor;
	GtkTextTag	*tag;
	
	frame = u_createTextView(&textView);
	gtk_text_view_set_editable(GTK_TEXT_VIEW(textView), FALSE);
	
	// create an end marker
	gtk_text_buffer_get_end_iter(gtk_text_view_get_buffer(GTK_TEXT_VIEW(textView)), &iter);
	gtk_text_buffer_create_mark(gtk_text_view_get_buffer(GTK_TEXT_VIEW(textView)), "last_pos", &iter, TRUE);
	
	settings_getSettings()->getProperties("logwindow",
							  			  "errorColor", &errorColor,
							  			  "warningColor", &warningColor,
							  			  "packetColor", &packetColor,
										  "defaultColor", &defaultColor,
							  			  NULL);
	
	tag = gtk_text_tag_new("default");
	g_object_set(tag, "foreground-gdk", &defaultColor, NULL);
	gtk_text_tag_table_add(gtk_text_buffer_get_tag_table(gtk_text_view_get_buffer(GTK_TEXT_VIEW(textView))), tag);

	tag = gtk_text_tag_new("packet");
	g_object_set(tag, "foreground-gdk", &packetColor, NULL);
	gtk_text_tag_table_add(gtk_text_buffer_get_tag_table(gtk_text_view_get_buffer(GTK_TEXT_VIEW(textView))), tag);

	tag = gtk_text_tag_new("warning");
	g_object_set(tag, "foreground-gdk", &warningColor, NULL);
	gtk_text_tag_table_add(gtk_text_buffer_get_tag_table(gtk_text_view_get_buffer(GTK_TEXT_VIEW(textView))), tag);

	tag = gtk_text_tag_new("error");
	g_object_set(tag, "foreground-gdk", &defaultColor, "weight", PANGO_WEIGHT_BOLD, NULL);
	gtk_text_tag_table_add(gtk_text_buffer_get_tag_table(gtk_text_view_get_buffer(GTK_TEXT_VIEW(textView))), tag);


	// create text tags
	
	vbox = gtk_vbox_new(FALSE, 2);
	gtk_container_set_border_width(GTK_CONTAINER(vbox), 5);
	gtk_box_pack_start(GTK_BOX(vbox), frame, TRUE, TRUE, 0);
	gtk_box_pack_start(GTK_BOX(vbox), gtk_hseparator_new(), FALSE, TRUE, 2);
	gtk_box_pack_start(GTK_BOX(vbox), createButtonbar(), FALSE, TRUE, 0);
	
	displayMessages(logDaemon->loggedInfo);
	
	setWindowSize(500, 300);
	
	return vbox;
}


GtkWidget* logWindow::createButtonbar()
{
	GtkWidget	*hbox,
				*saveButton,
				*closeButton,
				*clearButton;
	
	saveButton = gtk_button_new_from_stock(GTK_STOCK_SAVE);
	g_signal_connect_swapped(saveButton, "clicked", G_CALLBACK(this->cb_saveButtonClicked), this);
	clearButton = gtk_button_new_from_stock(GTK_STOCK_CLEAR);
	g_signal_connect_swapped(clearButton, "clicked", G_CALLBACK(this->cb_clearButtonClicked), this);
	closeButton = gtk_button_new_from_stock(GTK_STOCK_CLOSE);
	g_signal_connect_swapped(closeButton, "clicked", G_CALLBACK(this->cb_closeButtonClicked), this);
	
	hbox = gtk_hbutton_box_new();
	gtk_button_box_set_layout(GTK_BUTTON_BOX(hbox), GTK_BUTTONBOX_END);
	gtk_box_set_spacing(GTK_BOX(hbox), 2);
	
	gtk_box_pack_start(GTK_BOX(hbox), saveButton, FALSE, TRUE, 0);
	gtk_box_pack_start(GTK_BOX(hbox), clearButton, FALSE, TRUE, 0);
	gtk_box_pack_start(GTK_BOX(hbox), closeButton, FALSE, TRUE, 0);
	
	return hbox;
}


void logWindow::displayMessages(GList* msg)
{
	logInfo			*info;
	GtkTextIter		iter;
	GtkTextBuffer	*buf;
	GtkTextMark		*mark;
	gchar			*useTag;
	
	buf = gtk_text_view_get_buffer(GTK_TEXT_VIEW(textView));
	
	while(msg)
	{
		info = (logInfo*)msg->data;
		
		useTag = "default";
		switch(info->type)
		{
			case L_ERROR: useTag = "error"; break;
			case L_WARN: useTag = "warning"; break;
			case L_PACKET: useTag = "packet"; break;
		}
		
		gtk_text_buffer_get_end_iter(buf, &iter);
		gtk_text_buffer_insert_with_tags_by_name(buf, &iter, info->message, -1, useTag, NULL);
				
		msg = msg->next;
	}
	
	// scroll down
	gtk_text_buffer_get_end_iter(buf, &iter);
	mark = gtk_text_buffer_get_mark(buf, "last_pos");
	gtk_text_buffer_move_mark(buf, mark, &iter);
	gtk_text_view_scroll_to_mark(GTK_TEXT_VIEW(textView), mark, 0, FALSE, 0, 0);
}



void logWindow::destroyWindowContent()
{
	logDaemon->unsetCallback();
}


	
gboolean logWindow::cb_loggingCallback(userEventType type, gint command, gint lastResult, GList* log, logWindow* self)
{
	self->displayMessages(log);
	
	return TRUE;
}



void logWindow::cb_clearButtonClicked(logWindow* self)
{
	GtkTextIter		startIter,
					endIter;
	GtkTextBuffer	*buf;
	
	buf = gtk_text_view_get_buffer(GTK_TEXT_VIEW(self->textView));
	gtk_text_buffer_get_bounds(buf, &startIter, &endIter);
	gtk_text_buffer_delete(buf, &startIter, &endIter);
	
	self->logDaemon->clearLog();
}



void logWindow::cb_saveButtonClicked(logWindow* self)
{
	GtkWidget 	*dialog;
	gchar	  	*fileName,
				*tmp,
				*toWrite;
	FILE	  	*logFile;
	GtkTextIter	startIter,
				endIter;

	dialog = gtk_file_chooser_dialog_new(tr("Save Log File"),
				      					 GTK_WINDOW(self->window),
				      					 GTK_FILE_CHOOSER_ACTION_SAVE,
				      					 GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
				      					 GTK_STOCK_SAVE, GTK_RESPONSE_ACCEPT,
										 NULL);
	
	fileName = NULL;
	
	do
	{	
		if (gtk_dialog_run(GTK_DIALOG(dialog)) == GTK_RESPONSE_ACCEPT)
		{
			fileName = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(dialog));
		
			// check if file already exists
			if (g_file_test(fileName, G_FILE_TEST_EXISTS))
				switch(u_showYesNoDialog(tr("File exists"), tr("The file already exists. Do you want to overwrite it?"), GTK_STOCK_DIALOG_QUESTION, TRUE))
				{
					case GTK_RESPONSE_CANCEL:
						g_free(fileName);
						fileName = (gchar*)1;
						break;

					case GTK_RESPONSE_NO:
						g_free(fileName);
						fileName = NULL;
						break;
						
				}
		} else
			fileName = (gchar*)1;
		
	} while(!fileName);

		
	if (fileName>(gchar*)1)
	{
		logFile = fopen(fileName, "w");
		
		gtk_text_buffer_get_bounds(gtk_text_view_get_buffer(GTK_TEXT_VIEW(self->textView)), &startIter, &endIter);
		tmp = gtk_text_buffer_get_text(gtk_text_view_get_buffer(GTK_TEXT_VIEW(self->textView)), &startIter, &endIter, FALSE);
		toWrite = systemCharsetToLocale(tmp);
				
		
		if (fprintf(logFile, toWrite)<0)
			u_showAlertMessage(tr("Couldn't write"), tr("The log file could not be written. Please check your permissions on the selected folder."), GTK_STOCK_DIALOG_ERROR);
		else
			u_showAlertMessage(tr("Success!"), tr("The log file was successfully written."), GTK_STOCK_DIALOG_INFO);
		
		fclose(logFile);
		g_free(tmp);
		g_free(toWrite);
		g_free(fileName);
	}
	gtk_widget_destroy (dialog);
}



void logWindow::cb_closeButtonClicked(logWindow* self)
{
	self->destroyWindow();
	delete self;
}
