/***************************************************************************
 *            mainwindow.cpp
 *
 *  Tue Jan 25 22:22:32 2005
 *  Copyright  2005  User
 *  Email
 ****************************************************************************/

#include "mainwindow.h"

mainWindow::mainWindow(mwMenuClickFunction mcf, GCallback quitfunc, gpointer cbdata)
{
	GtkWidget	*frame3,
				*hbox1,
				*nhbox,
				*vbox,
				*viewPort,
				*tmp;
	gulong		width, height, posX, posY;
	settings	*config;

	
	config = settings_getSettings();
	
	
	/* set callback data */
	menuCallBack = mcf;
	menuCallBackData = cbdata;
	
	
	/* set up the state icons */
	tmp = gtk_button_new();
	stateIcons = NULL;
	stateIcons = g_list_append(stateIcons, gtk_widget_render_icon(tmp, 
																  ICON_TYPING1,
																  GTK_ICON_SIZE_SMALL_TOOLBAR,
																  NULL));
	gtk_widget_destroy(tmp);																  
	
	
	/* Create the contact list */
	contacts = new contactList(stateIcons);
	// load the last view mode
	updateContactListViewMode();
	// normal button press
	g_signal_connect(G_OBJECT(contacts->tv), "button_press_event", G_CALLBACK(this->cb_contactListClicked), this);
	// double click or RETURN
	g_signal_connect(G_OBJECT(contacts->tv), "row-activated", G_CALLBACK(this->cb_contactListActivated), this);
	

	// create all the menus
	createMenus();


	// ** create system notify box **
	notifyText = gtk_label_new(NULL);
	gtk_misc_set_alignment(GTK_MISC(notifyText), 0.0f, 0.5f);
	gtk_widget_set_size_request(notifyText, 100, -1);
	
	// add a viewport so the text can be truncated
	viewPort = gtk_viewport_new(NULL, NULL);
	gtk_viewport_set_shadow_type(GTK_VIEWPORT(viewPort), GTK_SHADOW_NONE);
	gtk_container_add(GTK_CONTAINER(viewPort), notifyText);
	gtk_widget_add_events(viewPort, GDK_BUTTON1_MOTION_MASK);
	g_signal_connect(viewPort, "button_press_event", G_CALLBACK(this->cb_notifyBoxClicked), this);

	nhbox = gtk_hbox_new(FALSE, 2);
	gtk_box_pack_start(GTK_BOX(nhbox), viewPort, TRUE, TRUE, 0);
	
	frame3 = gtk_frame_new(NULL);
	gtk_frame_set_shadow_type(GTK_FRAME(frame3), GTK_SHADOW_IN);
	gtk_container_add(GTK_CONTAINER(frame3), nhbox);
	

	

	// create the status bar
	statusButtonText = gtk_label_new("");
	statusButtonIcon = gtk_image_new();
	hbox1 = gtk_hbox_new(FALSE, 2);
	gtk_box_pack_start(GTK_BOX(hbox1), statusButtonIcon, FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(hbox1), statusButtonText, FALSE, TRUE, 0);
	statusButton = gtk_button_new();
	gtk_button_set_relief(GTK_BUTTON(statusButton), GTK_RELIEF_NONE);
	gtk_container_add(GTK_CONTAINER(statusButton), hbox1);
	g_signal_connect(G_OBJECT(statusButton), "clicked", G_CALLBACK(this->cb_menuClicked), statusMenu);

	
	
	// ** create main window **
	vbox = gtk_vbox_new(FALSE, 0);
	gtk_box_pack_start(GTK_BOX(vbox), mainMenu, FALSE, TRUE, 0);
	gtk_box_pack_start(GTK_BOX(vbox), contacts->listWidget, TRUE , TRUE, 3);
	gtk_box_pack_start(GTK_BOX(vbox), frame3, FALSE, TRUE, 0);
	gtk_box_pack_start(GTK_BOX(vbox), statusButton, FALSE, TRUE, 0);
	
	window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
	gtk_window_set_title(GTK_WINDOW(window), title);

	g_signal_connect_swapped(G_OBJECT(window), "delete-event", G_CALLBACK(quitfunc), cbdata);
	g_signal_connect_swapped(G_OBJECT(window), "delete-event", G_CALLBACK(this->cb_destroyWindow), this);
	g_signal_connect_swapped(G_OBJECT(window), "visibility-notify-event", G_CALLBACK(this->cb_visibilityChange), this);
	/* g_signal_connect(G_OBJECT(wnd), "configure-event", G_CALLBACK(MWpositionChange), this); */
	gtk_widget_add_events(window, GDK_VISIBILITY_NOTIFY_MASK | GDK_STRUCTURE_MASK);
	
	
	config->getProperties("mainwindow",
						  "width", &width,
						  "height", &height,
						  "positionX", &posX,
						  "positionY", &posY,
						  NULL);
	
	gtk_window_set_default_size(GTK_WINDOW(window), width, height);
	gtk_window_move(GTK_WINDOW(window), posX, posY);
										
	gtk_container_add(GTK_CONTAINER(window), vbox);
	
	
	
	// show window!
	gtk_widget_show_all(window);
	
	
	// set up tray icon
	tray = new trayIcon();
	if (!tray->installTrayIcon(gtk_image_new()))
	{
		delete tray;
		tray = NULL;
	} else
		g_signal_connect_swapped(tray->eventWidget, "button_press_event", G_CALLBACK(this->cb_trayClicked), this);
	
	
	// connect to the settings callback
	settings_getSettings()->installListener("contactlist", (settingChangedFunc)this->cb_settingsChangedCallback, this);	
	
	updateStatus(BUDDY_STATUS_OFFLINE, FALSE);
	
	trayConnectSource = 0;
	trayBlinkSource = 0;
	eventPixbuf = NULL;
	title = NULL;
}




mainWindow::~mainWindow()
{
	if (tray)
	{
		tray->removeTrayIcon();
		delete tray;
	}
}



void mainWindow::setTitle(gchar* _title)
{
	if (title)
		g_free(title);
	
	title = g_strdup(_title);
	gtk_window_set_title(GTK_WINDOW(window), title);	
}


void mainWindow::setPendingEventNotify(gboolean state)
{
	GString	*newTitle;
	
	if (!title)
		return;
	
	if (state)
		newTitle = g_string_new("*");
	else
		newTitle = g_string_new("");
	
	newTitle = g_string_append(newTitle, title);
	
	gtk_window_set_title(GTK_WINDOW(window), newTitle->str);
	
	g_string_free(newTitle, TRUE);	
}



void mainWindow::updateSystemInfo(gint noe, gchar* evname, gchar* appendix)
{
	gchar	*tmp, *tmp2;
	
	if (noe == 0)
		gtk_label_set_markup(GTK_LABEL(notifyText), "");

	else
	{
		if (noe>1)
			tmp2 = appendix;
		else
			tmp2 = "";
		tmp = g_strdup_printf("%d %s%s", noe, evname, tmp2);
		gtk_label_set_markup(GTK_LABEL(notifyText), tmp);
		g_free(tmp);
	}
}



void mainWindow::updateStatus(guint status, gboolean isInvisible)
{
	gchar		*s, 
				*sName;
	
	
	if (status == BUDDY_STATUS_CONNECT)
		startTrayAnimation();
	else
	if (trayConnectSource)
		stopTrayAnimation();
	
	s = uu_getStatusSymbol(status, isInvisible);
	
	if (isInvisible)
	{
		sName = g_strdup_printf("(%s)", uu_getStatusDescription(status));
		u_editMenuEntry(menu_statusInvisible, ICON_INVISIBLE_ON, NULL);
	}
	else
	{
		sName = g_strdup(uu_getStatusDescription(status));
		u_editMenuEntry(menu_statusInvisible, ICON_INVISIBLE, NULL);
	}

	lastStatusPixbuf = u_getPixbufFromStock(s, GTK_ICON_SIZE_BUTTON);

	// FIXME: free u_getFrom ... icon
	gtk_image_set_from_stock(GTK_IMAGE(statusButtonIcon), s, GTK_ICON_SIZE_SMALL_TOOLBAR);
	gtk_label_set_label(GTK_LABEL(statusButtonText), sName);
	if (tray)
		gtk_image_set_from_pixbuf(GTK_IMAGE(tray->iconWidget), lastStatusPixbuf);
	
	lastStatus = status;
	
	g_free(sName);
}



void mainWindow::setLockUserMenus(gboolean lock)
{
	gtk_widget_set_sensitive(menu_users, !lock);
	gtk_widget_set_sensitive(menu_status, !lock);
	gtk_widget_set_sensitive(menu_editPersonalInfo, !lock);
	gtk_widget_set_sensitive(menu_reloadContacts, !lock);
	gtk_widget_set_sensitive(statusButton, !lock);	
}



void mainWindow::destroyWindow()
{
	cb_destroyWindow(this);
	// gtk_widget_destroy(window);
}



void mainWindow::trayClicked(GdkEventType type, guchar button)
{
/*	
	// a single click
	if (type == GDK_BUTTON_PRESS && button == 3)
			// right button was clicked -> show menu
			gtk_menu_popup(GTK_MENU(createMainMenu()), NULL, NULL, NULL, NULL, 1, gtk_get_current_event_time());
*/
	if (type == GDK_2BUTTON_PRESS && button == 1)
	{
		if (trayBlinkSource!=0)
			menuCallBack(NULL, MENU_NOTIFYBOX, 0, menuCallBackData);	
		else
		{
			// left button was clicked -> show / hide window
			if(GTK_WIDGET_VISIBLE(window))
			{
				// show window when it's iconified or obscured
				if ((gdk_window_get_state(window->window) & GDK_WINDOW_STATE_ICONIFIED) || isObscured)
					gtk_window_present(GTK_WINDOW(window));
				else
					gtk_window_iconify(GTK_WINDOW(window));
			}
			else
				gtk_window_present(GTK_WINDOW(window));					
		}
	}
}


void mainWindow::startTrayAnimation()
{
	if (!tray)
		return;
	
	trayIter = 0;
	trayConnectSource = g_timeout_add(150, (GSourceFunc)this->cb_trayAnimation, this);
}


void mainWindow::stopTrayAnimation()
{
	if (tray && trayConnectSource != 0)
	{
		g_source_remove(trayConnectSource);
		trayConnectSource = 0;
		// set the original pixbuf
		gtk_image_set_from_pixbuf(GTK_IMAGE(tray->iconWidget), lastStatusPixbuf);
		
	}
}



void mainWindow::setTrayBlinkEvent(basicEventInfo *ev)
{
	gchar	*evSymbol;
		
	if ((!ev && !trayBlinkSource) || !tray)
		return;
	
	if (!ev)
	{
		if (trayBlinkSource)
		{
			g_source_remove(trayBlinkSource);
			gtk_image_set_from_pixbuf(GTK_IMAGE(tray->iconWidget), lastStatusPixbuf);
			trayBlinkSource = 0;
		}
	} else
	{
		if (eventPixbuf)
			g_object_unref(eventPixbuf);
		
		evSymbol = uu_getEventSymbol(ev->type);
		eventPixbuf = u_getPixbufFromStock(evSymbol, GTK_ICON_SIZE_BUTTON);
		
		if (!trayBlinkSource)
			trayBlinkSource = g_timeout_add(500, (GSourceFunc)this->cb_trayEventBlinking, this);
		
	}
}



void mainWindow::createMenus()
{
	GtkMenu	*viewMenu,
			*myAccountsMenu,
			*debuggingMenu,
			*usersMenu,
			*systemMenu;
	GtkWidget	*mainItem;
	gboolean	showGroups,
				showEmptyGroups,
				showOfflineUsers;

	
	// create the user menu
	// ====================
	u_menuItem buddyMenuList[]=
	{
		{UMENU_HEADLINE	, MENU_NONE				 , NULL, "Hallo", NULL, FALSE, FALSE, 0, &menu_userMenuLabel},
		{UMENU_NORMAL	, MENU_OPENEVENT		 , ICON_MESSAGE		, tr_NOOP("Open Event")			  , NULL, FALSE, FALSE, 0, &menu_openUserEvent},
		{UMENU_SEPARATOR, MENU_NONE, NULL, NULL, NULL, TRUE, FALSE, 0, NULL},
		{UMENU_NORMAL	, UE_MESSAGE			 , ICON_MESSAGE		, tr_NOOP("Send Message")			  , NULL, TRUE, FALSE, 0, NULL},
		{UMENU_NORMAL	, UE_URL				 , ICON_URL			, tr_NOOP("Send URL")				  , NULL, TRUE, FALSE, 0, NULL},
		{UMENU_NORMAL	, UE_FILETRANSFERREQUEST , ICON_FILE		, tr_NOOP("Send Files")				  , NULL, TRUE, FALSE, 0, NULL},
		{UMENU_NORMAL	, UE_CHATREQUEST		 , ICON_CHAT		, tr_NOOP("Request A Chat")			  , NULL, TRUE, FALSE, 0, NULL},
		{UMENU_NORMAL	, UE_CONTACTSREQUEST	 , NULL				, tr_NOOP("Send Contacts")			  , NULL, FALSE, FALSE, 0, NULL},
		{UMENU_NORMAL	, UE_AUTHORIZATIONREQUEST, NULL				, tr_NOOP("Ask For Authorization")	  , NULL, TRUE, FALSE, 0, NULL},
		{UMENU_NORMAL	, UE_AUTHORIZATION		 , NULL				, tr_NOOP("Send Authorization")		  , NULL, TRUE, FALSE, 0, NULL},
		{UMENU_NORMAL	, UE_ENCRYPTION			 , NULL				, tr_NOOP("Encrypt Connection")		  , NULL, TRUE, FALSE, 0, NULL},
		{UMENU_SEPARATOR, MENU_NONE, NULL, NULL, NULL, TRUE, FALSE, 0, NULL},
		{UMENU_NORMAL	, UE_AUTORESPONSEGET	 , NULL				, tr_NOOP("Check Auto Response")	  , NULL, TRUE, FALSE, 0, NULL},
		{UMENU_NORMAL	, UE_AUTORESPONSESET	 , NULL				, tr_NOOP("Set Special Auto Response"), NULL, TRUE, FALSE, 0, NULL},
		{UMENU_SEPARATOR, MENU_NONE, NULL, NULL, NULL, TRUE, FALSE, 0, NULL},
		{UMENU_NORMAL	, UE_EDITUSERMODES		 , NULL				, tr_NOOP("Edit Groups / Modes")	  , NULL, TRUE, FALSE, 0, NULL},
		{UMENU_NORMAL	, UE_USERINFO			 , ICON_INFO		, tr_NOOP("Users Information")		  , NULL, TRUE, FALSE, 0, NULL},
		{UMENU_NORMAL	, UE_HISTORY			 , ICON_HISTORY		, tr_NOOP("History")				  , NULL, TRUE, FALSE, 0, NULL},
		{UMENU_SEPARATOR, MENU_NONE				 , NULL				, NULL, NULL, TRUE, FALSE, 0, NULL},
		{UMENU_NORMAL	, MENU_REMOVEUSER		 , GTK_STOCK_CANCEL	, tr_NOOP("Remove User")			  , NULL, TRUE, FALSE, 0, NULL}
	};
	
	userMenu = u_createMenuList(buddyMenuList, 20, (GtkSignalFunc)this->cb_menuItemClicked, MENU_USER, this, TRUE);
	
	
	// create the status menu
	// ======================
	u_menuItem statusMenuList[]=
	{
		{UMENU_NORMAL	, BUDDY_STATUS_ONLINE	, ICON_ONLINE		, tr_NOOP("Online")			, NULL, TRUE, FALSE, 0, NULL},
		{UMENU_NORMAL	, BUDDY_STATUS_AWAY		, ICON_AWAY			, tr_NOOP("Away")			, NULL, TRUE, FALSE, 0, NULL},
		{UMENU_NORMAL	, BUDDY_STATUS_NA		, ICON_NA			, tr_NOOP("Not Available")	, NULL, TRUE, FALSE, 0, NULL},
		{UMENU_NORMAL	, BUDDY_STATUS_OCC		, ICON_OCCUPIED		, tr_NOOP("Busy")			, NULL, TRUE, FALSE, 0, NULL},
		{UMENU_NORMAL	, BUDDY_STATUS_DND		, ICON_DND			, tr_NOOP("Do Not Disturb")	, NULL, TRUE, FALSE, 0, NULL},
		{UMENU_NORMAL	, BUDDY_STATUS_FFC		, ICON_FFC			, tr_NOOP("Free For Chat")	, NULL, TRUE, FALSE, 0, NULL},
		{UMENU_NORMAL	, BUDDY_STATUS_OFFLINE	, ICON_OFFLINE		, tr_NOOP("Offline")		, NULL, TRUE, FALSE, 0, NULL},
		{UMENU_SEPARATOR, MENU_NONE, NULL, NULL, NULL, FALSE, FALSE, 0, NULL},
		{UMENU_NORMAL	, BUDDY_STATUS_INV		, ICON_INVISIBLE	, tr_NOOP("Invisible")		, NULL, TRUE, FALSE, 0, &menu_statusInvisible}
	};
	

	statusMenu = u_createMenuList(statusMenuList, 9, (GtkSignalFunc)this->cb_menuItemClicked, MENU_STATUS, this, TRUE);	

	

	// create the main menu
	// ====================	
	
	// Menus of the second level. These menus will be sub menus of the first level menus
	settings_getSettings()->getProperties("contactlist",
										  "showGroups", &showGroups,
										  "showEmptyGroups", &showEmptyGroups,
										  "showOfflineUsers", &showOfflineUsers,
										  NULL);

	u_menuItem viewMenuList[5]=
	{
		{UMENU_CHECK	, MENU_SHOWGROUPS		, NULL, tr_NOOP("Show Groups")			, NULL, TRUE		, showGroups	  , 0, &menu_showGroups},
		{UMENU_CHECK	, MENU_SHOWEMPTYGROUPS	, NULL, tr_NOOP("Show Empty Groups")	, NULL, showGroups	, showEmptyGroups , 0, &menu_showEmptyGroups},
		{UMENU_CHECK	, MENU_SHOWOFFLINEUSERS	, NULL, tr_NOOP("Show Offline Users")	, NULL, TRUE		, showOfflineUsers, 0, &menu_showOfflineUsers},
		{UMENU_SEPARATOR, MENU_NONE, NULL, NULL, NULL, FALSE, FALSE, 0, NULL},
		{UMENU_NORMAL	, MENU_RELOADLIST		, NULL, tr_NOOP("Reload Contact List")	, NULL, FALSE, FALSE, 0, &menu_reloadContacts}
	};		
	
	
	u_menuItem myAccountsMenuList[2]=
	{
		{UMENU_NORMAL, UE_USERINFO		,ICON_INFO, tr_NOOP("Edit Personal Info"), NULL, TRUE, FALSE, 0, &menu_editPersonalInfo},
		{UMENU_NORMAL, UE_MANAGEOWNERS	,NULL, tr_NOOP("Owner Manager")		, NULL, TRUE, FALSE, 0, NULL}
	};
	
	
	u_menuItem debuggingMenuList[2]=
	{
		{UMENU_NORMAL, UE_LOGMESSAGES	, GTK_STOCK_NETWORK, tr_NOOP("Network Window"), NULL, TRUE, FALSE, 0, NULL},
		{UMENU_NORMAL, MENU_NONE	, NULL, tr_NOOP("Debug Level")	, NULL, FALSE, FALSE, 0, NULL}	
	};
	
	
	// create the menus
	viewMenu = u_createMenuList(viewMenuList, 5, (GtkSignalFunc)this->cb_menuItemClicked, MENU_SYSTEM, this, FALSE);
	myAccountsMenu = u_createMenuList(myAccountsMenuList, 2, (GtkSignalFunc)this->cb_menuItemClicked, MENU_SYSTEM, this, TRUE);
	debuggingMenu = u_createMenuList(debuggingMenuList, 2, (GtkSignalFunc)this->cb_menuItemClicked, MENU_SYSTEM, this, TRUE);
	
	
	// Menus of the first level
	u_menuItem usersMenuList[13]=
	{
		{UMENU_NORMAL	, MENU_OPENALL			 , ICON_MESSAGE, tr_NOOP("Open All Messages")	 , NULL, FALSE, FALSE, 0, &menu_openAllEvents},
		{UMENU_SEPARATOR, MENU_NONE				 , NULL, NULL, NULL, FALSE, FALSE, 0, NULL},
		{UMENU_NORMAL	, UE_WHITEPAGES			 , GTK_STOCK_FIND, tr_NOOP("Find Users")			 , NULL, TRUE, FALSE, 0, NULL},
		{UMENU_NORMAL	, UE_ADDUSER			 , GTK_STOCK_ADD, tr_NOOP("Add User")			 , NULL, TRUE, FALSE, 0, NULL},
		{UMENU_NORMAL	, UE_AUTHORIZATIONGRANT	 , NULL, tr_NOOP("Authorize User")		 , NULL, TRUE, FALSE, 0, NULL},
		{UMENU_NORMAL	, UE_AUTHORIZATIONREQUEST, NULL, tr_NOOP("Request Authorization"), NULL, TRUE, FALSE, 0, NULL},
		{UMENU_NORMAL	, MENU_NONE				 , ICON_CHAT, tr_NOOP("Set Random Chat Group"), NULL, FALSE, FALSE, 0, NULL},
		{UMENU_NORMAL	, MENU_NONE				 , ICON_CHAT, tr_NOOP("Start Random Chat")	 , NULL, FALSE, FALSE, 0, NULL},
		{UMENU_SEPARATOR, MENU_NONE, NULL, NULL, NULL, FALSE, FALSE, 0, NULL},
		{UMENU_NORMAL	, UE_EDITGROUPS			 , GTK_STOCK_INDEX, tr_NOOP("Edit Groups ...")		 , NULL, TRUE, FALSE, 0, NULL},
		{UMENU_SEPARATOR, MENU_NONE, NULL, NULL, NULL, FALSE, FALSE, 0, NULL},
		{UMENU_NORMAL	, MENU_UPDATEALL		 , NULL, tr_NOOP("Update All Users")	 , NULL, TRUE, FALSE, 0, NULL},
		{UMENU_NORMAL	, MENU_CHECKINVISIBLEALL , NULL, tr_NOOP("Check Invisible Users"), NULL, FALSE, FALSE, 0, NULL}		
	};
	
	
	u_menuItem systemMenuList[12]=
	{
		{UMENU_NORMAL	, MENU_NONE	, NULL, tr_NOOP("Status")		, statusMenu	, TRUE, FALSE, 0, &menu_status},
		{UMENU_NORMAL	, MENU_NONE	, NULL, tr_NOOP("View")			, viewMenu		, TRUE, FALSE, 0, NULL},
		{UMENU_NORMAL	, MENU_NONE	, NULL, tr_NOOP("My Accounts")	, myAccountsMenu, TRUE, FALSE, 0, NULL},
		{UMENU_NORMAL	, MENU_NONE	, NULL, tr_NOOP("Debugging")	, debuggingMenu	, TRUE, FALSE, 0, NULL},
		{UMENU_SEPARATOR, MENU_NONE, NULL, NULL, NULL, FALSE, FALSE, 0, NULL},
		{UMENU_NORMAL	, MENU_NONE	, GTK_STOCK_PREFERENCES, tr_NOOP("Options...")	, NULL			, FALSE, FALSE, 0, NULL},
		{UMENU_SEPARATOR, MENU_NONE, NULL, NULL, NULL, FALSE, FALSE, 0, NULL},
		{UMENU_NORMAL	, MENU_NONE	, NULL, tr_NOOP("Statistics")	, NULL			, FALSE, FALSE, 0, NULL},
		{UMENU_NORMAL	, MENU_NONE	, NULL, tr_NOOP("Tips")			, NULL			, FALSE, FALSE, 0, NULL},
		{UMENU_NORMAL	, MENU_NONE	, NULL, tr_NOOP("About")		, NULL			, FALSE, FALSE, 0, NULL},
		{UMENU_SEPARATOR, MENU_NONE, NULL, NULL, NULL, FALSE, FALSE, 0, NULL},
		{UMENU_NORMAL	, MENU_QUIT		, GTK_STOCK_QUIT, tr_NOOP("Exit")			, NULL			, TRUE, FALSE, 0, NULL}
	};

	
	usersMenu = u_createMenuList(usersMenuList, 13, (GtkSignalFunc)this->cb_menuItemClicked, MENU_SYSTEM, this, TRUE);
	systemMenu = u_createMenuList(systemMenuList, 12, (GtkSignalFunc)this->cb_menuItemClicked, MENU_SYSTEM, this, TRUE);
	
	
	// create the menu bar
	mainMenu = gtk_menu_bar_new();
	
	mainItem = gtk_menu_item_new_with_mnemonic("_System");
	gtk_menu_item_set_submenu(GTK_MENU_ITEM(mainItem), GTK_WIDGET(systemMenu));
	gtk_menu_shell_append(GTK_MENU_SHELL(mainMenu), mainItem);
	
	menu_users = gtk_menu_item_new_with_mnemonic("_Users");
	gtk_menu_item_set_submenu(GTK_MENU_ITEM(menu_users), GTK_WIDGET(usersMenu));
	gtk_menu_shell_append(GTK_MENU_SHELL(mainMenu), menu_users);
}


/*
GtkWidget* mainWindow::createMainMenu()
{
	GtkMenu	*systemMenu,
			*userMenu,
			*helpMenu,
			*_statusMenu;
	gboolean showGroups,
			 showEmptyGroups,
			 showOfflineUsers; 
	GtkWidget	*menuBar;
	
	if (mainMenu)
		return mainMenu;
	
	settings_getSettings()->getProperties("contactlist",
										  "showGroups", &showGroups,
										  "showEmptyGroups", &showEmptyGroups,
										  "showOfflineUsers", &showOfflineUsers,
										  NULL);	
		
	
	// order of a menu item
	// menu type (see utilities.h), radio group id, title, icon, id, enabled?, selected?, target (or NULL for none), submenu
	

	// menus of the 2nd level
	u_menuItem systemMenuList[5]=
	{
		{UMENU_NORMAL, 0, tr_NOOP("User information")			, ICON_INFO		, UE_USERINFO		, TRUE, TRUE, NULL, NULL},
		{UMENU_NORMAL, 0, tr_NOOP("Security / Password")  		, NULL			, UE_UNDEFINED		, TRUE, TRUE, NULL, NULL},
		{UMENU_SEPARATOR, 0, NULL ,NULL ,0 , FALSE, FALSE, NULL, NULL},
		{UMENU_NORMAL, 0, tr_NOOP("Network window") 	  		, NULL			, UE_UNDEFINED		, TRUE, TRUE, NULL, NULL},
		{UMENU_NORMAL, 0, tr_NOOP("Debugging")		  			, NULL			, UE_UNDEFINED		, TRUE, TRUE, NULL, NULL}
	};

	u_menuItem userMenuList[12]=
	{
		{UMENU_NORMAL, 0, tr_NOOP("Find user") 				, GTK_STOCK_FIND, USER_FIND			, TRUE, TRUE, NULL, NULL},
		{UMENU_NORMAL, 0, tr_NOOP("Add user")	  			, GTK_STOCK_ADD	, USER_ADD			, TRUE, TRUE, NULL, NULL},
		{UMENU_NORMAL, 0, tr_NOOP("Authorize user")			, NULL		  	, USER_AUTHORIZE	, TRUE, TRUE, NULL, NULL},
		{UMENU_NORMAL, 0, tr_NOOP("Request authorization")	, NULL		  	, USER_GETAUTH		, TRUE, TRUE, NULL, NULL},
		{UMENU_SEPARATOR, 0, NULL ,NULL ,0 , FALSE, FALSE, NULL, NULL}	,
		{UMENU_NORMAL, 0, tr_NOOP("Open all messages")		, ICON_MESSAGE	, USER_OPENALL		, TRUE, TRUE, NULL, NULL},
		{UMENU_NORMAL, 0, tr_NOOP("Random chat")  			, ICON_CHAT	  	, RANDOM_USERCHAT	, TRUE, TRUE, NULL, NULL},
		{UMENU_NORMAL, 0, tr_NOOP("Random chat group") 		, ICON_CHAT	  	, RANDOM_CHATGROUP	, TRUE, TRUE, NULL, NULL},
		{UMENU_SEPARATOR, 0, NULL ,NULL ,0 , FALSE, FALSE, NULL, NULL},
		{UMENU_NORMAL, 0, tr_NOOP("Update all users")		, NULL		  	, USER_UPDATEALL	, TRUE, TRUE, NULL, NULL},
		{UMENU_SEPARATOR, 0, NULL ,NULL ,0 , FALSE, FALSE, NULL, NULL},
		{UMENU_NORMAL, 0, tr_NOOP("Register user")   		, NULL		  	, USER_REGISTER		, TRUE, TRUE, NULL, NULL}
	};

	
	
	u_menuItem groupMenuList[4]=
	{
		{UMENU_NORMAL, 0, tr_NOOP("Edit groups")	  		, NULL		, GROUP_EDIT		, TRUE, TRUE, NULL, NULL},
		{UMENU_SEPARATOR, 0, NULL ,NULL ,0 , FALSE, FALSE, NULL, NULL},
		{UMENU_CHECK, 0, tr_NOOP("Show Groups")				, NULL			, GROUP_SHOW, TRUE, showGroups, NULL, NULL},
		{UMENU_CHECK, 0, tr_NOOP("Show empty groups")		, NULL			, GROUP_SHOWEMPTY, showGroups, showEmptyGroups, &showEmptyGroupsMenu, NULL}				
	};

	u_menuItem helpMenuList[3]=
	{
		{UMENU_NORMAL, 0, tr_NOOP("Tips")	  				, NULL		  	, HELP_TIPS			, TRUE, TRUE, NULL, NULL},
		{UMENU_NORMAL, 0, tr_NOOP("About")		  			, NULL		  	, HELP_ABOUT		, TRUE, TRUE, NULL, NULL},
		{UMENU_NORMAL, 0, tr_NOOP("Statistics")	  			, NULL		  	, HELP_STATS		, TRUE, TRUE, NULL, NULL}
	};

	// create menus
	systemMenu  = u_createMenuList(systemMenuList, 5, (GtkSignalFunc)this->cb_menuItemClicked, MENU_SYSTEM, this);
	userMenu    = u_createMenuList(userMenuList,  12, (GtkSignalFunc)this->cb_menuItemClicked, MENU_MAIN, this);
	groupMenu   = u_createMenuList(groupMenuList,  4, (GtkSignalFunc)this->cb_menuItemClicked, MENU_MAIN, this);
	helpMenu    = u_createMenuList(helpMenuList,   3, (GtkSignalFunc)this->cb_menuItemClicked, MENU_MAIN, this);
	_statusMenu = createStatusMenu();
	
	
	// create main menu
	u_menuItem mainMenuList[14]=
	{
		{UMENU_NORMAL, 0, tr_NOOP("System")				, NULL		  	, MAIN_NONE			, TRUE, TRUE, NULL, systemMenu},
		{UMENU_NORMAL, 0, tr_NOOP("Users") 				, NULL		  	, MAIN_NONE			, TRUE, TRUE, NULL, userMenu},
		{UMENU_NORMAL, 0, tr_NOOP("Groups")				, NULL		  	, MAIN_NONE			, TRUE, TRUE, NULL, groupMenu},
		{UMENU_NORMAL, 0, tr_NOOP("Status")				, NULL		  	, MAIN_NONE			, TRUE, TRUE, NULL, _statusMenu},
		{UMENU_SEPARATOR, 0, NULL ,NULL ,0 , FALSE, FALSE, NULL, NULL},
		{UMENU_NORMAL, 0, tr_NOOP("Mini mode") 			, NULL		  	, MAIN_MINIVIEW		, TRUE, TRUE, NULL, NULL},
		{UMENU_NORMAL, 0, tr_NOOP("Reload contact list"), NULL		  	, MAIN_ACTUALIZE	, TRUE, TRUE, NULL, NULL},
		{UMENU_CHECK,  0, tr_NOOP("Show offline users")	, NULL		  	, MAIN_SHOWOFFLINE	, TRUE, showOfflineUsers, NULL, NULL},
		{UMENU_NORMAL, 0, tr_NOOP("Options ...")		, NULL		  	, MAIN_OPTIONS		, TRUE, TRUE, NULL, NULL},
		{UMENU_NORMAL, 0, tr_NOOP("Plugins ...") 		, NULL		  	, MAIN_PLUGINS		, TRUE, TRUE, NULL, NULL},
		{UMENU_SEPARATOR, 0, NULL ,NULL ,0 , FALSE, FALSE, NULL, NULL},
		{UMENU_NORMAL, 0, tr_NOOP("Save settings") 		, NULL		  	, MAIN_SAVESETTINGS	, TRUE, TRUE, NULL, NULL},
		{UMENU_NORMAL, 0, tr_NOOP("Help") 				, NULL		  	, MAIN_NONE			, TRUE, TRUE, NULL, helpMenu},
		{UMENU_NORMAL, 0, tr_NOOP("Exit")	  			, NULL		  	, MAIN_EXIT			, TRUE, TRUE, NULL, NULL}
	};
	
	mainMenu = u_createMenuList(mainMenuList, 14, (GtkSignalFunc)this->cb_menuItemClicked, MENU_MAIN, this);
	
	return mainMenu;
	
} 




GtkMenu* mainWindow::createStatusMenu()
{
	if (statusMenu)
		return statusMenu;
	
	u_menuItem statusMenuList[]=
	{
		{UMENU_NORMAL, BUDDY_STATUS_ONLINE	, ICON_ONLINE		, tr_NOOP("Online")			, NULL, TRUE, FALSE, 0, NULL},
		{UMENU_NORMAL, BUDDY_STATUS_AWAY	, ICON_AWAY			, tr_NOOP("Away")			, NULL, TRUE, FALSE, 0, NULL},
		{UMENU_NORMAL, BUDDY_STATUS_NA		, ICON_NA			, tr_NOOP("Not Available")	, NULL, TRUE, FALSE, 0, NULL},
		{UMENU_NORMAL, BUDDY_STATUS_OCC		, ICON_OCCUPIED		, tr_NOOP("Busy")			, NULL, TRUE, FALSE, 0, NULL},
		{UMENU_NORMAL, BUDDY_STATUS_DND		, ICON_DND			, tr_NOOP("Do Not Disturb")	, NULL, TRUE, FALSE, 0, NULL},
		{UMENU_NORMAL, BUDDY_STATUS_FFC		, ICON_CHAT			, tr_NOOP("Free For Chat")	, NULL, TRUE, FALSE, 0, NULL},
		{UMENU_NORMAL, BUDDY_STATUS_OFFLINE	, ICON_OFFLINE		, tr_NOOP("Offline")		, NULL, TRUE, FALSE, 0, NULL},
		{UMENU_SEPARATOR, UE_UNDEFINED, NULL, NULL, NULL, FALSE, FALSE, 0, NULL},
		{UMENU_NORMAL, BUDDY_STATUS_INV		, ICON_INVISIBLE	, tr_NOOP("Invisible")		, NULL, TRUE, FALSE, 0, NULL}
	};
	

	statusMenu = u_createMenuList(statusMenuList, 9, (GtkSignalFunc)this->cb_menuItemClicked, MENU_STATUS, this);	

	return statusMenu;
}



GtkMenu* mainWindow::createUserMenu()
{
	GtkMenu *sendMenu;
	if (userMenu)
		return userMenu;
	
	u_menuItem userMenuSendList[]=
	{
		{UMENU_NORMAL, 0, tr_NOOP("Message")				, ICON_MESSAGE	, UE_MESSAGE			 , TRUE, TRUE, NULL, NULL},
		{UMENU_NORMAL, 0, tr_NOOP("URL")					, ICON_URL		, UE_URL				 , TRUE, TRUE, NULL, NULL},
		{UMENU_NORMAL, 0, tr_NOOP("Chat request")			, ICON_CHAT		, UE_CHATREQUEST		 , TRUE, TRUE, NULL, NULL},
		{UMENU_NORMAL, 0, tr_NOOP("Files")					, ICON_FILE		, UE_FILETRANSFERREQUEST , TRUE, TRUE, NULL, NULL},
		{UMENU_NORMAL, 0, tr_NOOP("Contacts")				, ICON_INFO		, UE_CONTACTSREQUEST	 , TRUE, TRUE, NULL, NULL},
		{UMENU_SEPARATOR, 0, NULL, NULL, 0, FALSE, FALSE	, NULL, NULL},
		{UMENU_NORMAL, 0, tr_NOOP("Authorization request")	, ICON_INFO		, UE_AUTHORIZATIONREQUEST, TRUE, TRUE, NULL, NULL},
		{UMENU_NORMAL, 0, tr_NOOP("Authorization")			, ICON_INFO		, UE_AUTHORIZATIONGRANT	 , TRUE, TRUE, NULL, NULL},
		{UMENU_SEPARATOR, 0, NULL, NULL, 0, FALSE, FALSE, NULL, NULL},
		{UMENU_NORMAL, 0, tr_NOOP("Encrypt connection")		, ICON_INFO		, UE_ENCRYPTION			 , TRUE, TRUE, NULL, NULL}
	};
	
	sendMenu = u_createMenuList(userMenuSendList, 10, (GtkSignalFunc)this->cb_menuItemClicked, MENU_SEND, this);
	
	
	
	u_menuItem userMenuList[]=
	{
		{UMENU_NORMAL, 0, tr_NOOP("Show messages")			, NULL			, POPUP_SHOWEVENTS, TRUE, TRUE, NULL, NULL},
		{UMENU_NORMAL, 0, tr_NOOP("Send event")				, NULL			, POPUP_SEND	, TRUE, TRUE, NULL, sendMenu},
		{UMENU_SEPARATOR, 0, NULL, NULL, 0, FALSE, FALSE	, NULL, NULL},
		{UMENU_NORMAL, 0, tr_NOOP("Check auto response")	, NULL			, POPUP_ARESPONSEGET, TRUE, TRUE, NULL, NULL},
		{UMENU_NORMAL, 0, tr_NOOP("Special auto response")	, NULL			, POPUP_ARESPONSESET, TRUE, TRUE, NULL, NULL},
		{UMENU_NORMAL, 0, tr_NOOP("User information")		, ICON_INFO		, POPUP_INFO	, TRUE, TRUE, NULL, NULL},
		{UMENU_NORMAL, 0, tr_NOOP("History")				, ICON_HISTORY	, POPUP_HISTORY	, TRUE, TRUE, NULL, NULL},
		{UMENU_NORMAL, 0, tr_NOOP("Remove user")			, ICON_BLANK	, POPUP_DELETE	, TRUE, TRUE, NULL, NULL}
	};
	
	
	// userMenu = u_createMenuList(userMenuList, 8, (GtkSignalFunc)this->cb_menuItemClicked, MENU_POPUP, this);

	return userMenu;
}




GtkMenu* mainWindow::createGroupMenu()
{
	gboolean showGroups,
			 showEmptyGroups;
	
	if (groupMenu)
		return groupMenu;
	
	settings_getSettings()->getProperties("contactlist",
										  "showGroups", &showGroups,
										  "showEmptyGroups", &showEmptyGroups,
										  NULL);
	
	
	u_menuItem groupMenuList[]=
	{
		{UMENU_NORMAL, 0, tr_NOOP("Rename group")		, ICON_MESSAGE	, 0	, TRUE, TRUE, NULL, NULL},
		{UMENU_NORMAL, 0, tr_NOOP("Edit group")			, ICON_URL		, 0	, TRUE, TRUE, NULL, NULL},
		{UMENU_SEPARATOR, 0, NULL, NULL, 0, FALSE, FALSE, NULL, NULL},
		{UMENU_NORMAL, 0, tr_NOOP("Remove group")		, ICON_FILE		, 0	, TRUE, TRUE, NULL, NULL}
	};
	
		
	groupMenu = u_createMenuList(groupMenuList, 4, (GtkSignalFunc)this->cb_menuItemClicked, MENU_POPUP, this);

	return groupMenu;	
	
}
*/




gboolean mainWindow::updateContactListViewMode()
{
	gboolean showOfflineUsers,
			 showGroups,
			 showEmptyGroups;
	
	gint	 newView;
	GList	 *groups,
			 *search;
	contactListGroup	*contactsGroup;
	groupInfo	*group;
	
	// save old group settings
	groups = contacts->getChildrenOfType(CE_GROUP);
	search = groups;
	while(search)
	{
		contactsGroup = (contactListGroup*)search->data;
		group = (groupInfo*)contactsGroup->data;
		group->opened = contactsGroup->getGroupOpened();
		
		search = search->next;
	}
	g_list_free(groups);
	
	settings_getSettings()->getProperties("contactlist", 
							  			  "showOfflineUsers", &showOfflineUsers,				
										  "showGroups", &showGroups,
										  "showEmptyGroups", &showEmptyGroups,
										  NULL);
								
	newView = 0;
	if (showGroups)
		newView = newView | CVIEW_GROUPS;
	if (!showEmptyGroups)
		newView = newView | CVIEW_NOEMPTYGROUPS;
	if (!showOfflineUsers)
		newView = newView | CVIEW_HIDEOFFLINE;
		
	contacts->deleteEntries();
	contacts->showEntries(newView);	

	// restore the opened groups
	groups = contacts->getChildrenOfType(CE_GROUP);
	search = groups;
	while(search)
	{
		contactsGroup = (contactListGroup*)search->data;
		contactsGroup->setGroupOpened(((groupInfo*)contactsGroup->data)->opened);
		search = search->next;
	}
	g_list_free(groups);
	
	return showGroups;
}





// =============================================================================

gboolean mainWindow::cb_contactListClicked(GtkTreeView* cList, GdkEventButton* event, mainWindow* mw)
{
	contactListEntry *focus;
		
	if (event->button != 3)
		return FALSE;
	
	focus = mw->contacts->getEntryAtCursor(event->x, event->y);
	
	if (!focus)
		return FALSE;
	
	/*
	if (focus->type == CE_GROUP)
		gtk_menu_popup(mw->groupMenu, NULL, NULL, NULL, NULL, 1, gtk_get_current_event_time());

	else */
 	if (focus->type == CE_USER)
	{
		gtk_widget_set_sensitive(mw->menu_openUserEvent, ((contactListUser*)focus)->isFlashing());
		gtk_label_set_label(GTK_LABEL(mw->menu_userMenuLabel), focus->name);
		gtk_menu_popup(mw->userMenu, NULL, NULL, NULL, NULL, 1, gtk_get_current_event_time());
	}
	return FALSE;
}




void mainWindow::cb_contactListActivated(GtkTreeView *tv, GtkTreePath *path, GtkTreeViewColumn *column, mainWindow* mw)
{
	contactListEntry *focus;
	
	focus = mw->contacts->getFocusedEntry();
	
	if (!focus)
		return;
	
	// a group was clicked -> collapse or expand it
	if (focus->type == CE_GROUP)
	{
		if (((contactListGroup*)focus)->getGroupOpened())
			((contactListGroup*)focus)->setGroupOpened(FALSE);
		else
			((contactListGroup*)focus)->setGroupOpened(TRUE);
	} else
	{
		mw->menuCallBack(NULL, MENU_DOUBLECLICK, 0, mw->menuCallBackData);		
	}
}



void mainWindow::cb_menuClicked(GtkWidget* w, GtkWidget* toshow)
{
	gtk_menu_popup(GTK_MENU(toshow), NULL, NULL, NULL, NULL, 1, gtk_get_current_event_time());

}


void mainWindow::cb_menuItemClicked(GtkWidget* item, mainWindow* mw)
{
	guint menu, 
		  selection;
	
	menu = (guint)g_object_get_data(G_OBJECT(item), "ICQmenuMENU");
	selection = (guint)g_object_get_data(G_OBJECT(item), "ICQmenuID");
			
	if (selection != MENU_NONE && mw->menuCallBack)
		mw->menuCallBack(item, menu, selection, mw->menuCallBackData);
}



gboolean mainWindow::cb_destroyWindow(mainWindow* mw)
{
	gint width, height, x, y;
	
	gtk_window_get_position(GTK_WINDOW(mw->window), &x, &y);
	gtk_window_get_size(GTK_WINDOW(mw->window), &width, &height);
	
	settings_getSettings()->setProperties(FALSE, "mainwindow",
										  "width", width,
										  "height", height,
										  "positionX", x,
										  "positionY", y,
										  NULL);
	
	gtk_main_quit();
	
	gtk_widget_destroy(mw->window);
	
	return FALSE;
}



gboolean mainWindow::cb_visibilityChange(mainWindow* mw, GdkEventVisibility *event)
{
	mw->isObscured = (event->state == GDK_VISIBILITY_FULLY_OBSCURED);
	
	return FALSE;
}




gboolean mainWindow::cb_trayClicked(mainWindow* mw, GdkEventButton* event)
{
	mw->trayClicked(event->type, event->button);
	
	return FALSE;
}


gboolean mainWindow::cb_trayAnimation(mainWindow* mw)
{
	gchar	*tmp;
		
	mw->trayIter ++;
	mw->trayIter = mw->trayIter % 8;
	
	tmp = g_strdup_printf("icqnd-connect%d", mw->trayIter);
	
	// FIXME: finetune this
	g_object_unref(gtk_image_get_pixbuf(GTK_IMAGE(mw->tray->iconWidget)));
	gtk_image_set_from_pixbuf(GTK_IMAGE(mw->tray->iconWidget), u_getPixbufFromStock(tmp, GTK_ICON_SIZE_BUTTON));
	g_free(tmp);
	
	return TRUE;
}



gboolean mainWindow::cb_trayEventBlinking(mainWindow* mw)
{
	// don't do anything if the connect animation is visible
	if (mw->trayConnectSource)
		return TRUE;
	
	mw->trayShowEventIcon = !mw->trayShowEventIcon;
	
	if (mw->trayShowEventIcon)
		gtk_image_set_from_pixbuf(GTK_IMAGE(mw->tray->iconWidget), mw->eventPixbuf);
	else
		gtk_image_set_from_pixbuf(GTK_IMAGE(mw->tray->iconWidget), mw->lastStatusPixbuf);
	
	return TRUE;
}



gboolean mainWindow::cb_notifyBoxClicked(GtkWidget* box, GdkEventButton* event, mainWindow* mw)
{
	if (event->type == GDK_2BUTTON_PRESS && mw->menuCallBack)
		mw->menuCallBack(NULL, MENU_NOTIFYBOX, 0, mw->menuCallBackData);
	return FALSE;
}




void mainWindow::cb_settingsChangedCallback(gchar* sname, GList *values, mainWindow* self)
{
	GList			 *search;
	settingsProperty *prop;
	gboolean		 changeCView;
	
	
	changeCView = TRUE;
	
	search = values;
	
	while(search)
	{
		prop = (settingsProperty*)search->data;
	
		if (changeCView)
		{			
			if (!strcmp(prop->name, "showGroups") || 
				!strcmp(prop->name, "showEmptyGroups") ||
				!strcmp(prop->name, "showOfflineUsers")
			   )
			{				
				gtk_widget_set_sensitive(self->menu_showEmptyGroups, self->updateContactListViewMode());
				
				// set changeCView to false so the list won't be updated again
				// if there's another value that would change its look
				changeCView = FALSE;		
			}
	
		}
		
		search = search->next;
	}
}
