#include "settings.h"



struct _snames
{
	gchar	*token,
			*filename;
} smileyNames[28] = 
{
	{":-)"		 , "happy.gif"},
	{":-("		 , "sad.gif"},
	{";-)"		 , "wink.gif"},
	{":-P"		 , "tongueout.gif"},
	{":-\\"		 , "pensive.gif"},
	{"]:->"		 , "devil.gif"},
	{":-$"		 , "confused.gif"},
	{":-*"		 , "redlips.gif"},
	{":-["		 , "embarrassed.gif"},
	{"O:-)"		 , "angel.gif"},
	{":-X"		 , "silent.gif"},
	{">:O"		 , "angry.gif"},
	{":-D"		 , "laughing.gif"},
	{":-O"		 , "shocked.gif"},
	{"8-)"		 , "cool.gif"},
	{":-!"		 , "sick.gif"},
	{"*jokingly*", "biggrin.gif"},
	{"*kissed*"	 , "kissed.gif"},
	{"*kissing*" , "kissing.gif"},
	{"*crying*"	 , "crying.gif"},
	{"[:-}"		 , "helmet.gif"},
	{"*tired*"	 , "yawning.gif"},
	{"*stop*"	 , "stop.gif"},
	{"*thumbsup*", "thumb.gif"},
	{"*drink*"	 , "martini.gif"},
	{"*in love*" , "arrowheart.gif"},
	{"@="		 , "bomb.gif"},
	{"@}->--"	 , "rose.gif"}		
};




settings::settings(gchar *filename)
{
	gchar *path;
	
	
	sections = NULL;
	
	licqFile = new CIniFile(INI_FxALLOWxCREATE);	
	
	// open config file
	path = g_strdup_printf("%s/%s", BASE_DIR, filename);	
	licqFile->LoadFile(path);
	g_free(path);
	
	// load icons
	loadIcons();
	
	// load smileys
	loadSmileys();
	
	// get the custom text properties
	loadTextTags();
}



settings::~settings()
{
	licqFile->FlushFile();
	licqFile->CloseFile();
	// FIXME: remove all setting elements
}



void settings::installProperties(gchar* sname, ...)
{
	GList 			*search;
	settingsSection *section;
	settingsProperty *prop;
	va_list 		arguments;
	gchar			*name;
	GType			type;
	GValue			*val;
	
	// find the section
	search = sections;
	while(search)
	{
		section = (settingsSection*)search->data;
		
		if (!strcmp(section->name, sname))
			break;
		
		search = search->next;
	}		
	
	// section was not found -> create a new
	if (!search)
	{
		section = (settingsSection*)g_malloc0(sizeof(settingsSection));
		section->name = g_strdup(sname);
		sections = g_list_append(sections, section);
	}
	
	
	licqFile->SetSection(sname);
	
	va_start(arguments, sname);
	
	
	// check whether the properties exist and install them if they don't
	while((name = va_arg(arguments, gchar*)) && (type = va_arg(arguments, GType)))
	{
		search = section->properties;
		while(search)
		{
			prop = (settingsProperty*)search->data;
			
			// if this property exists don't search for it any longer
			if (G_VALUE_TYPE(prop->val) == type && !strcmp(prop->name, name))
				break;
			
			search = search->next;
		}
		
		// the property doesn't exist yet -> create it
		if (!search)
		{
			val = (GValue*)g_malloc0(sizeof(GValue));
			val = g_value_init(val, type); 
			
			switch(type)
			{
				case G_TYPE_ULONG:
					gulong toSetInt;
					toSetInt = va_arg(arguments, gulong);
					licqFile->ReadNum(name, toSetInt, toSetInt);
					g_value_set_ulong(val, toSetInt);
					break; 
				
				case G_TYPE_STRING:
					gchar* toSetString;
					toSetString = g_strdup(va_arg(arguments, gchar*));
					licqFile->ReadStr(name, toSetString, toSetString);
					g_value_set_string(val, toSetString);
					break;
				
				case G_TYPE_BOOLEAN:
					bool tmp;
				
					tmp = va_arg(arguments, gboolean);
				
					licqFile->ReadBool(name, tmp, tmp);
					g_value_set_boolean(val, tmp != 0);
					break;
				
				
				case ICQND_TYPE_COLOR:
					GdkColor *col1, *col2;
					GdkColor dummy;
					gchar	 *txt;
				
					col1 = (GdkColor*)va_arg(arguments, gpointer);
				
					if (!col1)
						col1 = &dummy;
					
					col2 = (GdkColor*)g_malloc0(sizeof(GdkColor));
				
					txt = g_strdup_printf("%sred", name);
					licqFile->ReadNum(txt, (gulong&)col2->red, col1->red);					
					g_free(txt);

					txt = g_strdup_printf("%sgreen", name);
					licqFile->ReadNum(txt, (gulong&)col2->green, col1->green);					
					g_free(txt);

					txt = g_strdup_printf("%sblue", name);
					licqFile->ReadNum(txt, (gulong&)col2->blue, col1->blue);					
					g_free(txt);				
					
					g_value_set_pointer(val, col2);
					break;
					
					
				default:
					printf("settings::installProperties: Unknown variable type to initialize!\n");
					return;
			}
			
			prop = (settingsProperty*)g_malloc0(sizeof(settingsProperty));
			prop->name = g_strdup(name);
			prop->val = val;
			section->properties = g_list_append(section->properties, prop);
		}	
	}
}
	



void settings::setProperties(gboolean inform, gchar* sname, ...)
{
	GList 			*search,
					*properties2,
					*informProps;
	settingsSection 	*section;
	settingsProperty *prop;
	settingsFunc	*func;
	va_list 		arguments;
	gchar			*name;
	gboolean		propChanged;

	
	// find the section
	search = sections;
	while(search)
	{
		section = (settingsSection*)search->data;
		
		if (!strcmp(section->name, sname))
			break;
		
		search = search->next;
	}	

	if (!search)
		return;		
		
	
	
	va_start(arguments, sname);

	properties2 = g_list_copy(section->properties);
	informProps = NULL;
	
	
	// check whether the properties exist
	while((name = va_arg(arguments, gchar*)))
	{
		search = properties2;
		while(search)
		{
			prop = (settingsProperty*)search->data;
			
			// if this property exists don't search for it any longer
			if (!strcmp(prop->name, name))
			{
				propChanged = FALSE;
				
				switch(G_VALUE_TYPE(prop->val))
				{
					case G_TYPE_STRING:
						gchar	*toSetString,
								*oldString;
					
						toSetString = va_arg(arguments, gchar*);
						oldString = (gchar*)g_value_get_string(prop->val);
					
						if (strcmp(toSetString, oldString)!=0)
						{
							g_value_set_string(prop->val, toSetString);
							propChanged = TRUE;
						}
						break;
					
						
					case G_TYPE_ULONG:
						gulong 	toSetLong,
								oldLong;
					
						toSetLong = va_arg(arguments, gulong);
						oldLong = g_value_get_ulong(prop->val);
					
						if (toSetLong != oldLong)
						{
							g_value_set_ulong(prop->val, toSetLong);
							propChanged = TRUE;
						}
						break;
					
					case G_TYPE_BOOLEAN:
						gboolean toSetBool,
								 oldBool;
					
						toSetBool = va_arg(arguments, gboolean);
						oldBool	  = g_value_get_boolean(prop->val);
					
						if (toSetBool != oldBool)
						{
							g_value_set_boolean(prop->val, toSetBool);
							propChanged = TRUE;
						}
						break;
					
					case G_TYPE_POINTER:
						GdkColor	*col1, 
									*col2;
					
						col1 = va_arg(arguments, GdkColor*);
						col2 = (GdkColor*)g_value_get_pointer(prop->val);
					
					
						if (col1->red != col2->red || 
							col1->green != col2->green || 
							col1->blue != col2->blue)
						{									
							memcpy(col2, col1, sizeof(GdkColor));
							propChanged = TRUE;
						}
						break;
						
						
					default:
						printf("settings::setProperties: Unknown property to change\n");
						break;
				}
				
				
				// FIXME: don't add properties to the changed list that didn't change their value
				if (inform && propChanged)
					informProps = g_list_append(informProps, prop);
				
				
				// remove this element from the copied list, so it won't be browsed again
				properties2 = g_list_remove(properties2, search->data);
				break;
			}
			
			search = search->next;
		}
		
		// nothing was found -> stop searching
		if (!search)
		{
			printf("settings::setProperties: Property %s could not be found\n", name);
			break;
		}	
	}
	
	
	// some properties changed -> call their functions
	if (informProps)
	{
		search = section->funcs;
		while(search)
		{
			func = (settingsFunc*)search->data;
			
			func->func(sname, informProps, func->data);
			search = search->next;
		}
			
		g_list_free(informProps);
	}
	
	g_list_free(properties2);
	
}


	
void settings::getProperties(gchar* sname, ...)
{
	GList 			*search,
					*properties2;
	settingsSection 	*section;
	settingsProperty *prop;
	va_list 		arguments;
	gchar			*name;
	
	// find the section
	search = sections;
	while(search)
	{
		section = (settingsSection*)search->data;
		
		if (!strcmp(section->name, sname))
			break;
		
		search = search->next;
	}	
	
	if (!search)
		return;		
		
	
	va_start(arguments, sname);
	
	properties2 = g_list_copy(section->properties);
	
	// check whether the properties exist
	while((name = va_arg(arguments, gchar*)))
	{
		search = properties2;
		while(search)
		{
			prop = (settingsProperty*)search->data;		
			
			// if this property exists don't search for it any longer
			if (!strcmp(prop->name, name))
			{
				switch(G_VALUE_TYPE(prop->val))
				{
					case G_TYPE_ULONG:
						*va_arg(arguments, gulong*) = g_value_get_ulong(prop->val);
						break;
					
					case G_TYPE_STRING:
						*va_arg(arguments, gchar**) = (gchar*)g_value_get_string(prop->val);
						break;
						
					case G_TYPE_BOOLEAN:
						*va_arg(arguments, gboolean*) = g_value_get_boolean(prop->val);
						break;
						
					case G_TYPE_POINTER:
						memcpy(va_arg(arguments, GdkColor*), g_value_get_pointer(prop->val), sizeof(GdkColor));
						break;
						
					default:
						printf("settings::getProperties: invalid property type!\n");
						return;
				}	

				// remove this element from the copied list, so it won't be browsed again
				properties2 = g_list_remove(properties2, search->data);
				break;
			}
			
			search = search->next;
		}
		
		// nothing was found -> stop searching
		if (!search)
		{
			printf("settings::getProperties: Property %s could not be found\n", name);
			break;
		}	
	}	
	
	g_list_free(properties2);
}
	


void settings::installListener(gchar* sname, settingChangedFunc func, gpointer data)
{
	GList 			*search;
	settingsSection *section;
	settingsFunc	*newFunc;
	
	// find the section
	search = sections;
	while(search)
	{
		section = (settingsSection*)search->data;
		
		if (!strcmp(section->name, sname))
			break;
		
		search = search->next;
	}	
	
	if (!search)
		return;		

	newFunc = (settingsFunc*)g_malloc0(sizeof(settingsFunc));

	newFunc->func = func;
	newFunc->data = data;

	section->funcs = g_list_append(section->funcs, newFunc);	
}
	



void settings::removeListenerByData(gpointer data)
{
	GList 			*search,
					*searchf;
	settingsSection *section;
	settingsFunc	*func;

	search = sections;

	while(search)
	{
		section = (settingsSection*)search->data;
		searchf = section->funcs;
		
		while(searchf)
		{
			func = (settingsFunc*)searchf->data;
			
			if (func->data == data)
			{
				section->funcs = g_list_remove(section->funcs, func);
				g_free(func);
				break;
			}
			
			searchf = searchf->next;
		}
		
		search = search->next;
	}		
}



void settings::createSmileyTree(smileysNode *node, guchar pos)
{
	smileysNode *node2;
	GList		*search,
				*search2;
	guint		index;
	
	
	search = node->similies;
	
	while(search)
	{
		index = GPOINTER_TO_UINT(search->data);
		
		if (strlen(smileyNames[index].token)>pos)
		{					
			search2 = node->children;
	
			while(search2)
			{
				node2 = (smileysNode*)search2->data;
			
				if (node2->character == smileyNames[index].token[pos])
					break;
				
				search2 = search2->next;
			}
			
			if (!search2)
			{
				node2 = (smileysNode*)g_malloc0(sizeof(smileysNode));
				node2->summary = g_string_new("");
				node2->character = smileyNames[index].token[pos];
				node2->smileysIndex = index;
				g_string_append_c(node->summary, smileyNames[index].token[pos]);
				node->children = g_list_append(node->children, node2);
				
				node2->similies = g_list_append(node2->similies, GUINT_TO_POINTER(index));
				
			} else
				node2->similies = g_list_append(node2->similies, GUINT_TO_POINTER(index)); 
		}
		
		search = search->next;
	}
	
	
	search = node->children;
	while(search)
	{
		node2 = (smileysNode*)search->data;
		
		createSmileyTree(node2, pos+1);
		search = search->next;
	}
	
	// add cross references so the user doesn't need to type a "nose" when typing a smiley
	search = node->children;
	while(search)
	{
		node2 = (smileysNode*)search->data;

		if (node2->character == '-')
			g_list_concat(node->children, node2->children); 			
		
		// createSmileyTree(node2, pos+1);
		search = search->next;
	}
}



void settings::loadSmileys()
{
	gint i;
	gchar smileyName[512];
	GtkWidget	*img;


	
	// load the smileys	
	for (i=0;i<28;i++)
	{	
		sprintf(smileyName, "%sicqnd/smileys/%s",SHARE_DIR,smileyNames[i].filename);
		img = gtk_image_new_from_file(smileyName);
		images.smileys[i].icon = gdk_pixbuf_scale_simple(gtk_image_get_pixbuf(GTK_IMAGE(img)), 20, 20, GDK_INTERP_BILINEAR);
		gtk_widget_destroy(img);		
		images.smileys[i].token = g_strdup(smileyNames[i].token);
	}	
	
	// create a tree containing the smiley information
	// to avoid looking for every character of a smiley token on every text
	// character a tree is generated.
	// if we have eg. the smileys ":) :( :P", the first node only contains
	// one element: ":" and three branches: ")", "(", "P". This way programs
	// have only to look for the ":" and only THEN look for other possibilities
	// for this smiley.

	
	smileyTree = (smileysNode*)g_malloc0(sizeof(smileysNode));
	
	smileyTree->summary = g_string_new("");
	for (i=0;i<28;i++)
		smileyTree->similies = g_list_append(smileyTree->similies, GUINT_TO_POINTER(i));
	
	// create the smileys tree
	createSmileyTree(smileyTree, 0);
}



void settings::loadIcons()
{
	struct _icons
	{
		gchar*	stockID;
		gchar*	fileName;
	} icons[] = {	{ICON_AWAY, "away.png"},
					{ICON_BIRTHDAY, "birthday.gif"},
					{ICON_BLANK, "blank.gif"},
					{ICON_CHARSET, "charset.gif"},
					{ICON_CHAT, "chat.png"},
					{ICON_CONNECT, "connect1.png"},
					{ICON_DND, "dnd.png"},
					{ICON_FFC, "ffc.png"},
					{ICON_FILE, "file.png"},
					{ICON_HISTORY, "history.png"},
					{ICON_INFO, "info.png"},
					{ICON_INVISIBLE, "invisible.png"},
					{ICON_INVISIBLE_ON, "invisible_set.png"},
					{ICON_MESSAGE, "message.png"},
					{ICON_NA, "na.png"},
					{ICON_OCCUPIED, "occ.png"},
					{ICON_OFFLINE, "offline.png"},
					{ICON_ONLINE, "online.png"},
					{ICON_SECURE_ON, "secure_on.gif"},
					{ICON_SECUREBDAY, "securebday.gif"},
					{ICON_SMILEYS, "smiley.png"},
					{ICON_URL, "url.png"},
					{ICON_CONNECT0, "connect1.png"},
					{ICON_CONNECT1, "connect2.png"},
					{ICON_CONNECT2, "connect3.png"},
					{ICON_CONNECT3, "connect4.png"},
					{ICON_CONNECT4, "connect5.png"},
					{ICON_CONNECT5, "connect6.png"},
					{ICON_CONNECT6, "connect7.png"},
					{ICON_CONNECT7, "connect8.png"},
					{ICON_DONTSHOWOFFLINE, "dontshowoffline.png"},
					{ICON_GROUPS, "groups.png"},
					{ICON_TYPING1, "typing1.png"},
					{ICON_TYPING2, "typing2.png"},
					{ICON_TYPING3, "typing3.png"},		
					{ICON_SYSMSG, "sysmsg.png"}					
				};
	gint i;
	GtkIconFactory	*iconFactory; // the icon factory containing all icons
	GtkIconSet		*icon; 	      // an icon set (we use one pic for every size)
	GdkPixbuf		*pixbuf;
	gchar			*fileName;
	
	iconFactory = gtk_icon_factory_new();

				
	// load all icons into stock
	for (i=0;i<36;i++)
	{
		fileName = g_strdup_printf("%sicqnd/%s", SHARE_DIR, icons[i].fileName);
		if (g_file_test(fileName, G_FILE_TEST_EXISTS))
		{
			pixbuf=gtk_image_get_pixbuf( GTK_IMAGE(gtk_image_new_from_file(fileName)) );
			icon = gtk_icon_set_new_from_pixbuf(pixbuf);
			gtk_icon_factory_add(iconFactory, icons[i].stockID, icon); 
		}
		else
			printf("image file not found: %s\n", fileName);
		
		g_free(fileName);
	}
	
	gtk_icon_factory_add_default(iconFactory);
	
	
	// load searching-animation
	fileName = g_strdup_printf("%sicqnd/search.gif", SHARE_DIR);
	images.searchingAni = gtk_image_get_animation( GTK_IMAGE(gtk_image_new_from_file(fileName)));
	g_free(fileName);
}



void settings::loadTextTags()
{
	GtkTextTag		*atag;
	GdkColor		colRed,
					colBlue,
					colHRed,
					colHBlue,
					colLink,
					colTyping,
					colTypingStop;
	GtkWidget		*tmpW;
	
	// predefine colors
	GDK_COLOR_SET(&colRed, 0xAD00, 0x2C00, 0x2900);
	GDK_COLOR_SET(&colBlue, 0x1000, 0x5500, 0x9C00);
	GDK_COLOR_SET(&colTyping, 0x0900, 0x5500, 0x2E00);
	GDK_COLOR_SET(&colTypingStop, 0x7D00, 0x7700, 0x1300);
	GDK_COLOR_SET(&colLink, 0x0000, 0x0000, 0xFFFF);
	
	// set the average between the text area's background and our normal colors
	tmpW = gtk_text_view_new();
	gtk_widget_ensure_style(tmpW);
	GDK_COLOR_SET_AV(&colHRed, &colRed, &tmpW->style->base[GTK_STATE_NORMAL]);
	GDK_COLOR_SET_AV(&colHBlue, &colBlue, &tmpW->style->base[GTK_STATE_NORMAL]);
	gtk_widget_destroy(tmpW);
	
	
	// install their properties and set defaults if there are no values
	installProperties("appearance",
					  "colorReceive", ICQND_TYPE_COLOR, &colRed,
					  "colorSend"	, ICQND_TYPE_COLOR, &colBlue,
					  "colorHReceive", ICQND_TYPE_COLOR, &colHRed,
					  "colorHSend", ICQND_TYPE_COLOR, &colHBlue,
					  "colorTyping"	, ICQND_TYPE_COLOR, &colTyping,
					  "colorNoTyping", ICQND_TYPE_COLOR, &colTypingStop,
					  "colorLink"	, ICQND_TYPE_COLOR, &colLink, 
					  NULL);
					  
	// read the colors from the config file
	getProperties("appearance",
				  "colorReceive", &colRed,
				  "colorSend", &colBlue,
				  "colorTyping", &colTyping,
				  "colorNoTyping", &colTypingStop,
				  "colorLink", &colLink,
				  NULL);
	
		
	textTags = gtk_text_tag_table_new();


	// incoming messages
	atag = gtk_text_tag_new("incoming");
	g_object_set(atag, "foreground-gdk", &colRed,
					   NULL);
	gtk_text_tag_table_add(textTags, atag);
	
	//outgoing messages
	atag = gtk_text_tag_new("outgoing");
	g_object_set(atag, "foreground-gdk", &colBlue, 
						NULL);
	gtk_text_tag_table_add(textTags, atag);	
	
	
	// history receive
	atag = gtk_text_tag_new("history_incoming");
	g_object_set(atag, "foreground-gdk", &colHRed, NULL);
	gtk_text_tag_table_add(textTags, atag);
      
	  
	// history send
	atag = gtk_text_tag_new("history_outgoing");
	g_object_set(atag, "foreground-gdk", &colHBlue, NULL);
	gtk_text_tag_table_add(textTags, atag);
	
	
	//notification
	atag = gtk_text_tag_new("notification");
	g_object_set(atag, "weight", PANGO_WEIGHT_BOLD, NULL);
	gtk_text_tag_table_add(textTags, atag);
	
	// url
	atag = gtk_text_tag_new("url");
	g_object_set(atag, "foreground-gdk", &colLink,
					   "underline", PANGO_UNDERLINE_SINGLE,
					   NULL);
	gtk_text_tag_table_add(textTags, atag);
	
	// headlines
	atag = gtk_text_tag_new("HL");
	g_object_set(atag, 	"weight", PANGO_WEIGHT_BOLD, NULL);
	gtk_text_tag_table_add(textTags, atag);		
	
	// new line space
	atag = gtk_text_tag_new("newline");
	g_object_set(atag, 	"size", 3*PANGO_SCALE, NULL);
	gtk_text_tag_table_add(textTags, atag);	

	// margin
	atag = gtk_text_tag_new("margin");
	g_object_set(atag, 	"left_margin", 20, NULL);
	gtk_text_tag_table_add(textTags, atag);		

	// timestamp
	atag = gtk_text_tag_new("timestamp");
	g_object_set(atag,  "size", 8*PANGO_SCALE,
						"weight", PANGO_WEIGHT_LIGHT,
						NULL);
	gtk_text_tag_table_add(textTags, atag);	
}



gboolean settings::writeSettings()
{
	GList				*searchs,
						*searchp;
	settingsSection		*section;
	settingsProperty 	*property;
	
	searchs = sections;
	
	while(searchs)
	{
		section = (settingsSection*)searchs->data;
		if (!licqFile->SetSection(section->name))
			licqFile->CreateSection(section->name);

		searchp = section->properties;
		while(searchp)
		{
			property = (settingsProperty*)searchp->data;
			
			switch(G_VALUE_TYPE(property->val))
			{
				case G_TYPE_ULONG:
					licqFile->WriteNum(property->name, g_value_get_ulong(property->val));
					break;
				
				case G_TYPE_STRING:
					licqFile->WriteStr(property->name, g_value_get_string(property->val));
					break;
				
				case G_TYPE_BOOLEAN:
					licqFile->WriteBool(property->name, g_value_get_boolean(property->val));
					break;
				
				case G_TYPE_POINTER:
					GdkColor *col;
					gchar	 *txt;
					col = (GdkColor*)g_value_get_pointer(property->val);
					if (!col)
						break;
														
					txt = g_strdup_printf("%sred", property->name);
					licqFile->WriteNum(txt, (gulong)col->red);
					g_free(txt);

					txt = g_strdup_printf("%sgreen", property->name);
					licqFile->WriteNum(txt, (gulong)col->green);
					g_free(txt);

					txt = g_strdup_printf("%sblue", property->name);
					licqFile->WriteNum(txt, col->blue);
					g_free(txt);
					break;					
			}
			
			searchp = searchp->next;
		}			
		
		
		searchs = searchs->next;
	}
	
	return TRUE;
}




settings* settings_d;

settings* settings_getSettings()
{
	return settings_d;
}


void	  settings_setSettings(settings* s)
{
	settings_d = s;	
}
