/***************************************************************************
 *            systemcharset.cpp
 *
 *  Sat Jul 24 01:16:54 2004
 *  Copyright  2004  User
 *  Email
 ****************************************************************************/

#include "systemcharset.h"
#include "licq_chat.h"	// for ENCODING tokens

static charsetType charsets[] =
{
	{ "Unicode"					, "UTF-8"			, ENCODING_DEFAULT},
	{ "Unicode-16"				, "ISO-10646-UCS-2"	, ENCODING_DEFAULT},
	{ "Arabisch"				, "ISO 8859-6"		, ENCODING_DEFAULT},
	{ "Arabisch"				, "CP 1256"			, ENCODING_ARABIC},
	{ "Baltisch"				, "ISO 8859-13"		, ENCODING_DEFAULT},
	{ "Baltisch"				, "CP 1257"			, ENCODING_BALTIC},
	{ "Zentral-Europäisch"		, "ISO 8859-2"		, ENCODING_DEFAULT},
	{ "Zentral-Europäisch"		, "CP 1250"			, ENCODING_EASTEUROPE},
	{ "Chinesisch"				, "GBK"				, ENCODING_GB2312},
	{ "Chinesisch Traditionell"	, "Big5"			, ENCODING_CHINESEBIG5},
	{ "Kyrillisch"				, "ISO 8859-5"		, ENCODING_DEFAULT},
	{ "Kyrillisch"				, "KOI8-R"			, ENCODING_DEFAULT},
	{ "Kyrillisch"				, "CP 1251"			, ENCODING_RUSSIAN},
	{ "Esperanto"				, "ISO 8859-3"		, ENCODING_DEFAULT},
	{ "Griechisch"				, "ISO 8859-7"		, ENCODING_DEFAULT},
	{ "Griechisch"				, "CP 1253"			, ENCODING_GREEK},
	{ "Hebräisch"				, "ISO 8859-8-I"	, ENCODING_DEFAULT},
	{ "Hebräisch"				, "CP 1255"			, ENCODING_HEBREW},
	{ "Japanisch"				, "Shift-JIS"		, ENCODING_SHIFTJIS},
	{ "Japanisch"				, "JIS7"			, ENCODING_DEFAULT},
	{ "Japanisch"				, "eucJP"			, ENCODING_DEFAULT},
	{ "Koreanisch"				, "eucKR"			, ENCODING_DEFAULT},
	{ "West-Europäisch"			, "ISO 8859-1"		, ENCODING_DEFAULT},
	{ "West-Europäisch"			, "ISO 8859-15"		, ENCODING_DEFAULT},
	{ "West-Europäisch"			, "CP 1252"			, ENCODING_ANSI},
	{ "Tamil"					, "TSCII"			, ENCODING_DEFAULT},
	{ "Thailändisch"			, "TIS-620"			, ENCODING_THAI},
	{ "Türkisch"				, "ISO 8859-9"		, ENCODING_DEFAULT},
	{ "Türkisch"				, "CP 1254"			, ENCODING_TURKISH},
	{ "Ukrainisch"				, "KOI8-U"			, ENCODING_DEFAULT},
	{ NULL, NULL, 0}
};

#define CHARSETS_SIZE sizeof(charsets) / sizeof(charsetType)





gchar* convertToCharset(gchar* toconvert, gchar* ics, gchar* ocs)
{
	gsize bIn, bOut;
	char *res, *inputcs, *outputcs;
	
	
	// check whether there is something to convert
	if (!toconvert)
		return NULL; 
	if (!*toconvert)
		return g_strdup(toconvert);
	
	
	// check whether an output charset was set or set it to ISO8859-1
	if (!ocs || !*ocs)
		outputcs = g_strdup_printf("ISO8859-1");
	else
		outputcs = g_strdup(ocs);
	
	
	// check whether there is a charset given -> convert from locale if not
	if (!ics || !*ics)
		inputcs = g_strdup_printf("ISO8859-1");
	else
		inputcs = g_strdup(ics);
	
	/*
	// check whether an input charset was set or set it to ISO8859-1
	if (!ics || !*ics)
		inputcs = g_strdup_printf("ISO8859-1");
	else
		inputcs = g_strdup(ics);
	*/

	// check if charsets already equal
	// if (!strcmp(inputcs, outputcs))
	//	return g_strdup(toconvert);
	
	
	//printf("Converting %s --- (%s -> %s)\n",toconvert, inputcs, outputcs);
	
	
	// FIXME: maybe use g_convert_with_fallback here?
	
	res = g_convert_with_fallback(toconvert, strlen(toconvert), outputcs, inputcs, "?", &bIn, &bOut, NULL);

	// conversion did not work, fall back to ISO8859-1
	if (!res)
	{
		res = g_convert_with_fallback(toconvert, strlen(toconvert), outputcs, "ISO8859-1", "?", &bIn, &bOut, NULL);
	}		

	g_free(inputcs);
	g_free(outputcs);
	
	if (res)
		return res;
	else
	// conversion still doesn't work - nothing more we can do here ... 
		return g_strdup_printf("Charset Conversion Error");
}


gchar* convertToSystemCharset(gchar* toconvert, gchar* charset)
{
	const char* syscs;
	gchar		*toReturn;
		
	g_get_charset(&syscs);
	
	toReturn = convertToCharset(toconvert, charset, (gchar*)syscs);
	
	if (!g_utf8_validate(toReturn, -1, NULL))
	{
		g_free(toReturn);
		toReturn = g_convert_with_fallback(toconvert, strlen(toconvert), (gchar*)syscs, "ISO8859-1", "?", NULL, NULL, NULL);

		
	}
	
	return convertToCharset(toconvert, charset, (gchar*)syscs);
}



gchar* convertFromSystemCharset(gchar* toconvert, gchar* charset)
{
	const char* syscs;
	
	g_get_charset(&syscs);
	return convertToCharset(toconvert, (gchar*)syscs, charset);	
}



gchar* localeToSystemCharset(gchar* toconvert)
{
	gchar* ret;
	gsize bIn, bOut;
	// FIXME: only valid for UTF-8
	
	ret = g_locale_to_utf8(toconvert, strlen(toconvert), &bIn, &bOut, NULL);

	// conversion failure -> fall back to ISO8859-1
	if (!ret)
		ret = g_convert_with_fallback(toconvert, strlen(toconvert), "UTF-8", "ISO8859-1", "?", &bIn, &bOut, NULL);		
	
	return ret;
}



gchar* systemCharsetToLocale(gchar* toconvert)
{
	gchar* ret;
	gsize bIn, bOut;
	// FIXME: only valid for UTF-8
	
	ret = g_locale_from_utf8(toconvert, strlen(toconvert), &bIn, &bOut, NULL);

	// conversion failure -> fall back to ISO8859-1
	if (!ret)
	{
		printf("***WARNING: conversion of string (\"%s\") failed!\n", toconvert);
		ret = g_convert_with_fallback(toconvert, strlen(toconvert), "ISO8859-1", "UTF-8", "?", &bIn, &bOut, NULL);		
	}
	
	return ret;
}



charsetType* charset_getCharsetList()
{
	return charsets;	
}



charsetType* charset_findCharsetById(gchar* id)
{
	guint i;
	
	if (!id || !*id)
		return &charsets[0];
	
	for (i=0;i<CHARSETS_SIZE;i++)
	{
		if (!strcmp(charsets[i].id, id))
			return &charsets[i];
	}
	
	return NULL;
}



charsetType* charset_findCharsetByNumber(guchar number)
{
	guint i;
	
	for (i=0;i<CHARSETS_SIZE;i++)
	{
		if (charsets[i].number == number)
			return &charsets[i];
	}
	
	return NULL;
}
