/***************************************************************************
 *            userutilities.cpp
 *
 *  Mon Feb 14 02:24:32 2005
 *  Copyright  2005  User
 *  Email
 ****************************************************************************/

#include "userutilities.h"


gchar* uu_getStatusSymbol(guint status, gboolean isInvisible)
{
	if (isInvisible && status != BUDDY_STATUS_OFFLINE)
		return ICON_INVISIBLE;
	
	switch(status)
	{
		case BUDDY_STATUS_CONNECT:
			return ICON_OFFLINE;
		
		case BUDDY_STATUS_OFFLINE:
			 return ICON_OFFLINE;
		
		case BUDDY_STATUS_DND:
			 return ICON_DND;
		
		case BUDDY_STATUS_OCC:
			 return ICON_OCCUPIED;
		
		case BUDDY_STATUS_NA:
			 return ICON_NA;
		
		case BUDDY_STATUS_AWAY:
			 return ICON_AWAY;
		
		case BUDDY_STATUS_FFC:
			 return ICON_CHAT;
		
		case BUDDY_STATUS_INV:
			 return ICON_INVISIBLE;
		
		default:
			 return ICON_ONLINE;		
	}		
	
}


gchar* uu_getEventSymbol(gint last)
{
	switch(last)
	{
		case -1:
			return NULL;
		
		case USER_EVENT_MESSAGE:
			return ICON_MESSAGE;
		
		case USER_EVENT_CHAT:
			return ICON_CHAT;
		
		case USER_EVENT_FILE:
			return ICON_FILE;
		
		case USER_EVENT_URL:
			return ICON_URL;
		
		default:
			return ICON_SYSMSG;
	}	
}



gchar* uu_getStatusDescription(guint status)
{
	switch(status)
	{
		case BUDDY_STATUS_CONNECT:
			return tr("Connecting...");
		
		case BUDDY_STATUS_OFFLINE:
			 return tr("Offline");
		
		case BUDDY_STATUS_DND:
			 return tr("Do not disturb");
		
		case BUDDY_STATUS_OCC:
			 return tr("Busy");
		
		case BUDDY_STATUS_NA:
			 return tr("Not available");
		
		case BUDDY_STATUS_AWAY:
			 return tr("Away");
		
		case BUDDY_STATUS_FFC:
			 return tr("Free for chat");
		
		case BUDDY_STATUS_INV:
			 return tr("Invisible");
		
		default:
			 return tr("Online");	
	}
}





gchar* uu_getFormattedBuddyName(IMUserInfo* info, gboolean addRealName)
{
	GString *toReturn,
			*realName;
		
	if (info->formattedAlias && strcmp(info->formattedAlias, "None"))
		toReturn = g_string_new(info->formattedAlias);
	else
		toReturn = g_string_new(info->licqID);
	
	if (addRealName)
	{
		realName = g_string_new(info->firstname);
		g_string_append_printf(realName, " %s", info->lastname);
		if (strlen(realName->str)>1)
			g_string_append_printf(toReturn, " (%s)", realName->str);
		g_string_free(realName, TRUE);
	}
	
	
	return g_string_free(toReturn, FALSE);
}



void uu_showSendErrorMessage(gchar* title, gchar* beginning, guint result, guint status)
{
	GString	*toShow;
	
	toShow = g_string_new(beginning);
	
	if (status == BUDDY_STATUS_OFFLINE)
		toShow = g_string_append(toShow, tr(", because you are not connected to the network. Please change your status to \"Online\" by clicking the status button of the main window or by selecting it in the \"System\" -> \"Status\" menu."));
	else
	switch(result)
	{
		case EVENT_RESULT_FAILED:
			toShow = g_string_append(toShow, tr(", because there was an error while communicating with the remote. Please check your connection."));
			break;
		case EVENT_RESULT_ERROR:
			toShow = g_string_append(toShow, tr(", because there was an unknown error. Please check your connection, the user settings and the remote user that you sent this event."));
			break;
		case EVENT_RESULT_TIMEOUT:
			toShow = g_string_append(toShow, tr(", because the remote user did not respond and caused a timeout. Please try to send this event via the server."));
			break;
	}
	
	u_showAlertMessage(title, toShow->str, GTK_STOCK_DIALOG_ERROR);
	
	g_string_free(toShow, TRUE);
}
